/**
 * Copyright (C) 2010 The Roslin Institute <contact andy.law@roslin.ed.ac.uk>
 *
 * This file is part of the Ensembl Java API demonstration project developed by the
 * Bioinformatics Group at The Roslin Institute, The Royal (Dick) School of
 * Veterinary Studies, University of Edinburgh.
 *
 * This is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License (version 3) as published by
 * the Free Software Foundation.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * in this software distribution. If not, see <http://www.gnu.org/licenses/gpl-3.0.html/>.
 */

package uk.ac.roslin.ensembl.config;

import java.util.HashMap;
import uk.ac.roslin.ensembl.model.core.CoordSystemType;
import uk.ac.roslin.ensembl.model.core.CoreObjectType;

/**
 *
 * @author paterson
 */
public class EnsemblCoordSystemType extends EnsemblType implements CoreObjectType, CoordSystemType {


    public static EnsemblCoordSystemType contig;
    public static EnsemblCoordSystemType supercontig;
    public static EnsemblCoordSystemType clone;
    public static EnsemblCoordSystemType chromosome;
    public static EnsemblCoordSystemType scaffold;
    public static EnsemblCoordSystemType plasmid;
    public static EnsemblCoordSystemType plastid;
    public static EnsemblCoordSystemType unknown;
    public static EnsemblCoordSystemType lrg;

    public static String sequenceLevel = "sequence_level";
    public static String defaultVersion = "default_version";

    private static HashMap<String,EnsemblCoordSystemType> types = EnsemblCoordSystemType.init();

    private EnsemblCoordSystemType(String type) {
        label = type;
    }

    private static HashMap<String,EnsemblCoordSystemType> init() {
        HashMap<String,EnsemblCoordSystemType> out = new HashMap<String,EnsemblCoordSystemType>();
        contig = new EnsemblCoordSystemType("contig");
        out.put(contig.toString(),contig);
        supercontig = new EnsemblCoordSystemType("supercontig");
        out.put(supercontig.toString(),supercontig);
        clone = new EnsemblCoordSystemType("clonedes");
        out.put(clone.toString(),clone);
        chromosome = new EnsemblCoordSystemType("chromosome");
        out.put(chromosome.toString(),chromosome);
        plasmid = new EnsemblCoordSystemType("plasmid");
        out.put( plasmid.toString(), plasmid);
        plastid = new EnsemblCoordSystemType("plastid");
        out.put(plastid.toString(), plastid);
        scaffold = new EnsemblCoordSystemType("scaffold");
        out.put(scaffold.toString(), scaffold);
        unknown = new EnsemblCoordSystemType("");
        out.put(unknown.toString(),unknown);
        lrg = new EnsemblCoordSystemType("lrg");
        out.put(lrg.toString(),lrg);

        return out;
    }

    public static EnsemblCoordSystemType getType(String name) {

        EnsemblCoordSystemType out = types.get(name);

        if (out == null) {
            out = EnsemblCoordSystemType.unknown;
        }

        return out;
        
    }
    

        @Override
        public String toString() {
            return label;
        }

    

}
