/**
 * Copyright (C) 2010 The Roslin Institute <contact andy.law@roslin.ed.ac.uk>
 *
 * This file is part of the Ensembl Java API demonstration project developed by the
 * Bioinformatics Group at The Roslin Institute, The Royal (Dick) School of
 * Veterinary Studies, University of Edinburgh.
 *
 * This is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License (version 3) as published by
 * the Free Software Foundation.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * in this software distribution. If not, see <http://www.gnu.org/licenses/gpl-3.0.html/>.
 */

package uk.ac.roslin.ensembl.config;

import java.util.Collection;
import java.util.HashMap;

/**
 *
 * @author tpaterso
 */
public class FeatureType extends EnsemblType{

    //Commonly used annotated features
    //typically annotated at 'toplevel'

    public static FeatureType exon;
    public static FeatureType gene;
    public static FeatureType repeat_feature;
    public static FeatureType simple_feature;
    public static FeatureType prediction_exon;
    public static FeatureType prediction_transcript;
    public static FeatureType transcript;
    public static FeatureType dna_align_feature;
    public static FeatureType protein_align_feature;

    //tables exist in the schema
    //misc_feature, marker_feature, density_feature, qtl_feature, protein_feature
    //and in the Perl API there are various other extensions of Feature

    //Features on DNA
    public static FeatureType assembly_exception;
    public static FeatureType density_feature;
    //nb the Perl API treats these similar to marker Features -
    //(i.e there is a ditag object analagous to a marker)
    public static FeatureType ditag_feature;
    public static FeatureType intron;
    public static FeatureType karyotype_band;
    //nb the PERL API has Markers and MarkerFeatures (the mapping of a Marker)
    public static FeatureType marker_feature;
    public static FeatureType misc_feature;
    public static FeatureType qtl_feature;
    public static FeatureType splicing_event;
    
    //Features on Peptide, i.e. cordinates are on the peptide
    public static FeatureType protein_feature;
    
    //Pairwise alignment features
    public static FeatureType dna_peptide_align_feature;
    public static FeatureType peptide_dna_align_feature;


    private FeatureType(String value) {
        this.label = value;

    }


    private static HashMap<String, FeatureType> typeListHash = FeatureType.initialize();

    public static Collection<FeatureType> getAllTypes() {
        return typeListHash.values();
    }

    private static HashMap<String, FeatureType> initialize() {

        HashMap<String, FeatureType> out = new HashMap<String, FeatureType>();


        assembly_exception = new FeatureType("assembly_exception");
        out.put(assembly_exception.toString(), assembly_exception);
        density_feature = new FeatureType("density_feature");
        out.put(density_feature.toString(), density_feature);
        ditag_feature = new FeatureType("ditag_feature");
        out.put(ditag_feature.toString(), ditag_feature);
        dna_align_feature = new FeatureType("dna_align_feature");
        out.put(dna_align_feature.toString(), dna_align_feature);
        dna_peptide_align_feature = new FeatureType("dna_peptide_align_feature");
        out.put(dna_peptide_align_feature.toString(), dna_peptide_align_feature);
        peptide_dna_align_feature = new FeatureType("peptide_dna_align_feature");
        out.put(peptide_dna_align_feature.toString(), peptide_dna_align_feature);
        exon = new FeatureType("exon");
        out.put(exon.toString(), exon);
        gene = new FeatureType("gene");
        out.put(gene.toString(), gene);
        karyotype_band = new FeatureType("karyotype_band");
        out.put(karyotype_band.toString(), karyotype_band);
        marker_feature = new FeatureType("marker_feature");
        out.put(marker_feature.toString(), marker_feature);
        misc_feature = new FeatureType("misc_feature");
        out.put(misc_feature.toString(), misc_feature);
        prediction_exon = new FeatureType("prediction_exon");
        out.put(prediction_exon.toString(), prediction_exon);
        prediction_transcript = new FeatureType("prediction_transcript");
        out.put(prediction_transcript.toString(), prediction_transcript);
        protein_align_feature = new FeatureType("protein_align_feature");
        out.put(protein_align_feature.toString(), protein_align_feature);
        protein_feature = new FeatureType("protein_feature");
        out.put(protein_feature.toString(), protein_feature);
        repeat_feature = new FeatureType("repeat_feature");
        out.put(repeat_feature.toString(), repeat_feature);
        simple_feature = new FeatureType("simple_feature");
        out.put(simple_feature.toString(), simple_feature);
        splicing_event = new FeatureType("splicing_event");
        out.put(splicing_event.toString(), splicing_event);
        transcript = new FeatureType("transcript");
        out.put(transcript.toString(), transcript);

        return out;

    }

    public static FeatureType getFeatureType(String value) {
        return typeListHash.get(value);
    }
}
