/**
 * Copyright (C) 2010 The Roslin Institute <contact andy.law@roslin.ed.ac.uk>
 *
 * This file is part of the Ensembl Java API demonstration project developed by the
 * Bioinformatics Group at The Roslin Institute, The Royal (Dick) School of
 * Veterinary Studies, University of Edinburgh.
 *
 * This is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License (version 3) as published by
 * the Free Software Foundation.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * in this software distribution. If not, see <http://www.gnu.org/licenses/gpl-3.0.html/>.
 */

package uk.ac.roslin.ensembl.dao.database;

import java.util.Properties;
import uk.ac.roslin.ensembl.dao.factory.DAOFactory;
import uk.ac.roslin.ensembl.exception.DAOException;


public abstract class DBBaseDAO {

    protected DAOFactory daoFactory = null;
    protected Properties configuration = null;

    public DBBaseDAO() {
    }

    /**
     * Constructor for (subclasses of) this class. Takes configuration from
     * the parent DBDAOFactory
     * @param inFactory the DAOFactory that constructs the DAO Object
     */
    public DBBaseDAO(DAOFactory inFactory) throws DAOException {
        if (inFactory==null) {
            throw new DAOException ("No DAOFactory to initialize DBDAO");
        }
        this.setFactory(inFactory);
    }

    /**
     * Constructor for (subclasses of) this class. Takes configuration from
     * a Properties File.
     * @param inConfiguration Properties for connection
     */
    public DBBaseDAO(Properties inConfiguration) throws DAOException  {
        if (inConfiguration==null) {
            throw new DAOException ("No Configuration to initialize DBDAO");
        }
        this.setConfiguration(inConfiguration);
    }


    public DAOFactory getFactory() {
        return this.daoFactory;
    }

    public void setConfiguration(Properties inConfiguration) {
        this.configuration = inConfiguration;
    }

    public void setFactory(DAOFactory factory) {
        this.daoFactory=factory;
        if(this.daoFactory!=null) {
            this.setConfiguration(this.daoFactory.getConfiguration());
        }
    }
  
  

}
