/**
 * Copyright (C) 2010 The Roslin Institute <contact andy.law@roslin.ed.ac.uk>
 *
 * This file is part of the Ensembl Java API demonstration project developed by the
 * Bioinformatics Group at The Roslin Institute, The Royal (Dick) School of
 * Veterinary Studies, University of Edinburgh.
 *
 * This is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License (dBversion 3) as published by
 * the Free Software Foundation.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * in this software distribution. If not, see <http://www.gnu.org/licenses/gpl-3.0.html/>.
 */

package uk.ac.roslin.ensembl.dao.database;

import java.util.Collection;
import java.util.HashMap;
import java.util.TreeSet;
import uk.ac.roslin.ensembl.model.database.CollectionDatabase;
import uk.ac.roslin.ensembl.model.core.CollectionSpecies;
import uk.ac.roslin.ensembl.model.database.DatabaseType;
import uk.ac.roslin.ensembl.model.core.CollectionOfSpecies;

public class DBCollection implements CollectionOfSpecies  {

    private String collectionName="";
    private String dBversion="";
    private String schemaVersion="";
    private TreeSet<DBCollectionSpecies> species = new TreeSet<DBCollectionSpecies>();
    private DBCollectionCoreDatabase coreDatabase;
    private TreeSet<DBCollectionDatabase> databases = new TreeSet<DBCollectionDatabase>();

    public DBCollection() {
    }

    public DBCollection(DBCollectionCoreDatabase database) {
        this.setPrimaryCoreDatabase(database);
    }

    public void setPrimaryCoreDatabase(DBCollectionCoreDatabase database) {
        this.coreDatabase = database;
        if (this.coreDatabase != null) {
            this.databases.add(database);
        }
        collectionName=this.coreDatabase!=null ? coreDatabase.getCollectionName() : null;
        schemaVersion = this.coreDatabase!=null ? coreDatabase.getSchemaVersion() : null;
        dBversion = this.coreDatabase != null ? coreDatabase.getDBVersion() : null ;
    }


    @Override
    public void setProperty(String id, HashMap row) {
    }

    @Override
    public String getDBVersion() {
        return dBversion;
    }

    @Override
    public String getCollectionName() {
        return collectionName;
    }

    @Override
    public TreeSet<DBCollectionDatabase> getDatabases() {
        return databases;
    }

    @Override
    public void addDatabases(TreeSet<? extends CollectionDatabase> databases) {

        for (CollectionDatabase d : databases) {
            try {
                this.databases.add((DBCollectionDatabase) d);
            } catch (Exception e) {
            }
        }
    }

    @Override
    public TreeSet<DBCollectionSpecies> getSpecies() {
        return this.species;
    }

    @Override
    public void addSpecies(Collection<? extends CollectionSpecies> spp) {
        for (CollectionSpecies cs : spp) {
                this.addSpecies(cs);
        }
    }

    @Override
    public void addSpecies(CollectionSpecies sp) {
            try {
                this.species.add((DBCollectionSpecies) sp);
            } catch (Exception e) {
            }
    }

    @Override
    public DBCollectionCoreDatabase getCoreDatabase() {
        return coreDatabase;
    }

    @Override
    public String getSchemaVersion() {
        return schemaVersion;
    }

    @Override
    public TreeSet<DBCollectionDatabase> getDatabasesByType(DatabaseType type) {
        TreeSet<DBCollectionDatabase> out = new TreeSet<DBCollectionDatabase>();
        for (DBCollectionDatabase d : databases) {
            if (d.getType()==type) {
                out.add(d);
            }
        }
        return out;
    }

    @Override
    public String toString() {
        return this.collectionName;
    }


}
