/**
 * Copyright (C) 2010 The Roslin Institute <contact andy.law@roslin.ed.ac.uk>
 *
 * This file is part of the Ensembl Java API demonstration project developed by the
 * Bioinformatics Group at The Roslin Institute, The Royal (Dick) School of
 * Veterinary Studies, University of Edinburgh.
 *
 * This is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License (version 3) as published by
 * the Free Software Foundation.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * in this software distribution. If not, see <http://www.gnu.org/licenses/gpl-3.0.html/>.
 */

package uk.ac.roslin.ensembl.dao.database;

import java.util.HashMap;
import uk.ac.roslin.ensembl.model.database.CollectionDatabase;
import uk.ac.roslin.ensembl.exception.ConfigurationException;
import uk.ac.roslin.ensembl.model.core.CollectionOfSpecies;
import uk.ac.roslin.ensembl.config.EnsemblDBType;
import uk.ac.roslin.ensembl.exception.DAOException;
import uk.ac.roslin.ensembl.model.core.CollectionSpecies;
import uk.ac.roslin.ensembl.model.database.Registry;

/**
 *
 * @author paterson
 */
// will make this abstract once we fill in all the subtypes


public class DBCollectionDatabase extends DBDatabase  implements CollectionDatabase {

    protected String collectionName = null;
    protected DBCollection collection = null;
    protected HashMap<CollectionSpecies, String> assemblyMap = null;

    public DBCollectionDatabase(String db_name, EnsemblDBType type, Registry registry) throws ConfigurationException {
        super(db_name, type, registry);
        init();
    }

    private void init() throws ConfigurationException {

        this.collectionName = dBName.substring(0, dBName.indexOf("_" + this.type.toString()));
        String sub = dBName.replace(collectionName+"_"+type.toString()+"_","");
        if (sub.contains("_")) {
            this.dbVersion = sub.substring(0, sub.indexOf("_"));
            intDBVersion = Integer.parseInt(dbVersion);
            sub = sub.replace(dbVersion + "_", "");
            if (sub.contains("_")) {
                this.schemaVersion = sub.substring(0, sub.indexOf("_"));
                intSchemaVersion = Integer.parseInt(schemaVersion);
                build = sub.replace(schemaVersion+"_", "");
            } else {
                this.schemaVersion = sub;
                intSchemaVersion = Integer.parseInt(schemaVersion);
            }


        } else {
            this.dbVersion = sub;
            intDBVersion = Integer.parseInt(dbVersion);
            schemaVersion = dbVersion;
            intSchemaVersion = intDBVersion;
        }

    }

    @Override
    public String getCollectionName() {
        return collectionName;
    }

    @Override
    public void setCollectionName(String collectionName) {
        this.collectionName = collectionName;
    }


    @Override
    public void setCollection(CollectionOfSpecies r) {
        this.collection = (DBCollection) r;
    }

    @Override
    public DBCollection getCollection() {
        return this.collection;
    }

    @Override
    public String getAssembly(CollectionSpecies species) {

            if (this.assemblyMap== null) {
                this.assemblyMap = new HashMap<CollectionSpecies, String>();
                try {
                    this.lazyLoadAssemblies();
                } catch (DAOException e) {
                    //probably should throw this
                }
            }
            return this.assemblyMap.get(species);

    }

    private void lazyLoadAssemblies() throws DAOException  {

        try {
            this.assemblyMap = this.getRegistry().getAssemblies(this);
        } catch (Exception e) {
            throw new DAOException("Failure to getAssemblies for species of this collection",e);
        }
    }
 
}
