/**
 * Copyright (C) 2010 The Roslin Institute <contact andy.law@roslin.ed.ac.uk>
 *
 * This file is part of the Ensembl Java API demonstration project developed by the
 * Bioinformatics Group at The Roslin Institute, The Royal (Dick) School of
 * Veterinary Studies, University of Edinburgh.
 *
 * This is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License (version 3) as published by
 * the Free Software Foundation.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * in this software distribution. If not, see <http://www.gnu.org/licenses/gpl-3.0.html/>.
 */

package uk.ac.roslin.ensembl.dao.database;

import uk.ac.roslin.ensembl.config.EnsemblComparaDivision;
import uk.ac.roslin.ensembl.model.database.ComparisonDatabase;
import uk.ac.roslin.ensembl.dao.database.factory.DBDAOComparaFactory;
import uk.ac.roslin.ensembl.exception.ConfigurationException;
import uk.ac.roslin.ensembl.config.EnsemblDBType;

/**
 *
 * @author paterson
 */
public class DBComparisonDatabase extends DBDatabase implements ComparisonDatabase {

    private String comparisonDivisionName = "multi";
    private EnsemblComparaDivision comparisonDivision = EnsemblComparaDivision.MULTI;

    public DBComparisonDatabase(String db_name, EnsemblDBType type, DBRegistry registry) throws ConfigurationException {
        super(db_name, type, registry);
        init();
    }

    private void init() throws ConfigurationException {

        //ensembdbl are of style species_genus_type_version_build (i.e. 2 suffixes after type)
        //ensemblgenomes are of style species_genus_type_version_ensembldbversion_build (ie 3 suffixes after type)

        //delete the leading type
        String sub = dBName.replace(type + "_", "");

        //does suffix have >1 token
        if (sub.contains("_")) {
            String next = sub.substring(0, sub.indexOf("_"));

            //if we have a word here, it is a comparisonGroupName
            if (next.matches("^[a-z]+")) {
                comparisonDivisionName = next;
                sub = sub.replace(next + "_", "");
                //if >1 suffix token
                if (sub.contains("_")) {

                    next = sub.substring(0, sub.indexOf("_"));
                    //if double barreld comparisonGroupName name
                    if (next.matches("^[a-z]+")) {
                        comparisonDivisionName = comparisonDivisionName.concat("_" + next);
                        sub = sub.replace(next + "_", "");
                        //if >1 suffix token
                        if (sub.contains("_")) {
                            dbVersion = sub.substring(0,sub.indexOf("_"));
                            intDBVersion = Integer.parseInt(dbVersion);
                            schemaVersion = sub.replace(dbVersion + "_", "");
                            intSchemaVersion = Integer.parseInt(schemaVersion);
                        } else {
                            this.dbVersion = sub;
                            intDBVersion = Integer.parseInt(dbVersion);
                            this.schemaVersion = dbVersion;
                            this.intSchemaVersion = intDBVersion;
                        }


                    } else {
                        dbVersion = next;
                        intDBVersion = Integer.parseInt(dbVersion);
                        schemaVersion = sub.substring(sub.indexOf("_") + 1);
                        intSchemaVersion = Integer.parseInt(schemaVersion);
                    }
                } else // only one suffix token
                {
                    dbVersion = sub;
                    intDBVersion = Integer.parseInt(dbVersion);
                    schemaVersion = dbVersion;
                    intSchemaVersion = intDBVersion;
                }
            } else //no word so straight in to suffixes
            {
                dbVersion = next;
                intDBVersion = Integer.parseInt(dbVersion);
                if (sub.contains("_")) {
                    schemaVersion = sub.substring(sub.indexOf("_") + 1);
                    intSchemaVersion = Integer.parseInt(schemaVersion);
                } else {
                    schemaVersion = dbVersion;
                    intSchemaVersion = intDBVersion;
                }
            }
        } else //suffix has only one token, the ensembl release
        {
            this.dbVersion = sub;
            intDBVersion = Integer.parseInt(dbVersion);
            this.schemaVersion = dbVersion;
            this.intSchemaVersion = intDBVersion;
        }

        this.comparisonDivision = EnsemblComparaDivision.getEnsemblComparaDivision(comparisonDivisionName);

    }

    @Override
    public EnsemblComparaDivision getComparisonDivision() {
        return comparisonDivision;
    }

    public DBDAOComparaFactory getComparaFactory() {
        if (this.factory != null) {
            return (DBDAOComparaFactory) this.factory;
        } else {
            try {
                this.factory = new DBDAOComparaFactory(this);
            } catch (Exception ex) {
            }
            return (DBDAOComparaFactory) this.factory;

        }
    }
}
