/**
 * Copyright (C) 2010 The Roslin Institute <contact andy.law@roslin.ed.ac.uk>
 *
 * This file is part of the Ensembl Java API demonstration project developed by the
 * Bioinformatics Group at The Roslin Institute, The Royal (Dick) School of
 * Veterinary Studies, University of Edinburgh.
 *
 * This is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License (version 3) as published by
 * the Free Software Foundation.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * in this software distribution. If not, see <http://www.gnu.org/licenses/gpl-3.0.html/>.
 */

package uk.ac.roslin.ensembl.dao.database;

import java.io.InputStreamReader;
import java.io.Reader;
import java.util.Properties;
import org.apache.ibatis.session.SqlSession;
import org.apache.ibatis.session.SqlSessionFactory;
import org.apache.ibatis.session.SqlSessionFactoryBuilder;
import uk.ac.roslin.ensembl.model.database.Database;
import uk.ac.roslin.ensembl.dao.database.factory.DBDAOFactory;
import uk.ac.roslin.ensembl.exception.ConfigurationException;
import uk.ac.roslin.ensembl.config.EnsemblDBType;
import uk.ac.roslin.ensembl.exception.DAOException;
import uk.ac.roslin.ensembl.model.database.Registry;

/**
 *
 * @author paterson
 */
public abstract class DBDatabase implements Database {

    EnsemblDBType type  = null;
    String dBName = null;
    String dbVersion = null ;
    int intDBVersion ;
    String schemaVersion = null ;
    int intSchemaVersion ;
    String build  = null;
    DBDAOFactory factory = null;
    Registry registry = null;

    public DBDatabase() {
        
    }

    public DBDatabase (String db_name, EnsemblDBType type, Registry registry) throws ConfigurationException {
        this.setdBName(db_name);
        this.setType(type);
        this.setRegistry(registry);

    }

    @Override
    public String getdBName() {
        return dBName;
    }

    public void setdBName(String dBName) {
        this.dBName = dBName;
    }

    @Override
    public EnsemblDBType getType() {
        return type;
    }

    public void setType(EnsemblDBType type) {
        this.type = type;
    }

    @Override
    public String getDBVersion() {
        return dbVersion;
    }

    public void setDBVersion(String version) {
        this.dbVersion = version;
    }

    @Override
    public int getIntDBVersion() {
        return intDBVersion;
    }

    public void setIntDBVersion(int intVersion) {
        this.intDBVersion = intVersion;
    }

    @Override
    public String getSchemaVersion() {
        return schemaVersion;
    }

    public void setSchemaVersion(String ensemblSchemaVersion) {
        this.schemaVersion = ensemblSchemaVersion;
    }

    @Override
    public int getIntSchemaVersion() {
        return intSchemaVersion;
    }

    public void setIntSchemaVersion(int intEnsemblSchemaVersion) {
        this.intSchemaVersion = intEnsemblSchemaVersion;
    }

    @Override
    public String getBuild() {
        return build;
    }

    public void setBuild(String build) {
        this.build = build;
    }

    @Override
    public int compareTo(Database other) {
        if (this.getdBName()!=null && other != null &&  other.getdBName()!=null) {
            return this.getdBName().compareTo(other.getdBName());
        } else {
            if (this.getdBName()!=null) {
                return 1;
            } else {
                return -1;
            }
        }
    }

    
    @Override
    public Registry  getRegistry() {
        return registry;
    }

    public void setRegistry(Registry r) {
        this.registry=r;
    }

    @Override
    public String toString() {
        return "release: "+this.getDBVersion()+" [genome build: "+this.getBuild()+"]";
    }


   //*************************************************************************


    private SqlSessionFactory sqlSessionFactory = null;

    private SqlSessionFactory getSessionFactory() throws DAOException {

        if (sqlSessionFactory == null) {
            sqlSessionFactory = createSessionFactory();
        }
        return sqlSessionFactory;
    }

    @Override
    public SqlSession getNewSqlSession() throws DAOException {
        try {
            return getSessionFactory().openSession();
        } catch (Exception e) {
            throw new DAOException("Failed to retrieve an SqlSession from the Factory", e);
        }
    }

    private SqlSessionFactory createSessionFactory() throws DAOException {

          SqlSessionFactory out = null;
          Reader reader = null;

          Properties configuration = (Properties) registry.getConfigProperties().clone();
          String thisDBUrl = configuration.getProperty("url").concat("/").concat(this.getdBName() + "?zeroDateTimeBehavior=convertToNull");
          String mybatisSchemaFilePath = registry.findMybatisSchemaForSchemaVersion(this.getType(), this.getSchemaVersion());

          configuration.setProperty("url", thisDBUrl);
          configuration.setProperty("ensembl_release", this.getSchemaVersion());
          configuration.setProperty("mybatis_file", mybatisSchemaFilePath);


           try {

            String  resource = mybatisSchemaFilePath;
            reader = new InputStreamReader(this.getClass().getClassLoader().getResourceAsStream(resource));

            if (reader == null) {
                /* "belt and braces" check, as nothing is impossible */
                throw new DAOException("Failed to read Configuration.xml");
            }

            out = (new SqlSessionFactoryBuilder()).build(reader, "current", configuration);


            if (out == null) {
                /* also unlikely, but again a "belt and braces" check */
                throw new DAOException("Failed to build SqlMapClient");
            }

              reader.close();

        } catch (Exception e) {

            throw new DAOException(e);
        }


          return out;
   }

}
