/**
 * Copyright (C) 2010 The Roslin Institute <contact andy.law@roslin.ed.ac.uk>
 *
 * This file is part of the Ensembl Java API demonstration project developed by the
 * Bioinformatics Group at The Roslin Institute, The Royal (Dick) School of
 * Veterinary Studies, University of Edinburgh.
 *
 * This is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License (version 3) as published by
 * the Free Software Foundation.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * in this software distribution. If not, see <http://www.gnu.org/licenses/gpl-3.0.html/>.
 */

package uk.ac.roslin.ensembl.dao.database;

import uk.ac.roslin.ensembl.exception.ConfigurationException;
import uk.ac.roslin.ensembl.model.core.Species;
import uk.ac.roslin.ensembl.model.database.SingleSpeciesDatabase;
import uk.ac.roslin.ensembl.config.EnsemblDBType;
import uk.ac.roslin.ensembl.exception.DAOException;
import uk.ac.roslin.ensembl.model.database.Registry;

/**
 *
 * @author tpaterso
 */


// will make this abstract once we fill in all the subtypes

public class DBSingleSpeciesDatabase extends DBDatabase implements SingleSpeciesDatabase {

     protected DBSpecies species = null;
     protected String dbSpeciesName;
     protected String assembly = null;


    public DBSingleSpeciesDatabase(String db_name, EnsemblDBType type, Registry registry) throws ConfigurationException {
        super(db_name, type, registry);
        init();
    }

    private void init() throws ConfigurationException {

        //ensembdbl are of style species_genus_type_version_build (i.e. 2 suffixes after type)
        //ensemblgenomes are of style species_genus_type_version_ensembldbversion_build (ie 3 suffixes after type)

        //this code will fall over if the magic conventions of db naming change!
        //or if we have both styles in the same database!
        //see DBRegistry line 215

        this.dbSpeciesName = dBName.substring(0, dBName.indexOf("_" + this.type.toString()));
        String sub = dBName.replace(dbSpeciesName + "_" + type.toString() + "_", "");

        //does the suffix have >1 token
        if (sub.contains("_")) {
            this.dbVersion = sub.substring(0, sub.indexOf("_"));
            intDBVersion = Integer.parseInt(dbVersion);

            //check if it has three suffixes
            sub = sub.replace(dbVersion + "_", "");
            //does suffix have >2 tokens
            if (sub.contains("_")) {
                this.schemaVersion = sub.substring(0, sub.indexOf("_"));
                this.intSchemaVersion = Integer.parseInt(schemaVersion);
                build = sub.replace(schemaVersion + "_", "");

            } else //suffix has 2 tokens, therefore second is build
            {
                schemaVersion = dbVersion;
                intSchemaVersion = intDBVersion;
                build = sub;
            }

        } else //suffix has only one token, the ensembl release
        {
            this.dbVersion = sub;
            intDBVersion = Integer.parseInt(dbVersion);
            this.schemaVersion = dbVersion;
            this.intSchemaVersion = intDBVersion;
        }

    }

    @Override
    public String getDbSpeciesName() {
        return dbSpeciesName;
    }

    @Override
    public void setDbSpeciesName(String species) {
        this.dbSpeciesName = species;
    }


    @Override
    public void setSpecies(Species species) {
        this.species = (DBSpecies) species;
    }

    @Override
    public DBSpecies getSpecies() {
        return this.species;
    }

    @Override
    public String getAssembly() {
        if (assembly == null) {
            try {
                this.lazyLoadAssembly();
            } catch (DAOException e) {
                //probably should throw this
            }
        }
        return assembly;
    }

    public void lazyLoadAssembly() throws DAOException  {
        assembly = this.getRegistry().getAssembly(this);
    }
}
