/**
 * Copyright (C) 2010 The Roslin Institute <contact andy.law@roslin.ed.ac.uk>
 *
 * This file is part of the Ensembl Java API demonstration project developed by the
 * Bioinformatics Group at The Roslin Institute, The Royal (Dick) School of
 * Veterinary Studies, University of Edinburgh.
 *
 * This is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License (version 3) as published by
 * the Free Software Foundation.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * in this software distribution. If not, see <http://www.gnu.org/licenses/gpl-3.0.html/>.
 */

package uk.ac.roslin.ensembl.dao.database.coreaccess;

import uk.ac.roslin.ensembl.datasourceaware.core.DACoordinateSystem;
import java.util.HashMap;
import java.util.List;
import org.apache.ibatis.session.SqlSession;
import uk.ac.roslin.ensembl.model.Mapping;
import uk.ac.roslin.ensembl.model.MappingSet;
import uk.ac.roslin.ensembl.dao.coreaccess.AssemblyDAO;
import uk.ac.roslin.ensembl.datasourceaware.core.DAAssembledDNASequence;
import uk.ac.roslin.ensembl.datasourceaware.core.DADNASequence;
import uk.ac.roslin.ensembl.dao.database.factory.DBDAOCollectionCoreFactory;
import uk.ac.roslin.ensembl.dao.database.factory.DBDAOSingleSpeciesCoreFactory;
import uk.ac.roslin.ensembl.dao.factory.DAOCollectionCoreFactory;
import uk.ac.roslin.ensembl.dao.factory.DAOSingleSpeciesCoreFactory;
import uk.ac.roslin.ensembl.mapper.core.SequenceMapper;
import uk.ac.roslin.ensembl.exception.DAOException;
import uk.ac.roslin.ensembl.model.core.CoordinateSystem;
import uk.ac.roslin.ensembl.model.core.DNASequence;

/**
 *
 * @author paterson
 */
public class DBAssemblyDAO extends DBCoreObjectDAO implements AssemblyDAO{
    
    public DBAssemblyDAO()  {
        super();
    }

    public DBAssemblyDAO(DAOSingleSpeciesCoreFactory factory)  {
        super(factory);
    }
    
    public DBAssemblyDAO(DAOCollectionCoreFactory factory) {
        super(factory);
    }

    @Override
    public MappingSet getComponentMappingsByStartStop(DNASequence assembledSequence, Integer begin, Integer stop) throws DAOException {


        if (assembledSequence==null || begin==null || stop==null
                || assembledSequence.getId() == null
                || assembledSequence.getCoordSystem() == null
                || assembledSequence.getCoordSystem().getId() == null ) {
            throw new DAOException("Invalid parameters passed to getComponentMappingsByStartStopDNASequence assembledSequence, Integer begin, Integer stop ");
        }

        MappingSet componentMappings = new MappingSet();
        List<Mapping> results;
        DAAssembledDNASequence parent;
        SqlSession session = null;
        
        try {
            parent = (DAAssembledDNASequence) assembledSequence;
        } catch (ClassCastException e) {
            throw new DAOException("This sequence is not an assembly!");
        }

        Integer start = begin;
        Integer end = stop;
        CoordinateSystem seqLevelCS = null;

        try {
        if (singleSpecies) {
            seqLevelCS = ssFactory.getDatabase().getSequenceLevelCoordSystem();
        } else  {
            seqLevelCS = collFactory.getDatabase().getSequenceLevelCS(species);
        }

        if (seqLevelCS==null || seqLevelCS.getId()==null  ) {
            throw new DAOException("Failed to call to retrieve the CoordinateSystem for Sequences");
        }

        Integer seqLevelCoordSysID = seqLevelCS.getId();
        Integer parentCoordSysID = parent.getCoordSystem().getId();

        if (seqLevelCoordSysID == parentCoordSysID) {
            //the parent is at sequence level - so dont need a projection
            //probably prevent this call happening before here
             throw new DAOException("This sequence is not an assembly!");
        }

        Integer seqRegionID = parent.getId();

        //none of these should be null

        HashMap parameters = new HashMap();
        parameters.put("seqRegionID", seqRegionID);
        parameters.put("start", start);
        parameters.put("end", end);
        parameters.put("parentCoordSysID", parentCoordSysID);
        parameters.put("seqLevelCoordSysID", seqLevelCoordSysID);





            session = this.getFactory().getNewSqlSession();
            SequenceMapper mapper = session.getMapper(SequenceMapper.class);
            results = mapper.getComponentSequences(parameters);
        } catch (Exception e) {
            throw new DAOException("Failed to call getComponentMappingsByStartStop", e);
        } finally {
            if (session != null) {
                session.close();
            }
        }

        if (results ==null || results.isEmpty()) {
             return componentMappings;
        }

        if (results != null) {
            for (Mapping mapping: results) {

                mapping.setSource(parent);
                //everything returned from this query is a sequence level component
                ((DADNASequence) mapping.getTarget()).setDaoFactory(daoFactory);
                //this is now done by mybatis
                //((DADNASequence) mapping.getTarget()).setSequenceStorage(mapping.getReader());

                //((DADNASequence) mapping.getTarget()).setType(seqLevelCS.getType());
                ((DADNASequence) mapping.getTarget()).setCoordSystem(seqLevelCS);

                //this adds the reverse mapping to the DA DNASequence -
                //not sure if this is required, or if better stored as another type of relationship

                if (componentMappings.add(mapping)) {
                    Mapping.addReverseMapping(mapping);
                }
            }
        }


        return componentMappings;

    }



}
