/**
 * Copyright (C) 2010 The Roslin Institute <contact andy.law@roslin.ed.ac.uk>
 *
 * This file is part of the Ensembl Java API demonstration project developed by the
 * Bioinformatics Group at The Roslin Institute, The Royal (Dick) School of
 * Veterinary Studies, University of Edinburgh.
 *
 * This is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License (version 3) as published by
 * the Free Software Foundation.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * in this software distribution. If not, see <http://www.gnu.org/licenses/gpl-3.0.html/>.
 */
package uk.ac.roslin.ensembl.dao.database.coreaccess;

import java.util.ArrayList;
import uk.ac.roslin.ensembl.datasourceaware.core.DACoordinateSystem;
import java.util.HashMap;
import java.util.List;
import org.apache.ibatis.session.SqlSession;
import uk.ac.roslin.ensembl.dao.coreaccess.ChromosomeDAO;
import uk.ac.roslin.ensembl.datasourceaware.core.DAChromosome;
import uk.ac.roslin.ensembl.dao.database.factory.DBDAOCollectionCoreFactory;
import uk.ac.roslin.ensembl.dao.database.factory.DBDAOSingleSpeciesCoreFactory;
import uk.ac.roslin.ensembl.dao.factory.DAOCollectionCoreFactory;
import uk.ac.roslin.ensembl.dao.factory.DAOSingleSpeciesCoreFactory;
import uk.ac.roslin.ensembl.mapper.core.ChromosomeMapper;
import uk.ac.roslin.ensembl.exception.DAOException;
import uk.ac.roslin.ensembl.model.core.CoordinateSystem;

/**
 *
 * @author paterson
 */
public class DBChromosomeDAO extends DBCoreObjectDAO implements ChromosomeDAO {

    public DBChromosomeDAO() {
        super();
    }

    public DBChromosomeDAO(DAOSingleSpeciesCoreFactory factory) {
        super(factory);
    }

    public DBChromosomeDAO(DAOCollectionCoreFactory factory)  {
        super(factory);
    }

    @Override
    public DAChromosome getChromosomeByName(String name) throws DAOException {

        if (name==null || name.isEmpty()) {
            return null;
        }

        DAChromosome out = null;
        String chrName = name;
        CoordinateSystem chrCS = null;

        if (singleSpecies) {
            chrCS = ssFactory.getDatabase().getChromosomeLevelCoordSystem();
        } else {
            chrCS = collFactory.getDatabase().getChromosomeLevelCS(species);
        }

        if (chrCS==null || chrCS.getId()==null  ) {
            throw new DAOException("Failed to call to retrieve the CoordinateSystem for Chromosomes");
        }

        Integer chrCSID = chrCS.getId();

        HashMap parameters = new HashMap();
        parameters.put("chrName",chrName);
        parameters.put("coordSysID", chrCSID);

        SqlSession session = null;

        try {
            session = this.getFactory().getNewSqlSession();
            ChromosomeMapper mapper = session.getMapper(ChromosomeMapper.class);
            out = mapper.getChromosomeByName_CoordSysID(parameters);
        } catch (Exception e) {
            throw new DAOException("Failed to call getChromsomeByName_CoordSysID", e);
        } finally {
            if (session != null) {
                session.close();
            }
        }

        if (out != null) {
            out.setDaoFactory(daoFactory);
            out.setCoordSystem(chrCS);
            out.setSpecies(species);
        }

        return out;
    }

    @Override
    public List<DAChromosome> getChromosomes() throws DAOException {

        List<DAChromosome> out = new ArrayList<DAChromosome>();
        CoordinateSystem chrCS = null;

        if (singleSpecies) {
            chrCS = ssFactory.getDatabase().getChromosomeLevelCoordSystem();
        } else {
            chrCS = collFactory.getDatabase().getChromosomeLevelCS(species);
        }

        if (chrCS==null || chrCS.getId()==null  ) {
            throw new DAOException("Failed to call to retrieve the CoordinateSystem for Chromosomes");
        }

        Integer chrCSID = chrCS.getId();

        HashMap parameters = new HashMap();
        parameters.put("coordSysID", chrCSID);

        SqlSession session = null;

        try {
            session = this.getFactory().getNewSqlSession();
            ChromosomeMapper mapper = session.getMapper(ChromosomeMapper.class);
            out = mapper.getChromosomesByCoordSysID(parameters);
        } catch (Exception e) {
            throw new DAOException("Failed to call getChromsomes", e);
        } finally {
            if (session != null) {
                session.close();
            }
        }
        for (DAChromosome c : out) {
            c.setDaoFactory(daoFactory);
            c.setCoordSystem(chrCS);
            c.setSpecies(species);
        }

        return out;
    }
}


