/**
 * Copyright (C) 2010 The Roslin Institute <contact andy.law@roslin.ed.ac.uk>
 *
 * This file is part of the Ensembl Java API demonstration project developed by the
 * Bioinformatics Group at The Roslin Institute, The Royal (Dick) School of
 * Veterinary Studies, University of Edinburgh.
 *
 * This is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License (version 3) as published by
 * the Free Software Foundation.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * in this software distribution. If not, see <http://www.gnu.org/licenses/gpl-3.0.html/>.
 */

package uk.ac.roslin.ensembl.dao.database.coreaccess;

import uk.ac.roslin.ensembl.dao.database.DBBaseDAO;
import uk.ac.roslin.ensembl.dao.database.DBCollectionSpecies;
import uk.ac.roslin.ensembl.dao.database.DBSpecies;
import uk.ac.roslin.ensembl.dao.database.factory.DBDAOCollectionCoreFactory;
import uk.ac.roslin.ensembl.dao.database.factory.DBDAOSingleSpeciesCoreFactory;
import uk.ac.roslin.ensembl.dao.factory.DAOCollectionCoreFactory;
import uk.ac.roslin.ensembl.dao.factory.DAOCollectionFactory;
import uk.ac.roslin.ensembl.dao.factory.DAOSingleSpeciesCoreFactory;
import uk.ac.roslin.ensembl.dao.factory.DAOSingleSpeciesFactory;
import uk.ac.roslin.ensembl.exception.DAOException;

/**
 *
 * @author paterson
 */
public abstract class DBCoreObjectDAO extends DBBaseDAO {

    protected DBSpecies species = null;
    protected Boolean singleSpecies = true;
    protected DAOSingleSpeciesCoreFactory ssFactory = null;
    protected DAOCollectionCoreFactory collFactory = null;

    public DBCoreObjectDAO() {
        super();
    }


    public DBCoreObjectDAO(DAOSingleSpeciesCoreFactory factory) {
        this();
        this.setFactory(factory);
      }

    public DBCoreObjectDAO(DAOCollectionCoreFactory factory)  {
        this();
        this.setFactory(factory);
    }


    public void setFactory(DAOSingleSpeciesCoreFactory factory) {
        super.setFactory(factory);
        //old databases for species removed from current ensembl don't have species associated!
        //cos their is no current species for them
        if (factory.getSpecies()!= null) {
            species = (DBSpecies) factory.getSpecies();
        }
        this.singleSpecies = true;
        this.ssFactory =  factory;
    }

    public void setFactory(DAOCollectionCoreFactory factory) {
        super.setFactory(factory);
        species = (DBCollectionSpecies) factory.getSpecies();
        this.singleSpecies = false;
        this.collFactory = factory;
    }



    public Boolean isSingleSpecies() {
        return this.singleSpecies;
    }

//    @Override
//    public DBDAOSpeciesFactory getFactory() {
//        return (DBDAOSpeciesFactory) this.daoFactory;
//    }

    public DBSpecies getSpecies() {
        return species;
    }

}
