/**
 * Copyright (C) 2010 The Roslin Institute <contact andy.law@roslin.ed.ac.uk>
 *
 * This file is part of the Ensembl Java API demonstration project developed by the
 * Bioinformatics Group at The Roslin Institute, The Royal (Dick) School of
 * Veterinary Studies, University of Edinburgh.
 *
 * This is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License (version 3) as published by
 * the Free Software Foundation.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * in this software distribution. If not, see <http://www.gnu.org/licenses/gpl-3.0.html/>.
 */

package uk.ac.roslin.ensembl.dao.database.coreaccess;

import uk.ac.roslin.ensembl.exception.DAOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import org.apache.ibatis.session.SqlSession;
import uk.ac.roslin.ensembl.dao.coreaccess.GeneDAO;
import uk.ac.roslin.ensembl.datasourceaware.core.DAGene;
import uk.ac.roslin.ensembl.config.FeatureType;
import uk.ac.roslin.ensembl.model.Mapping;
import uk.ac.roslin.ensembl.datasourceaware.DAXRef;
import uk.ac.roslin.ensembl.datasourceaware.core.DAAssembledDNASequence;
import uk.ac.roslin.ensembl.datasourceaware.core.DAChromosome;
import uk.ac.roslin.ensembl.datasourceaware.core.DADNASequence;
import uk.ac.roslin.ensembl.dao.database.DBCollectionSpecies;
import uk.ac.roslin.ensembl.mapper.handler.DBResultHandler;
import uk.ac.roslin.ensembl.mapper.query.FeatureQuery;
import uk.ac.roslin.ensembl.mapper.core.GeneMapper;
import uk.ac.roslin.ensembl.model.Coordinate;
import uk.ac.roslin.ensembl.model.core.CoreObject;
import uk.ac.roslin.ensembl.model.core.DNASequence;
import uk.ac.roslin.ensembl.config.EnsemblCoordSystemType;
import uk.ac.roslin.ensembl.dao.factory.DAOCollectionCoreFactory;
import uk.ac.roslin.ensembl.dao.factory.DAOCoreFactory;
import uk.ac.roslin.ensembl.dao.factory.DAOSingleSpeciesCoreFactory;
import uk.ac.roslin.ensembl.model.core.CoordinateSystem;

public class DBGeneDAO extends DBCoreObjectDAO implements GeneDAO {

    //if the meta table tells us that genebuild.level is 'toplevel'
    //we can grab the top ranked CS and use this for all genes
    //there may be other values than 'toplevel' - i dont know what :)
    //if the build level isn't annotated we have to get the possible CS levels
    //from meta_coord and use which ever one is approriate!
    CoordinateSystem geneBuildCS = null;


    public DBGeneDAO() {
        super();
    }

    public DBGeneDAO(DAOSingleSpeciesCoreFactory factory) throws DAOException {
        super(factory);

        //we want to initialize the GeneDAO so that it knows which COordinate system
        //it needs to quwery about genes

        geneBuildCS = this.ssFactory.getDatabase().getBuildCoordSystem(FeatureType.gene.toString());
        //what if this is null or throws an exception?
        //if it throws an exception we are probably in some sort of bad factory environment
        if (geneBuildCS==null) {
            geneBuildCS=this.ssFactory.getDatabase().getTopLevelCoordSystem();
        }
    }

    public DBGeneDAO(DAOCollectionCoreFactory factory) throws DAOException {
        super(factory);
        geneBuildCS= this.collFactory.getDatabase().getBuildCoordSystem(species, FeatureType.gene.toString());
        //what if this is null  or throw an exception?
        //if it throws an exception we are probably in some sorto f baf dfactory environment
        if (geneBuildCS==null) {
            geneBuildCS=this.collFactory.getDatabase().getTopLevelCS(species);
        }
    }

    /**
     * Uses the stableid of an object to fill in missing data that would have been
     * present if the the gene had intially been made by a call to getGeneByStableID
     * or getGeneByID.
     * @param object
     * @throws DAOException
     */
    @Override
    public void reInitialize(CoreObject object) throws DAOException {

        if (object == null ||  !( object instanceof DAGene) ) {
            throw new DAOException("Object not a DAGene");
        }

        DAGene gene = (DAGene) object;

        //the DAO method requires a stableID
        if (gene.getStableID()==null || gene.getStableID().isEmpty() ) {
            return;
        }

        DAGene temp = this.getGeneByStableID(gene.getStableID());

        gene.setDisplayXRef(temp.getDisplayXRef());
        gene.setBiotype(temp.getBiotype());
        gene.setId(temp.getId());
        gene.setCanonicalTranscriptID(temp.getCanonicalTranscriptID());
        gene.setCreationDate(temp.getCreationDate());
        gene.setModificationDate(temp.getModificationDate());
        gene.setDescription(temp.getDescription());

        for (Mapping tempMapping :temp.getMappings()) {

            Mapping tempRevMapping = tempMapping.getReverseMapping();
            if (tempRevMapping != null) {
                tempRevMapping.setTarget(gene);
            }
            tempMapping.setSource(gene);
            gene.addMapping(tempMapping);

//            Mapping mapping = new Mapping();
//            mapping.setSource(gene);
//            mapping.setTargetCoordinates(tempMapping.getTargetCoordinates());
////            m.getTarget().clearAllMappings();
//            mapping.setTarget(tempMapping.getTarget());
//            if (gene.addMapping(mapping)) {
//                Mapping.addReverseMapping(mapping);
//            }
        }

    }

    /**
     * @param id
     * @return
     * @throws DAOException
     */
    @Override
    public DAGene getGeneByID(Integer id) throws DAOException {
        
        if (id==null) {
            return null;
        }

        FeatureQuery q = new FeatureQuery();
        q.setFeatureID(id);
        if (!singleSpecies) {
            try {
                q.setSpeciesID(species.getDBSpeciesID(this.getFactory().getDBVersion()));
            } catch (Exception e) {
                throw new DAOException("No species ID context for this query!", e);
            }
        }
        return this.getGene(q);
    }

    @Override
    public DAGene addGeneByIDOnParent(Integer id, DNASequence parent) throws DAOException {
        if (parent == null || !(parent instanceof DADNASequence) || ((CoreObject) parent).getId() == null) {
            throw new DAOException("No valid parent DNA provided");
        }
        if (id == null ) {
            return null;
        }
        FeatureQuery q = new FeatureQuery();
        if (!singleSpecies) {
            try {
                q.setSpeciesID(species.getDBSpeciesID(this.getFactory().getDBVersion()));
            } catch (Exception e) {
                throw new DAOException("No species ID context for this query!", e);
            }
        }
        q.setFeatureID(id);
        //this is redundant - if the rowGene doesnt map on this sequence
        //adding this parameter or not we would still get no genes back
        //adding this parameter has the effect of
        //making this a different query to getGeneByID and therefore
        //will return a differnt object  - not a cached one
        q.setParentSequenceID(parent.getId());
        return this.getGeneOnParent(q, (DADNASequence) parent);
    }

    /**
     * @param stableID
     * @return
     * @throws DAOException
     */
    @Override
    public DAGene getGeneByStableID(String stableID) throws DAOException {
        if (stableID==null || stableID.isEmpty()  ) {
            return null;
        }

        FeatureQuery q = new FeatureQuery();
        if (!singleSpecies) {
            try {
                q.setSpeciesID(species.getDBSpeciesID(this.getFactory().getDBVersion()));
            } catch (Exception e) {
                throw new DAOException("No species ID context for this query!", e);
            }
        }
        q.setFeatureStableID(stableID.trim());
        return this.getGene(q);
    }

    /**
     * The gene returned has mapping to a sequence region - and the id for 
     * the coordinate system that this region belongs to
     * @param stableID
     * @param parent
     * @return
     * @throws DAOException
     */
    @Override
    public DAGene addGeneByStableIDOnParent(String stableID, DNASequence parent) throws DAOException {
        if (parent == null || !(parent instanceof DADNASequence) || ((CoreObject) parent).getId() == null) {
            throw new DAOException("No valid parent DNA provided");
        }
        if (stableID == null || stableID.isEmpty() ) {
            return null;
        }
        FeatureQuery q = new FeatureQuery();
        if (!singleSpecies) {
            try {
                q.setSpeciesID(species.getDBSpeciesID(this.getFactory().getDBVersion()));
            } catch (Exception e) {
                throw new DAOException("No species ID context for this query!", e);
            }
        }
        q.setFeatureStableID(stableID.trim());
        //this is redundant - if the rowGene doesnt map on this sequence
        //adding this parameter or not we would still get no genes back
        //adding this parameter has the effect of
        //making this a different query to getGeneByID and therefore
        //will return a differnt object  - not a cached one
        q.setParentSequenceID(parent.getId());
        return this.getGeneOnParent(q, (DADNASequence) parent);
    }

    @Override
    public List<Mapping> getGeneMappingsOnRegion(DNASequence region, Coordinate coords) throws DAOException  {

        List<Mapping> mappings = new ArrayList<Mapping>();
        FeatureQuery q = new FeatureQuery();
        if (!singleSpecies) {
            try {
                q.setSpeciesID(species.getDBSpeciesID(this.getFactory().getDBVersion()));
            } catch (Exception e) {
                throw new DAOException("No species ID context for this query!", e);
            }
        }
        CoordinateSystem cs = null;

        if (region == null || region.getId()==null || region.getCoordSystem()== null ) {
            throw new DAOException("No valid region or coordinate system specified");
        } else {
            cs = region.getCoordSystem();
            q.setParentSequenceID(region.getId());
            q.setCoordinateSystemID(cs.getId());
        }

        if (coords != null) {
            q.setParentStart(coords.getStart());
            q.setParentStop(coords.getEnd());
        }

//        //what happens if we dont have a CS??
//        //do we want to try with the gene build CS is if there is one
//        //this is 'always' toplevel i.e. 1
//        //but the sequence might not be at this level
//        //so we would have to checke this before doing it
//        //probably safest to force a lazy load on sequence.getCoordSystem()
//        //i.e not do this....
//        else {
//            cs = this.geneBuildCS;
//            //q.setCoordinateSystemID(this.geneBuildCS.getId());
//        }

        //for maximum length
        //will need different method if constructed from CollectionDB

        Integer max = null;
        if (this.singleSpecies) {
                max= this.ssFactory.getDatabase().getMaxLengthForFeature(FeatureType.gene, this.geneBuildCS);
        } else {
                max= this.collFactory.getDatabase().getMaxLengthForFeature((DBCollectionSpecies) species, FeatureType.gene, this.geneBuildCS);
        }

        if (max != null) {
            q.setMaximumFeatureLength(max);
        }

        //out = this.getGenesOnParent(q, (DADNASequence) region);

        mappings = this.getMappingsOnParent(q, (DADNASequence) region);


        // this gets the desired genes filtered by stand if specified
        // probably should be done in the calling class

//        if (coords.getStrand() == null) {
//            for (Mapping m : mappings) {
//                out.add((DAGene) m.getSource());
//            }
//        } else {
//            for (Mapping m : mappings) {
//                if (m.getTargetCoordinates().getStrand().equals(coords.getStrand())) {
//                    out.add((DAGene) m.getSource());
//                }
//            }
//        }

        return mappings;
    }

   //*******************************

    private DAGene getGene(FeatureQuery query) throws DAOException {
        DAGene out = null;
        GeneMappingRowHandler handler = null;
        List<HashMap> result;

        SqlSession session = null;

        try {
            session = this.getFactory().getNewSqlSession();
            GeneMapper mapper = session.getMapper(GeneMapper.class);
            result = mapper.getGene(query);
        } catch (Exception e) {
            throw new DAOException("Failed to call getGene", e);
        } finally {
            if (session != null) {
                session.close();
            }
        }

        if (result != null && !result.isEmpty()) {

            handler = new GeneMappingRowHandler(result);
            handler.handleResult();
            out = handler.getObjectResult();
        }

        return out;
    }

    private DAGene getGeneOnParent(FeatureQuery query, DADNASequence parent) throws DAOException {
        DAGene out = null;
        GeneMappingRowHandler handler = null;
        List<HashMap> result;

        SqlSession session = null;

        try {
            session = this.getFactory().getNewSqlSession();
            GeneMapper mapper = session.getMapper(GeneMapper.class);
            result = mapper.getGene(query);
        } catch (Exception e) {
            throw new DAOException("Failed to call getGeneOnParent", e);
        } finally {
            if (session != null) {
                session.close();
            }
        }

        if (result != null && !result.isEmpty()) {

            handler = new GeneMappingRowHandler(result, parent);
            handler.handleResult();
            out = handler.getObjectResult();
        }

        return out;
    }

    private List<Mapping> getMappingsOnParent(FeatureQuery query, DADNASequence parent) throws DAOException {
        List<Mapping> out = null;

        MappingRowHandler handler = null;
        List<HashMap> result;

        SqlSession session = null;

        try {
            session = this.getFactory().getNewSqlSession();
            GeneMapper mapper = session.getMapper(GeneMapper.class);
            result = mapper.getGene(query);
        } catch (Exception e) {
            throw new DAOException("Failed to call getMappingsOnParent", e);
        } finally {
            if (session != null) {
                session.close();
            }
        }

        if (result != null && !result.isEmpty()) {

            handler = new MappingRowHandler(result, parent);
            handler.handleResult();
            out = handler.getListResult();
        }

        return out;
    }

    //*********************************

    // I haven't decided whether it is better to get theobjects
    //(with the mappings filled in)
    // or to get the mappings themselves


    //as an inner class it has access to the factory etc of the enclosing class
    public class GeneMappingRowHandler implements DBResultHandler {

        // the magic strings to be used as property keys for HashMap in Ibatis
        private final String gene = "gene";
        private final String xref = "xref";
        private final String target = "target";
        private final String coords = "coords";
        private final String coordSystemID = "csID";
        private DADNASequence parentSeq = null;
        private DAGene objectResult = null;
        private List<DAGene> listResult = new ArrayList<DAGene>();
        private List<HashMap> rawResults = null;

        public GeneMappingRowHandler(List<HashMap> results) {
            rawResults = results;
        }

        public GeneMappingRowHandler(List<HashMap> results, DADNASequence parent) {
            rawResults = results;
            parentSeq = parent;
        }

        @Override
        public List<DAGene> getListResult() {
            return listResult;
        }

        @Override
        public DAGene getObjectResult() {
            return objectResult;
        }

        public void handleResult() throws DAOException {

            if (rawResults == null || rawResults.isEmpty()) {
                return;
            }

            for (HashMap map : rawResults) {
                handleRow(map);
            }

        }

        private void handleRow(HashMap result) throws DAOException {

            if (result == null || result.isEmpty()) {
                return;
            }

            DAGene rowGene = null;
            if (parentSeq != null) {
                rowGene = this.parseResult(result, parentSeq);
            } else {
                rowGene = this.parseResult(result);
            }

            if (rowGene != null) {
                this.objectResult = rowGene;
                this.listResult.add(rowGene);
            }

        }

        private DAGene parseResult(HashMap result, DADNASequence parent) throws DAOException {

            if (result == null || parent == null) {
                return null;
            }

            DADNASequence pparentSeq = parent;

            DAGene pgene = null;
            DADNASequence ptarget = null;
            Coordinate pcoords = null;
            DAXRef pxref = null;
            Integer pcsID = null;

            pgene = (DAGene) result.get(this.gene);
            pcoords = (Coordinate) result.get(this.coords);
            ptarget = (DADNASequence) result.get(this.target);
            pcsID = (Integer) result.get(this.coordSystemID);
            pxref = (DAXRef) result.get(this.xref);

            //we only want genes on the parent sequence
            //this should never fail if we have done the sql correctly
            if (pgene == null || pcoords == null || ptarget == null || !ptarget.getId().equals(pparentSeq.getId())) {
                return null;
            }

            if (pgene.getDaoFactory() == null) {
                //pgene.setType(FeatureType.gene);
                pgene.setDaoFactory(daoFactory);
            }

            if (pgene.getDisplayXRef() == null) {
                pxref = (DAXRef) result.get(this.xref);
                if (pxref != null) {
                    pxref.setDaoFactory(daoFactory);
                    pgene.setDisplayXRef(pxref);
                }
            }

            //check if the gene object has somehow been returned from a cache and it
            //has a mapping to a seq region with the same id as the parentSeq: if
            //so, swap it for the parentSeq
            for (Mapping m : pgene.getMappings()) {
                if (m.getTarget().getId().equals(pparentSeq.getId())) {
                    if (pparentSeq.getDaoFactory() == null) {
                        pparentSeq.setDaoFactory(daoFactory);
                    }
                    if (pparentSeq.getCoordSystem() == null) {
                        if (singleSpecies) {
                            pparentSeq.setCoordSystem(ssFactory.getDatabase().getCSByID(pcsID));
                        } else {
                            pparentSeq.setCoordSystem(collFactory.getDatabase().getCSByID(species, pcsID));
                        }
                    }
                    //do the switch
                    m.setTarget(pparentSeq);
                    m.setTargetCoordinates(pcoords);
                    Mapping.addReverseMapping(m);
                    return pgene;
                }
            }


            Mapping mapping = new Mapping();
            mapping.setSource(pgene);
            mapping.setTargetCoordinates(pcoords);

            // at this point we could look to see if we have this sequence in cache

            if (pparentSeq.getDaoFactory() == null) {
                pparentSeq.setDaoFactory(daoFactory);
            }

            if (pparentSeq.getCoordSystem() == null) {
                 if (singleSpecies) {
                        pparentSeq.setCoordSystem(ssFactory.getDatabase().getCSByID(pcsID));
                 } else {
                        pparentSeq.setCoordSystem(collFactory.getDatabase().getCSByID(species, pcsID));
                 }
            }

            mapping.setTarget(pparentSeq);
            if(pgene.addMapping(mapping)) {
                Mapping.addReverseMapping(mapping);
            }
            return pgene;


        }

        private DAGene parseResult(HashMap result) throws DAOException {

            if (result == null || result.isEmpty()) {
                return null;
            }

            DAGene pgene = null;
            DADNASequence ptarget = null;
            Coordinate pcoords = null;
            DAXRef pxref = null;
            Integer pcsID = null;

            pgene = (DAGene) result.get(this.gene);

            if (pgene == null) {
                return null;
            }

            if (pgene.getDaoFactory() == null) {
               // pgene.setType(FeatureType.gene);
                pgene.setDaoFactory(daoFactory);
            }

            if (pgene.getDisplayXRef() == null) {
                pxref = (DAXRef) result.get(this.xref);
                if (pxref != null) {
                    pxref.setDaoFactory(daoFactory);
                    pgene.setDisplayXRef(pxref);
                }
            }

            pcoords = (Coordinate) result.get(this.coords);

            //later we will check to see if we already have this in cache
            //(at moment just for chromosomes)
            ptarget = (DADNASequence) result.get(this.target);

            pcsID = (Integer) result.get(this.coordSystemID);

            if (pcoords == null || ptarget == null) {
                return pgene;
            }

            Mapping mapping = new Mapping();
            mapping.setSource(pgene);
            mapping.setTargetCoordinates(pcoords);

            //we want to convert the DADNASequence to the correct type if it is an assembly!

            CoordinateSystem targetCS  = null;

            if (singleSpecies) {
                    targetCS = ssFactory.getDatabase().getCSByID(pcsID);
               } else {
                    targetCS =collFactory.getDatabase().getCSByID(species, pcsID);
               }

            if (targetCS.isSequenceLevel()) {
                ptarget.setCoordSystem(targetCS);
            } else if (targetCS.getType().equals(EnsemblCoordSystemType.chromosome)) {
                ptarget = new DAChromosome((DAOCoreFactory) daoFactory);
                ptarget.setId(((DADNASequence) result.get(this.target)).getId());
                ptarget.setSpecies(species);
                ptarget.setCoordSystem(targetCS);
                
                //loook to see if we have this sequence in cache
                ptarget = species.getCachedChromosome((DAChromosome) ptarget);

            } else {
                ptarget = new DAAssembledDNASequence((DAOCoreFactory) daoFactory);
                ptarget.setId(((DADNASequence) result.get(this.target)).getId());
                ptarget.setCoordSystem(targetCS);

                //not got a cache of assembled sequences yet to check
            }

            

            mapping.setTarget(ptarget);
            if (pgene.addMapping(mapping)) {
                Mapping.addReverseMapping(mapping);
            }
            return pgene;

        }
    }

    //as an inner class it has access to the factory etc of the enclosing class
    public class MappingRowHandler implements DBResultHandler {

        // the magic strings to be used as property keys for HashMap in Ibatis
        private final String gene = "gene";
        private final String xref = "xref";
        private final String target = "target";
        private final String coords = "coords";
        private final String coordSystemID = "csID";
        private DADNASequence parentSeq = null;
        private Mapping objectResult = null;
        private List<Mapping> listResult = new ArrayList<Mapping>();
        private List<HashMap> rawResults = null;

        public MappingRowHandler(List<HashMap> results) {
            rawResults = results;
        }

        public MappingRowHandler(List<HashMap> results, DADNASequence parent) {
            rawResults = results;
            parentSeq = parent;
        }

        @Override
        public List<Mapping> getListResult() {
            return listResult;
        }

        @Override
        public Mapping getObjectResult() {
            return objectResult;
        }


        public void handleResult() throws DAOException {

            if (rawResults == null || rawResults.isEmpty()) {
                return;
            }

            for (HashMap map : rawResults) {
                handleRow(map);
            }

        }

        private void handleRow(HashMap result) throws DAOException {

            if (result == null || result.isEmpty()) {
                return;
            }

            Mapping rowMapping = null;
            if (parentSeq != null) {
                rowMapping = this.parseMappingResult(result, parentSeq);
            } else {
                rowMapping = this.parseMappingResult(result);
            }

            if (rowMapping != null) {
                this.objectResult = rowMapping;
                this.listResult.add(rowMapping);
            }

        }



        private Mapping parseMappingResult(HashMap result, DADNASequence parent) throws DAOException {

            if (result == null || parent == null) {
                return null;
            }

            DADNASequence pparentSeq = parent;

            DAGene pgene = null;
            DADNASequence ptarget = null;
            Coordinate pcoords = null;
            DAXRef pxref = null;
            Integer pcsID = null;

            pgene = (DAGene) result.get(this.gene);
            pcoords = (Coordinate) result.get(this.coords);
            ptarget = (DADNASequence) result.get(this.target);
            pcsID = (Integer) result.get(this.coordSystemID);
            pxref = (DAXRef) result.get(this.xref);

            //we only want genes on the parent sequence
            //this should never fail if we have done the sql correctly
            if (pgene == null || pcoords == null || ptarget == null || !ptarget.getId().equals(pparentSeq.getId())) {
                return null;
            }

            if (pgene.getDaoFactory() == null) {
                //pgene.setType(FeatureType.gene);
                pgene.setDaoFactory(daoFactory);
            }

            if (pgene.getDisplayXRef() == null) {
                pxref = (DAXRef) result.get(this.xref);
                if (pxref != null) {
                    pxref.setDaoFactory(daoFactory);
                    pgene.setDisplayXRef(pxref);
                }
            }

            //check if the gene object has somehow been returned from a cache and it
            //has a mapping to a seq region with the same id as the parentSeq: if
            //so, swap it for the parentSeq
            for (Mapping m : pgene.getMappings()) {
                if (m.getTarget().getId().equals(pparentSeq.getId())) {
                    if (pparentSeq.getDaoFactory() == null) {
                        pparentSeq.setDaoFactory(daoFactory);
                    }
                    if (pparentSeq.getCoordSystem() == null) {
                        if (singleSpecies) {
                            pparentSeq.setCoordSystem(ssFactory.getDatabase().getCSByID(pcsID));
                        } else {
                            pparentSeq.setCoordSystem(collFactory.getDatabase().getCSByID(species, pcsID));
                        }
                    }

                    //do the switch
                    m.setTarget(pparentSeq);
                    m.setTargetCoordinates(pcoords);
                    Mapping.addReverseMapping(m);
                    return m;
                }
            }


            Mapping mapping = new Mapping();
            mapping.setSource(pgene);
            mapping.setTargetCoordinates(pcoords);

            // at this point we could look to see if we have this sequence in cache

            if (pparentSeq.getDaoFactory() == null) {
                pparentSeq.setDaoFactory(daoFactory);
            }

            if (pparentSeq.getCoordSystem() == null) {
            if (singleSpecies) {
                    pparentSeq.setCoordSystem(ssFactory.getDatabase().getCSByID(pcsID));
               } else {
                    pparentSeq.setCoordSystem(collFactory.getDatabase().getCSByID(species, pcsID));
               }
            }

            mapping.setTarget(pparentSeq);
            if (pgene.addMapping(mapping)) {
                Mapping.addReverseMapping(mapping);
            }
            return mapping;


        }

        private Mapping parseMappingResult(HashMap result) throws DAOException {

            if (result == null || result.isEmpty()) {
                return null;
            }

            DAGene pgene = null;
            DADNASequence ptarget = null;
            Coordinate pcoords = null;
            DAXRef pxref = null;
            Integer pcsID = null;

            pgene = (DAGene) result.get(this.gene);

            if (pgene == null) {
                return null;
            }

            if (pgene.getDaoFactory() == null) {
                //pgene.setType(FeatureType.gene);
                pgene.setDaoFactory(daoFactory);
            }

            if (pgene.getDisplayXRef() == null) {
                pxref = (DAXRef) result.get(this.xref);
                if (pxref != null) {
                    pxref.setDaoFactory(daoFactory);
                    pgene.setDisplayXRef(pxref);
                }
            }

            pcoords = (Coordinate) result.get(this.coords);
            ptarget = (DADNASequence) result.get(this.target);
            pcsID = (Integer) result.get(this.coordSystemID);

//            if (pcoords == null || ptarget == null) {
//                return pgene;
//            }

            //check to see we are not adding a duplicate mapping
            //to a rowGene retreived from cache, if we already have a mapping for
            //this id - don't add it again

//            for (Mapping m : pgene.getMappings()) {
//                if (m.getTarget().getId().equals(ptarget.getId())) {
//                    Mapping.addReverseMapping(m);
//                    return m;
//                }
//            }

            Mapping mapping = new Mapping();
            mapping.setSource(pgene);
            mapping.setTargetCoordinates(pcoords);

            // at this point we could look to see if we have this sequence in cache


            //we want to convert the DADNASequence to the correct type if it is an assembly!

            CoordinateSystem targetCS  = null;

            if (singleSpecies) {
                    targetCS = ssFactory.getDatabase().getCSByID(pcsID);
               } else {
                    targetCS =collFactory.getDatabase().getCSByID(species, pcsID);
               }

            if (targetCS.isSequenceLevel()) {
                ptarget.setCoordSystem(targetCS);
            } else if (targetCS.getType().equals(EnsemblCoordSystemType.chromosome)) {
                ptarget = new DAChromosome();
                ptarget.setId(((DADNASequence) result.get(this.target)).getId());
                ptarget.setCoordSystem(targetCS);
            } else {
                ptarget = new DAAssembledDNASequence();
                ptarget.setId(((DADNASequence) result.get(this.target)).getId());
                ptarget.setCoordSystem(targetCS);
            }

            ptarget.setDaoFactory(daoFactory);

            mapping.setTarget(ptarget);
            if (pgene.addMapping(mapping)) {
                Mapping.addReverseMapping(mapping);
            }
            return mapping;

        }


    }


}
