/**
 * Copyright (C) 2010 The Roslin Institute <contact andy.law@roslin.ed.ac.uk>
 *
 * This file is part of the Ensembl Java API demonstration project developed by the
 * Bioinformatics Group at The Roslin Institute, The Royal (Dick) School of
 * Veterinary Studies, University of Edinburgh.
 *
 * This is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License (version 3) as published by
 * the Free Software Foundation.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * in this software distribution. If not, see <http://www.gnu.org/licenses/gpl-3.0.html/>.
 */

package uk.ac.roslin.ensembl.dao.database.coreaccess;

import uk.ac.roslin.ensembl.exception.DAOException;
import java.util.ArrayList;
import java.util.List;
import org.apache.ibatis.session.SqlSession;
import uk.ac.roslin.ensembl.dao.coreaccess.ProteinFeatureDAO;
import uk.ac.roslin.ensembl.datasourceaware.core.DAProteinFeature;
import uk.ac.roslin.ensembl.dao.database.factory.DBDAOCollectionCoreFactory;
import uk.ac.roslin.ensembl.dao.database.factory.DBDAOSingleSpeciesCoreFactory;
import uk.ac.roslin.ensembl.dao.factory.DAOCollectionCoreFactory;
import uk.ac.roslin.ensembl.dao.factory.DAOSingleSpeciesCoreFactory;
import uk.ac.roslin.ensembl.mapper.core.ProteinFeatureMapper;


public class DBProteinFeatureDAO extends DBCoreObjectDAO implements ProteinFeatureDAO {

    public DBProteinFeatureDAO() {
        super();
    }

    public DBProteinFeatureDAO(DAOSingleSpeciesCoreFactory factory) {
        super(factory);
    }

    public DBProteinFeatureDAO(DAOCollectionCoreFactory factory)  {
        super(factory);
    }

    @Override
    public List<DAProteinFeature> getProteinFeaturesByTranslationID(int translationID) throws DAOException {

        List<DAProteinFeature> outList = new ArrayList<DAProteinFeature>();

        if ( translationID==0) {
            return outList;
        }

        

        SqlSession session = null;

        try {
            session = this.getFactory().getNewSqlSession();
            ProteinFeatureMapper mapper = session.getMapper(ProteinFeatureMapper.class);
            outList = mapper.getProteinFeaturesByTranslationID(translationID);
        } catch (Exception e) {
            throw new DAOException("Failed to call getProteinFeaturesByTranslationID", e);
        } finally {
            if (session != null) {
                session.close();
            }
        }

        if (outList != null && ! outList.isEmpty()) {
            for (DAProteinFeature pf : outList) {
                    pf.setDaoFactory(daoFactory);
            }
        }
        return outList;
    }

    
}
