/**
 * Copyright (C) 2010 The Roslin Institute <contact andy.law@roslin.ed.ac.uk>
 *
 * This file is part of the Ensembl Java API demonstration project developed by the
 * Bioinformatics Group at The Roslin Institute, The Royal (Dick) School of
 * Veterinary Studies, University of Edinburgh.
 *
 * This is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License (version 3) as published by
 * the Free Software Foundation.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * in this software distribution. If not, see <http://www.gnu.org/licenses/gpl-3.0.html/>.
 */

package uk.ac.roslin.ensembl.dao.database.coreaccess;

import java.util.HashMap;
import org.apache.ibatis.session.SqlSession;
import uk.ac.roslin.ensembl.dao.coreaccess.TranslationDAO;
import uk.ac.roslin.ensembl.datasourceaware.core.DATranslation;
import uk.ac.roslin.ensembl.dao.database.factory.DBDAOCollectionCoreFactory;
import uk.ac.roslin.ensembl.dao.database.factory.DBDAOSingleSpeciesCoreFactory;
import uk.ac.roslin.ensembl.dao.factory.DAOCollectionCoreFactory;
import uk.ac.roslin.ensembl.dao.factory.DAOSingleSpeciesCoreFactory;
import uk.ac.roslin.ensembl.mapper.core.TranslationMapper;
import uk.ac.roslin.ensembl.exception.DAOException;
import uk.ac.roslin.ensembl.model.core.Translation;

/**
 *
 * @author tpaterso
 */
public class DBTranslationDAO extends DBCoreObjectDAO implements TranslationDAO  {

    public DBTranslationDAO()  {
        super();
    }

    public DBTranslationDAO(DAOSingleSpeciesCoreFactory factory) {

        super(factory);
    }

    public DBTranslationDAO(DAOCollectionCoreFactory factory) {
        super(factory);

    }

    @Override
    public DATranslation getTranslationByTranscriptID(Integer id) throws DAOException {

        if (id == null) {
            return null;
        }

        DATranslation out = null;


        HashMap parameters = new HashMap();
        parameters.put("transcriptID", id);

        SqlSession session = null;

        try {
            session = this.getFactory().getNewSqlSession();
            TranslationMapper mapper = session.getMapper(TranslationMapper.class);
            out = mapper.getTranslationByTranscriptID(parameters);
        } catch (Exception e) {
            throw new DAOException("Failed to call getTranslationByTranscriptID", e);
        } finally {
            if (session != null) {
                session.close();
            }
        }

        if (out != null) {
            out.setDaoFactory(daoFactory);
            out.setSpecies(species);
        }

        return out;
    }

}
