/**
 * Copyright (C) 2010 The Roslin Institute <contact andy.law@roslin.ed.ac.uk>
 *
 * This file is part of the Ensembl Java API demonstration project developed by the
 * Bioinformatics Group at The Roslin Institute, The Royal (Dick) School of
 * Veterinary Studies, University of Edinburgh.
 *
 * This is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License (version 3) as published by
 * the Free Software Foundation.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * in this software distribution. If not, see <http://www.gnu.org/licenses/gpl-3.0.html/>.
 */

package uk.ac.roslin.ensembl.dao.database.factory;

import java.util.HashMap;
import uk.ac.roslin.ensembl.config.EnsemblComparaDivision;
import uk.ac.roslin.ensembl.dao.coreaccess.AssemblyDAO;
import uk.ac.roslin.ensembl.dao.coreaccess.ProteinFeatureDAO;
import uk.ac.roslin.ensembl.dao.coreaccess.DNASequenceDAO;
import uk.ac.roslin.ensembl.dao.coreaccess.TranslationDAO;
import uk.ac.roslin.ensembl.dao.database.DBCollectionCoreDatabase;



//not sure i want this separate hierarchy of DAOs to handle the Species requiremnet
import uk.ac.roslin.ensembl.dao.database.DBComparisonDatabase;
import uk.ac.roslin.ensembl.dao.database.coreaccess.DBAssemblyDAO;
import uk.ac.roslin.ensembl.dao.database.coreaccess.DBChromosomeDAO;
import uk.ac.roslin.ensembl.dao.database.coreaccess.DBCoordinateSystemDAO;
import uk.ac.roslin.ensembl.dao.database.coreaccess.DBGeneDAO;
import uk.ac.roslin.ensembl.dao.database.coreaccess.DBDNASequenceDAO;
import uk.ac.roslin.ensembl.dao.database.coreaccess.DBExonDAO;
import uk.ac.roslin.ensembl.dao.database.coreaccess.DBProteinFeatureDAO;
import uk.ac.roslin.ensembl.dao.database.coreaccess.DBTranscriptDAO;
import uk.ac.roslin.ensembl.dao.database.coreaccess.DBTranslationDAO;
import uk.ac.roslin.ensembl.dao.factory.DAOCollectionCoreFactory;
import uk.ac.roslin.ensembl.dao.factory.DAOCoreFactory;
import uk.ac.roslin.ensembl.exception.DAOException;
import uk.ac.roslin.ensembl.model.core.CollectionSpecies;
import uk.ac.roslin.ensembl.model.database.CollectionCoreDatabase;
import uk.ac.roslin.ensembl.model.database.CollectionDatabase;


public class DBDAOCollectionCoreFactory  extends  DBDAOCollectionFactory implements DAOCollectionCoreFactory {

    private HashMap<String, DBDAOComparaFactory> comparaFactories =
            new HashMap<String, DBDAOComparaFactory>();

    public DBDAOCollectionCoreFactory() {
        super();
    }


     public DBDAOCollectionCoreFactory(CollectionDatabase database, CollectionSpecies species)  throws DAOException {
        super(database,species);
    }

    @Override
    public CollectionCoreDatabase getDatabase() {
        return (CollectionCoreDatabase) this.database;
    }


    @Override
    public DBCoordinateSystemDAO getCoordinateSystemDAO() throws DAOException {
        return new DBCoordinateSystemDAO(this);
    }


    @Override
    public DBChromosomeDAO getChromosomeDAO()  throws DAOException{
        return new DBChromosomeDAO(this);
    }

    @Override
    public DBGeneDAO getGeneDAO()  throws DAOException{
        try {
            return new DBGeneDAO(this);
        } catch (Exception ex) {
            return null;
        }
    }

    @Override
    public DBProteinFeatureDAO getProteinFeatureDAO()  throws DAOException{
        return new DBProteinFeatureDAO(this);
    }


    @Override
    public DBAssemblyDAO getAssemblyDAO()  throws DAOException{
        return new DBAssemblyDAO(this);
    }

    @Override
    public DBDNASequenceDAO getSequenceDAO()  throws DAOException{
        return new DBDNASequenceDAO(this);
    }

    @Override
    public DBTranslationDAO getTranslationDAO()  throws DAOException{
        return new DBTranslationDAO(this);
    }

    @Override
    public DBTranscriptDAO getTranscriptDAO()  throws DAOException{
        return new DBTranscriptDAO(this);
    }

    @Override
    public DBExonDAO getExonDAO()  throws DAOException{
        return new DBExonDAO(this);
    }

    @Override
    public DBDAOComparaFactory getComparaFactory(EnsemblComparaDivision comparaDivision) {
       // throw new UnsupportedOperationException("Not supported yet.");

        if (comparaDivision == null) {
             comparaDivision = EnsemblComparaDivision.MULTI;
        }
        
        DBDAOComparaFactory out = null;

        if (this.comparaFactories.containsKey(comparaDivision.toString())) {
            return this.comparaFactories.get(comparaDivision.toString());
        }

        //so far we only have collections for bacteria
        //so default to the bacterial compara here
        
        DBComparisonDatabase db = null;

        if (this.getRegistry()!=null) {
            db= (DBComparisonDatabase) this.getRegistry().getComparaDatabase(comparaDivision != null ? comparaDivision : EnsemblComparaDivision.BACTERIA ,
                dbVersion != null ? dbVersion : String.valueOf(this.getRegistry().getHighestDBVersion()));
        }

        if (db != null) {
            out = db.getComparaFactory();
        }

        this.comparaFactories.put(comparaDivision.toString(), out);

        return out;


    }

}
