/**
 * Copyright (C) 2010 The Roslin Institute <contact andy.law@roslin.ed.ac.uk>
 *
 * This file is part of the Ensembl Java API demonstration project developed by the
 * Bioinformatics Group at The Roslin Institute, The Royal (Dick) School of
 * Veterinary Studies, University of Edinburgh.
 *
 * This is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License (version 3) as published by
 * the Free Software Foundation.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * in this software distribution. If not, see <http://www.gnu.org/licenses/gpl-3.0.html/>.
 */

package uk.ac.roslin.ensembl.dao.database.factory;


import uk.ac.roslin.ensembl.exception.DAOException;
import uk.ac.roslin.ensembl.exception.ConfigurationException;
import uk.ac.roslin.ensembl.exception.EnsemblException;
import uk.ac.roslin.ensembl.dao.database.DBCollectionDatabase;
import uk.ac.roslin.ensembl.dao.database.DBCollectionSpecies;
import uk.ac.roslin.ensembl.dao.factory.DAOCollectionFactory;
import uk.ac.roslin.ensembl.config.EnsemblDBType;
import uk.ac.roslin.ensembl.model.core.CollectionSpecies;
import uk.ac.roslin.ensembl.model.database.CollectionDatabase;
import uk.ac.roslin.ensembl.model.database.DatabaseType;


public abstract class DBDAOCollectionFactory  extends DBDAOSpeciesFactory implements DAOCollectionFactory {



    public DBDAOCollectionFactory()   {
        super();
    }

    public DBDAOCollectionFactory(CollectionDatabase database, CollectionSpecies species)  throws DAOException {

        super(database, species);

    }

    @Override
    public DBCollectionSpecies getSpecies() {
        return (DBCollectionSpecies) species;
    }

    public static DBDAOCollectionFactory makeFactory(CollectionDatabase database, CollectionSpecies species)
        throws EnsemblException {

            DatabaseType type =  database.getType();

            if (type.equals(EnsemblDBType.collection_core)) {
                return new DBDAOCollectionCoreFactory(database, species);
            }
            else if (type.equals(EnsemblDBType.collection_funcgen)) {
                return new DBDAOCollectionFuncgenFactory(database, species);
            }
            else if (type.equals(EnsemblDBType.collection_variation)) {
                return new DBDAOCollectionVariationFactory(database, species);
            }
            else {
                throw new ConfigurationException("The Factory could not make a DBDAOCollectionFactory of EnsemblDBType: "+type.toString());
            }

    }

    @Override
    public CollectionDatabase getDatabase() {
        return (CollectionDatabase) this.database;
    }
}
