/**
 * Copyright (C) 2010 The Roslin Institute <contact andy.law@roslin.ed.ac.uk>
 *
 * This file is part of the Ensembl Java API demonstration project developed by the
 * Bioinformatics Group at The Roslin Institute, The Royal (Dick) School of
 * Veterinary Studies, University of Edinburgh.
 *
 * This is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License (version 3) as published by
 * the Free Software Foundation.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * in this software distribution. If not, see <http://www.gnu.org/licenses/gpl-3.0.html/>.
 */

package uk.ac.roslin.ensembl.dao.database.factory;

import uk.ac.roslin.ensembl.model.database.DatabaseType;
import uk.ac.roslin.ensembl.exception.DAOException;
import java.util.Properties;
import java.io.InputStreamReader;
import java.io.Reader;
import org.apache.ibatis.session.SqlSession;
import org.apache.ibatis.session.SqlSessionFactory;
import org.apache.ibatis.session.SqlSessionFactoryBuilder;
import uk.ac.roslin.ensembl.dao.factory.DAOFactory;
import uk.ac.roslin.ensembl.model.database.Database;
import uk.ac.roslin.ensembl.model.database.Registry;

public abstract class DBDAOFactory implements DAOFactory {

    protected Properties configuration;
    protected String databaseType;
    protected String ensemblSchema;
    protected String dbVersion;
    protected String schemaVersion;
    protected String ibatisSchemaFilePath;
    protected String mybatisSchemaFilePath;
    protected String databaseName;
    protected Registry registry;
    protected String thisDBUrl;

    protected DatabaseType dBType;
    protected Database database;

    protected SqlSessionFactory sqlSessionFactory = null;

    public DBDAOFactory() {
    }

    public DBDAOFactory(Database database)  throws DAOException {
        this.setDatabase(database);
        //to reduce overhead only make this when first need it
        //sqlSessionFactory = createSessionFactory();
    }

    public final void setDatabase(Database db) throws DAOException {
        try {
            this.database = db;
            this.dbVersion = this.database.getDBVersion();
            this.setDatabaseName(this.database.getdBName());
            this.registry = database.getRegistry();
            this.setDBType(database.getType());
            this.configuration = (Properties) registry.getConfigProperties().clone();
            this.setEnsemblSchemaVersion(database.getSchemaVersion());
            this.setMybatisSchemaFilePath(registry.findMybatisSchemaForSchemaVersion(this.dBType, this.ensemblSchema));
            thisDBUrl = configuration.getProperty("url").concat("/").concat(database.getdBName() + "?zeroDateTimeBehavior=convertToNull");
            configuration.setProperty("url", thisDBUrl);
            configuration.setProperty("ensembl_release", ensemblSchema);
            configuration.setProperty("mybatis_file", mybatisSchemaFilePath);
        } catch (Exception e) {
            throw new DAOException("Fail to set Database on a DBDAOFactory", e);
        }
    }


    @Override
    public Properties getConfiguration() {
        return configuration;
    }

    @Override
    public String getEnsemblSchemaVersion() {
        return ensemblSchema;
    }

    @Override
    public String getDBVersion() {
        return dbVersion;
    }

    @Override
    public void setEnsemblSchemaVersion(String schemaVersion) {
        this.ensemblSchema = schemaVersion;
    }

    @Override
    public void setDBVersion(String dbversion) {
        this.dbVersion = dbversion;
    }

    @Override
    public Registry getRegistry() {
        return registry;
    }

    @Override
    public void setDBType(DatabaseType type) {
        dBType = type;
    }

    @Override
    public DatabaseType getDBType() {
        return dBType;
    }


    @Override
    public void setMybatisSchemaFilePath(String schema) {
        this.mybatisSchemaFilePath = schema;
    }

    @Override
    public String getMybatisSchemaFilePath() {
        return this.mybatisSchemaFilePath;
    }

    @Override
    public String getDatabaseName() {
        return this.databaseName;
    }

    @Override
    public void setDatabaseName(String database) {
        this.databaseName = database;
    }

    @Override
    public Database getDatabase() {
        return this.database;
    }

    //*************************************************************************

    @Override
    public SqlSession getNewSqlSession() throws DAOException {
        if (database == null) {
            throw new DAOException("No Database set for Factory");
        }
        return (SqlSession) database.getNewSqlSession();
    }

//    private SqlSessionFactory getSessionFactory() throws DAOException {
//
//        if (sqlSessionFactory == null) {
//            sqlSessionFactory = createSessionFactory();
//        }
//        return sqlSessionFactory;
//    }
//
//    @Override
//    public SqlSession getNewSqlSession() throws DAOException {
//        try {
//            return getSessionFactory().openSession();
//        } catch (Exception e) {
//            throw new DAOException("Failed to retrieve an SqlSession from the Factory", e);
//        }
//    }
//
//    private SqlSessionFactory createSessionFactory() throws DAOException {
//
//          SqlSessionFactory out = null;
//          Reader reader = null;
//
//
//           try {
//
//            String  resource = this.mybatisSchemaFilePath;
//            reader = new InputStreamReader(this.getClass().getClassLoader().getResourceAsStream(resource));
//
//            if (reader == null) {
//                /* "belt and braces" check, as nothing is impossible */
//                throw new DAOException("Failed to read Configuration.xml");
//            }
//
//            out = (new SqlSessionFactoryBuilder()).build(reader, "current", configuration);
//
//
//            if (out == null) {
//                /* also unlikely, but again a "belt and braces" check */
//                throw new DAOException("Failed to build SqlMapClient");
//            }
//
//              reader.close();
//
//        } catch (Exception e) {
//
//            throw new DAOException(e);
//        }
//
//
//          return out;
//   }

}
