/**
 * Copyright (C) 2010 The Roslin Institute <contact andy.law@roslin.ed.ac.uk>
 *
 * This file is part of the Ensembl Java API demonstration project developed by the
 * Bioinformatics Group at The Roslin Institute, The Royal (Dick) School of
 * Veterinary Studies, University of Edinburgh.
 *
 * This is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License (version 3) as published by
 * the Free Software Foundation.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * in this software distribution. If not, see <http://www.gnu.org/licenses/gpl-3.0.html/>.
 */

package uk.ac.roslin.ensembl.dao.database.factory;

import uk.ac.roslin.ensembl.exception.DAOException;
import uk.ac.roslin.ensembl.exception.ConfigurationException;
import uk.ac.roslin.ensembl.exception.EnsemblException;
import uk.ac.roslin.ensembl.dao.factory.DAOSingleSpeciesFactory;
import uk.ac.roslin.ensembl.config.EnsemblDBType;
import uk.ac.roslin.ensembl.model.database.DatabaseType;
import uk.ac.roslin.ensembl.model.database.SingleSpeciesDatabase;


public abstract class DBDAOSingleSpeciesFactory  extends DBDAOSpeciesFactory implements DAOSingleSpeciesFactory {

    protected String genus_species;

    public DBDAOSingleSpeciesFactory() {
        super();
    }
    public DBDAOSingleSpeciesFactory(SingleSpeciesDatabase database)  throws DAOException {
        super(database);
        this.setGenus_Species(database.getDbSpeciesName());
    }


    @Override
    public void setGenus_Species(String genus_species) {
        this.genus_species = genus_species;
    }

    @Override
    public String getGenus_Species() {
        return genus_species;
    }

    public static DBDAOSingleSpeciesFactory makeFactory(SingleSpeciesDatabase database) throws EnsemblException {

         DatabaseType type =  database.getType();

            if (type.equals(EnsemblDBType.core)) {
                return new DBDAOSingleSpeciesCoreFactory(database);
            }
            else if (type.equals(EnsemblDBType.funcgen)) {
                return new DBDAOSingleSpeciesFuncgenFactory(database);
            }
            else if (type.equals(EnsemblDBType.variation)) {
                return new DBDAOSingleSpeciesVariationFactory(database);
            }
            else {
                throw new ConfigurationException("The Factory could not make a DBDAOSingleSpeciesFactory of EnsemblDBType: "+type.toString());
            }

     }

    @Override
    public SingleSpeciesDatabase getDatabase() {
        return (SingleSpeciesDatabase) this.database;
    }
}
