/**
 * Copyright (C) 2010 The Roslin Institute <contact andy.law@roslin.ed.ac.uk>
 *
 * This file is part of the Ensembl Java API demonstration project developed by the
 * Bioinformatics Group at The Roslin Institute, The Royal (Dick) School of
 * Veterinary Studies, University of Edinburgh.
 *
 * This is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License (version 3) as published by
 * the Free Software Foundation.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * in this software distribution. If not, see <http://www.gnu.org/licenses/gpl-3.0.html/>.
 */

package uk.ac.roslin.ensembl.datasourceaware;

import java.io.Serializable;
import java.util.Comparator;
import uk.ac.roslin.ensembl.model.database.Registry;
import uk.ac.roslin.ensembl.dao.factory.DAOFactory;
import uk.ac.roslin.ensembl.model.IdentifiableObject;


/**
 * @author paterson
 */
public abstract class DAObject implements IdentifiableObject {


    protected DAOFactory daoFactory = null;
    protected String schemaVersion = null;
    protected String dbVersion = null;
    protected Registry registry = null;
    protected Integer id = null;

    public DAObject() {

    }

    public DAObject(DAOFactory factory) {
        this.setDaoFactory(factory);
    }

    public DAOFactory getDaoFactory() {
        return daoFactory;
    }

    public void setDaoFactory(DAOFactory daoFactory) {
        if (daoFactory != null) {
            this.daoFactory = daoFactory;
            this.setSchemaVersion(daoFactory.getEnsemblSchemaVersion());
            this.setDBVersion(daoFactory.getDBVersion());
            this.setRegistry(daoFactory.getRegistry());
        }
    }

    public String getSchemaVersion() {
            return schemaVersion;
    }

    private void setSchemaVersion(String eSchemaVersion) {
        this.schemaVersion = eSchemaVersion;
    }

    public String getDBVersion() {
            return dbVersion;
    }

    private void setDBVersion(String dbversion) {
        this.dbVersion = dbversion;
    }

    public Registry getRegistry() {
            return registry;
    }

    public void setRegistry(Registry datasource) {
        this.registry = datasource;
    }

    public Integer getId() {
        return id;
    }

    public void setId(Integer id) {
        this.id = id;
    }


    public String getHashID() {
        return
                ((this.getDaoFactory()!=null) ? this.getDaoFactory().getDatabaseName() : "NODATABASE")

                +"_"

                +((this.getType()!=null) ? this.getType().toString() : "NOTYPE")

                +"_"

                +((this.getId()!=null) ? this.getId().toString() : "NOID");
    }


    public static final class DAComparator implements Comparator<DAObject>, Serializable {

        public int compare(DAObject o1, DAObject o2) {

            if (o1 != null && o2 != null) {
                    return ((DAObject)o1 ).getHashID().compareTo(((DAObject)o2).getHashID());
                }

            return 0;
        }

    }

    public static final DAComparator daComparator = new DAComparator();

}
