/**
 * Copyright (C) 2010 The Roslin Institute <contact andy.law@roslin.ed.ac.uk>
 *
 * This file is part of the Ensembl Java API demonstration project developed by the
 * Bioinformatics Group at The Roslin Institute, The Royal (Dick) School of
 * Veterinary Studies, University of Edinburgh.
 *
 * This is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License (version 3) as published by
 * the Free Software Foundation.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * in this software distribution. If not, see <http://www.gnu.org/licenses/gpl-3.0.html/>.
 */
package uk.ac.roslin.ensembl.datasourceaware.compara;

import java.util.ArrayList;
import java.util.List;
import java.util.TreeSet;
import uk.ac.roslin.ensembl.dao.factory.DAOComparaFactory;
import uk.ac.roslin.ensembl.datasourceaware.DAObject;
import uk.ac.roslin.ensembl.datasourceaware.core.DAGene;
import uk.ac.roslin.ensembl.model.compara.HomologyAlignmentProperties;
import uk.ac.roslin.ensembl.model.compara.HomologyPairRelationship;
import uk.ac.roslin.ensembl.model.core.Species;

public  class DAHomologyPairRelationship extends DAHomologyRelationship 
        implements HomologyPairRelationship<DAGene> {

    public DAHomologyPairRelationship() {
        super();
    }

//    public DAHomologyPairRelationship(DAOComparaFactory factory) {
//        super(factory);
//    }

    DAGene source = null;
    DAGene target = null;

    /**
     * Because a DAHomologyPairRelationship is tyeically instantiated from the
     * compara databases - the target gene in particular is likely to not
     * be a core-datasource aware object, and therefore species & and mapping details
     * are only simple fields grabbed from compara: these are held in properties
     * objects
     */
    HomologyAlignmentProperties sourceProperties = null;
    HomologyAlignmentProperties targetProperties = null;

    @Override
    public TreeSet<DAGene> getMembers() {
        TreeSet<DAGene> genes = new TreeSet<DAGene>(DAObject.daComparator);
        if (this.source != null ) {
            genes.add(this.source);
        }
        if (this.target != null ) {
            genes.add(this.target);
        }
        return genes;
    }

    @Override
    public List<Species> getSpecies() {
        List<Species> spp = new ArrayList<Species>( );
        if (this.source != null && this.source.getSpecies()!= null) {
            spp.add(this.source.getSpecies());
        }
        if (this.target != null && this.target.getSpecies() != null
                && ! spp.contains(this.target.getSpecies())
                ) {
            spp.add(this.target.getSpecies());
        }
        return spp;
    }

    public HomologyAlignmentProperties getSourceProperties() {
        return this.sourceProperties;
    }

    public HomologyAlignmentProperties getTargetProperties() {
        return this.targetProperties;
    }

    public void setSourceProperties(HomologyAlignmentProperties sourceProperties) {
        this.sourceProperties = sourceProperties;
    }

    public void setTargetProperties(HomologyAlignmentProperties targetProperties) {
        this.targetProperties = targetProperties;
    }

    public void setSource(DAGene source) {
        this.source = source;
    }

    public void setTarget(DAGene target) {
        this.target = target;
    }

    public DAGene getSource() {
        return this.source;
    }

    public DAGene getTarget() {
        return this.target;
    }

}
