/**
 * Copyright (C) 2010 The Roslin Institute <contact andy.law@roslin.ed.ac.uk>
 *
 * This file is part of the Ensembl Java API demonstration project developed by the
 * Bioinformatics Group at The Roslin Institute, The Royal (Dick) School of
 * Veterinary Studies, University of Edinburgh.
 *
 * This is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License (version 3) as published by
 * the Free Software Foundation.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * in this software distribution. If not, see <http://www.gnu.org/licenses/gpl-3.0.html/>.
 */
package uk.ac.roslin.ensembl.datasourceaware.compara;

import java.util.List;
import java.util.TreeSet;
import uk.ac.roslin.ensembl.datasourceaware.core.DAGene;
import uk.ac.roslin.ensembl.model.compara.HomologyAlignmentProperties;
import uk.ac.roslin.ensembl.model.compara.HomologyType;
import uk.ac.roslin.ensembl.model.core.Species;
import uk.ac.roslin.ensembl.model.relationship.InverseBinaryRelationshipView;

public  class InverseHomologyPairRelationshipView
        extends DAHomologyPairRelationship
        implements InverseBinaryRelationshipView<DAHomologyPairRelationship> {

    DAHomologyPairRelationship originalHPR;

    public InverseHomologyPairRelationshipView(DAHomologyPairRelationship original) {
        originalHPR = original;
    }

    @Override
    public HomologyAlignmentProperties getSourceProperties() {
        return this.originalHPR.getTargetProperties();
    }

    @Override
    public HomologyAlignmentProperties getTargetProperties() {
        return this.originalHPR.getSourceProperties();
    }

    @Override
    public DAGene getSource() {
        return this.originalHPR.getTarget();
    }

    @Override
    public DAGene getTarget() {
        return this.originalHPR.getSource();
    }

    @Override
    public DAHomologyPairRelationship getBinaryRelationship() {
        return this.originalHPR;
    }

    @Override
    public TreeSet<DAGene> getMembers() {
        return this.originalHPR.getMembers();
    }

    @Override
    public List<Species> getSpecies() {
        return this.originalHPR.getSpecies();
    }

    @Override
    public HomologyType getType() {
        return this.originalHPR.getType();
    }

    @Override
    public HomologyType getRelationshipType() {
        return this.originalHPR.getRelationshipType();
    }

    @Override
    public String getLastCommonAncestor() {
        return this.originalHPR.getLastCommonAncestor();
    }
}
