/**
 * Copyright (C) 2010 The Roslin Institute <contact andy.law@roslin.ed.ac.uk>
 *
 * This file is part of the Ensembl Java API demonstration project developed by the
 * Bioinformatics Group at The Roslin Institute, The Royal (Dick) School of
 * Veterinary Studies, University of Edinburgh.
 *
 * This is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License (version 3) as published by
 * the Free Software Foundation.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * in this software distribution. If not, see <http://www.gnu.org/licenses/gpl-3.0.html/>.
 */

package uk.ac.roslin.ensembl.datasourceaware.core;

import org.biojava3.core.sequence.Strand;
import uk.ac.roslin.ensembl.dao.factory.DAOCoreFactory;
import uk.ac.roslin.ensembl.exception.DAOException;
import uk.ac.roslin.ensembl.exception.RangeException;
import uk.ac.roslin.ensembl.model.core.AssembledDNASequence;

/**
 *
 * @author paterson
 */
public class DAAssembledDNASequence extends DADNASequence implements AssembledDNASequence {

    DAAssembly completeAssembly = null;

    public DAAssembledDNASequence() {
        super();
    }

    public DAAssembledDNASequence(DAOCoreFactory factory) {
        super(factory);
    }

    public DAAssembly getCompleteAssembly() {
        if (completeAssembly == null) {
            completeAssembly = DAAssembly.getAssembly(this);
        }
        return completeAssembly;
    }

     /**
     *
     * @param begin
     * @param end
     * @return
     */
    @Override
    public String getSequenceAsString(Integer begin, Integer end) {
        return this.getSequenceAsString(begin, end, Strand.POSITIVE);
    }
     /**
     *
     * @param begin
     * @param end
     * @param strand
     * @return
     */
    @Override
    public String getSequenceAsString(Integer begin, Integer end, Strand strand) {

        String out = "";
        
        try {
            out = this.getCompleteAssembly().getSequenceAsString(begin, end, strand).toString();
        } catch (RangeException ex) {

        } catch (DAOException ex) {

        }

        return out;
    }

    /**
     * Default case is to assume strand is positive because only CDSSequence can be either positive or negative Strand.
     * @return
     */
    @Override
    public String getSequenceAsString() {
        if (this.getBioEnd()==0) {
            return "";
        }
        return getSequenceAsString(this.getBioBegin(), this.getBioEnd(), Strand.POSITIVE);
    }

     /**
     *
     * @param begin
     * @param end
     * @param strand
     * @return
     */
    @Override
    public String getReverseComplementSequenceAsString(Integer begin, Integer end) {

        String out = "";

        try {
            out = this.getCompleteAssembly().getReverseComplementSequenceAsString(begin, end).toString();
        } catch (RangeException ex) {

        } catch (DAOException ex) {

        }

        return out;
    }

    /**
     * Default case is to assume strand is positive because only CDSSequence can be either positive or negative Strand.
     * @return
     */
    @Override
    public String getReverseComplementSequenceAsString() {
        if (this.getBioEnd()==0) {
            return "";
        }
        return getReverseComplementSequenceAsString(this.getBioBegin(), this.getBioEnd());
    }


}
