/**
 * Copyright (C) 2010 The Roslin Institute <contact andy.law@roslin.ed.ac.uk>
 *
 * This file is part of the Ensembl Java API demonstration project developed by the
 * Bioinformatics Group at The Roslin Institute, The Royal (Dick) School of
 * Veterinary Studies, University of Edinburgh.
 *
 * This is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License (version 3) as published by
 * the Free Software Foundation.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * in this software distribution. If not, see <http://www.gnu.org/licenses/gpl-3.0.html/>.
 */

package uk.ac.roslin.ensembl.datasourceaware.core;

import java.util.HashMap;
import java.util.List;
import uk.ac.roslin.ensembl.dao.compara.HomologyDAO;
import uk.ac.roslin.ensembl.dao.factory.DAOCoreFactory;
import uk.ac.roslin.ensembl.exception.DAOException;
import uk.ac.roslin.ensembl.model.Coordinate;
import uk.ac.roslin.ensembl.model.MappingSet;
import uk.ac.roslin.ensembl.model.compara.HomologyPairRelationship;
import uk.ac.roslin.ensembl.model.core.Chromosome;
import uk.ac.roslin.ensembl.model.core.Species;

/**
 *
 * @author paterson
 */
public class DAChromosome extends DAAssembledDNASequence implements Chromosome{
    
    public DAChromosome() {
        super();
    }

    public DAChromosome(DAOCoreFactory factory) {
        super(factory);
    }

    public String getChromosomeName() {
        return this.getName();
    }

    public void setChromosomeName(String name) {
        this.setName(name);
    }

    @Override
    public String toString() {
        return this.getChromosomeName();
    }

    @Override
    public HashMap<DADNASequence, MappingSet> getRegionsOfConservedSynteny(Coordinate range,
            Species target) throws DAOException {

        if (target==null) {
            throw new DAOException("No Sspecies specified for getRegionsOfConservedSynteny");
        }


        try {
        HashMap<DADNASequence, MappingSet> syntenies = null;
        
        HomologyDAO dao =
                this.getComparaFactory().getHomologyDAO();

        syntenies = (HashMap<DADNASequence, MappingSet>) dao.getRegionsOfConservedSynteny(this, range, target);

        return syntenies;
        } catch (DAOException dex) {
            throw dex;
        } catch (Exception e) {
            throw new DAOException("Failed to Access Compara to get Regions of Conserved Synteny", e);
        }
    }
}
