/**
 * Copyright (C) 2010 The Roslin Institute <contact andy.law@roslin.ed.ac.uk>
 *
 * This file is part of the Ensembl Java API demonstration project developed by the
 * Bioinformatics Group at The Roslin Institute, The Royal (Dick) School of
 * Veterinary Studies, University of Edinburgh.
 *
 * This is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License (version 3) as published by
 * the Free Software Foundation.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * in this software distribution. If not, see <http://www.gnu.org/licenses/gpl-3.0.html/>.
 */

package uk.ac.roslin.ensembl.datasourceaware.core;

import uk.ac.roslin.ensembl.model.core.CoordSystemType;
import uk.ac.roslin.ensembl.model.core.CoordinateSystem;
import uk.ac.roslin.ensembl.config.EnsemblCoordSystemType;

/**
 *
 * @author paterson
 */
public class DACoordinateSystem implements CoordinateSystem {

    protected Integer id = null;
    protected Integer speciesID = null;
    protected EnsemblCoordSystemType type;
    protected Integer rank;
    protected String version;
    protected Boolean defaultVersion = false;
    protected Boolean sequenceLevel = false;

    public DACoordinateSystem() {
        
    }

    @Override
    public Boolean isTopLevel() {

        return (rank !=null) ? this.rank.equals(1) : false;
    }

    /**
     * Get the value of speciesID
     *
     * @return the value of speciesID
     */
    @Override
    public Integer getSpeciesID() {
        return speciesID;
    }

    /**
     * Set the value of speciesID
     *
     * @param speciesID new value of speciesID
     */
    @Override
    public void setSpeciesID(Integer speciesID) {
        this.speciesID = speciesID;
    }


    /**
     * Get the value of id
     *
     * @return the value of id
     */
    @Override
    public Integer getId() {
        return id;
    }

    /**
     * Set the value of id
     *
     * @param id new value of id
     */
    @Override
    public void setId(Integer id) {
        this.id = id;
    }

    /**
     * Get the value of type
     *
     * @return the value of type
     */
    @Override
    public EnsemblCoordSystemType getType() {
        return type;
    }

    /**
     * Set the value of type
     *
     * @param type new value of type
     */
    @Override
    public void setType(CoordSystemType type) {
        this.type = (EnsemblCoordSystemType) type;
    }
    /**
     * Get the value of version
     *
     * @return the value of version
     */
    @Override
    public String getVersion() {
        return version;
    }

    /**
     * Set the value of version
     *
     * @param version new value of version
     */
    @Override
    public void setVersion(String version) {
        this.version = version;
    }

    /**
     * Get the value of rank
     *
     * @return the value of rank
     */
    @Override
    public Integer getRank() {
        return rank;
    }

    /**
     * Set the value of rank
     *
     * @param rank new value of rank
     */
    @Override
    public void setRank(Integer rank) {
        this.rank = rank;
    }

    @Override
    public Boolean isDefaultVersion() {
        return defaultVersion;
    }

    @Override
    public void setDefaultVersion(Boolean defaultVersion) {
        this.defaultVersion = defaultVersion;
    }

    @Override
    public Boolean isSequenceLevel() {
        return sequenceLevel;
    }

    @Override
    public void setSequenceLevel(Boolean sequenceLevel) {
        this.sequenceLevel = sequenceLevel;
    }

}
