/**
 * Copyright (C) 2010 The Roslin Institute <contact andy.law@roslin.ed.ac.uk>
 *
 * This file is part of the Ensembl Java API demonstration project developed by the
 * Bioinformatics Group at The Roslin Institute, The Royal (Dick) School of
 * Veterinary Studies, University of Edinburgh.
 *
 * This is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License (version 3) as published by
 * the Free Software Foundation.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * in this software distribution. If not, see <http://www.gnu.org/licenses/gpl-3.0.html/>.
 */
package uk.ac.roslin.ensembl.datasourceaware.core;

import java.util.HashMap;
import uk.ac.roslin.ensembl.config.EnsemblComparaDivision;
import uk.ac.roslin.ensembl.config.EnsemblDBType;
import uk.ac.roslin.ensembl.dao.factory.DAOComparaFactory;
import uk.ac.roslin.ensembl.datasourceaware.DAObject;
import uk.ac.roslin.ensembl.dao.factory.DAOCoreFactory;
import uk.ac.roslin.ensembl.dao.factory.DAOSpeciesFactory;
import uk.ac.roslin.ensembl.model.core.CollectionSpecies;
import uk.ac.roslin.ensembl.model.core.CoreObject;
import uk.ac.roslin.ensembl.model.core.Species;
import uk.ac.roslin.ensembl.model.database.CollectionCoreDatabase;
import uk.ac.roslin.ensembl.model.database.CollectionDatabase;
import uk.ac.roslin.ensembl.model.database.SingleSpeciesCoreDatabase;
import uk.ac.roslin.ensembl.model.database.SingleSpeciesDatabase;

public abstract class DACoreObject extends DAObject implements CoreObject {

    protected Species species = null;
//    protected DAOComparaFactory instantiatingComparaFactory = null;
    protected HashMap<EnsemblComparaDivision, DAOComparaFactory> comparaFactories =
            new HashMap<EnsemblComparaDivision, DAOComparaFactory>();

    public DACoreObject() {
        super();
    }

    public DACoreObject(DAOCoreFactory factory) {
        super(factory);
    }

    @Override
    public DAOCoreFactory getDaoFactory() {

        //@start of snip to instantiate a coreFactory from species +version info
        //if their is no corefactory set : see matching change in DBHomologyDAO svn 132

        //try and make a factory if there isn't one, but we do have set the db version and species
        if (daoFactory == null && this.getDBVersion() != null
                && this.getSpecies() != null) {

            DAOCoreFactory f = null;

            try {
                //are we single or multispecies
                if (this.getSpecies() instanceof CollectionSpecies) {
                    f = (DAOCoreFactory) ((CollectionCoreDatabase) ((CollectionSpecies) this.getSpecies()).getDatabaseByTypeAndVersion(
                            EnsemblDBType.collection_core, this.getDBVersion())).getCoreFactory(this.getSpecies());
                } else {
                    f = (DAOCoreFactory) ((SingleSpeciesCoreDatabase) this.getSpecies().getDatabaseByTypeAndVersion(
                            EnsemblDBType.core, this.getDBVersion())).getCoreFactory();
                }
            } catch (Exception e) {
                // LOGGER.warn("Failed to retrieve a Factory for new homologous gene object", e);
            }
            this.setDaoFactory(f);

        }
        //@end snip change svn 132

        return (DAOCoreFactory) daoFactory;
    }

    public Species getSpecies() {
        if (species == null && this.daoFactory != null && this.daoFactory instanceof DAOSpeciesFactory) {
            species = ((DAOSpeciesFactory) this.daoFactory).getSpecies();
        }
        return species;
    }

    public void setSpecies(Species species) {
        this.species = species;
    }

    public DAOComparaFactory getComparaFactory(EnsemblComparaDivision comparaDivision) {

        if (comparaFactories.containsKey(comparaDivision)) {
            return comparaFactories.get(comparaDivision);
        } else if (this.getDaoFactory() != null) {
            DAOComparaFactory f = this.getDaoFactory().getComparaFactory(comparaDivision);
            comparaFactories.put(comparaDivision, f);
            return f;
        }
        return null;
    }

    public EnsemblComparaDivision getComparaDivision() {
        if (this.getSpecies() != null) {
            return this.getSpecies().getComparaDivision();
        } else return null;
    }

    public DAOComparaFactory getComparaFactory() {

        return this.getComparaFactory(this.getComparaDivision());

    }

//   public DAOComparaFactory getInstantiatingComparaFactory() {
//        return instantiatingComparaFactory;
//   }

//    public void setInstantiatingComparaFactory(DAOComparaFactory instantiatingComparaFactory) {
//        this.instantiatingComparaFactory = instantiatingComparaFactory;
//
//    }

    public void setDBVersion(String version) {
        this.dbVersion = version;
    }

    public String getAssembly() {

        if (this.getSpecies() == null || this.getDaoFactory() == null
                || this.getDaoFactory().getDatabase() == null) {
            return null;
        }

        try {

            if (this.getSpecies() instanceof CollectionSpecies) {
                return ((CollectionDatabase) this.getDaoFactory().getDatabase()).getAssembly(
                        (CollectionSpecies) this.getSpecies());

            } else {
                return ((SingleSpeciesDatabase) this.getDaoFactory().getDatabase()).getAssembly();

            }
        } catch (Exception e) {
            return null;
        }

    }

}
