/**
 * Copyright (C) 2010 The Roslin Institute <contact andy.law@roslin.ed.ac.uk>
 *
 * This file is part of the Ensembl Java API demonstration project developed by the
 * Bioinformatics Group at The Roslin Institute, The Royal (Dick) School of
 * Veterinary Studies, University of Edinburgh.
 *
 * This is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License (version 3) as published by
 * the Free Software Foundation.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * in this software distribution. If not, see <http://www.gnu.org/licenses/gpl-3.0.html/>.
 */

package uk.ac.roslin.ensembl.datasourceaware.core;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import uk.ac.roslin.ensembl.config.EnsemblComparaDivision;
import uk.ac.roslin.ensembl.model.Mapping;
import uk.ac.roslin.ensembl.model.MappingSet;
import uk.ac.roslin.ensembl.model.Coordinate;
import uk.ac.roslin.ensembl.model.Coordinate.Strand;
import uk.ac.roslin.ensembl.model.Mapping;
import uk.ac.roslin.ensembl.model.database.Registry;
import uk.ac.roslin.ensembl.model.core.CoordinateSystem;
import uk.ac.roslin.ensembl.dao.factory.DAOCoreFactory;
import uk.ac.roslin.ensembl.dao.factory.DAOFactory;
import uk.ac.roslin.ensembl.exception.DAOException;
import uk.ac.roslin.ensembl.model.CoordinateSet;
import uk.ac.roslin.ensembl.model.ObjectType;
import uk.ac.roslin.ensembl.model.core.CoreObject;
import uk.ac.roslin.ensembl.model.core.Species;
import uk.ac.roslin.ensembl.config.FeatureType;
import uk.ac.roslin.ensembl.dao.factory.DAOComparaFactory;
import uk.ac.roslin.ensembl.dao.factory.DAOSpeciesFactory;
import uk.ac.roslin.ensembl.model.core.CollectionSpecies;
import uk.ac.roslin.ensembl.model.database.CollectionDatabase;
import uk.ac.roslin.ensembl.model.database.SingleSpeciesDatabase;

/**
 *
 * @author paterson
 */
public class DADNASequence extends org.biojava3.core.sequence.DNASequence implements CoreObject, uk.ac.roslin.ensembl.model.core.DNASequence {

    protected DAOFactory daoFactory = null;
    protected String schemaVersion = null;
    protected String dbVersion = null;
    protected Registry registry = null;
    protected Integer id = null;
    protected Integer seqRegionID = null;
    protected String name = null;
    protected String dbSpeciesName = null;
    protected Species species = null;
    protected Integer DBSeqLength = null;
    protected CoordinateSystem coordSystem = null;
    protected MappingSet mappings = new MappingSet();
    protected HashMap<ObjectType, CoordinateSet> mappedRegions
            = new HashMap<ObjectType, CoordinateSet>();
    protected HashMap<ObjectType, MappingSet> objectTypeMappings
            = new HashMap<ObjectType, MappingSet>();
    protected HashMap<EnsemblComparaDivision, DAOComparaFactory> comparaFactories
            = new HashMap<EnsemblComparaDivision, DAOComparaFactory>();

    public DADNASequence() {
        super();

    }

    public DADNASequence(DAEnsemblDNASequenceReader proxyLoader) {
        super(proxyLoader);
        ((DAEnsemblDNASequenceReader) this.getProxySequenceReader()).setParent(this);
        this.setId(proxyLoader.getSeqRegionID());
        this.setDBSeqLength(proxyLoader.getLengthInteger());
    }

    public DADNASequence(String sequence) {
        super(new DAEnsemblDNASequenceReader());
        ((DAEnsemblDNASequenceReader) this.getProxySequenceReader()).setParent(this);
        ((DAEnsemblDNASequenceReader) this.getProxySequenceReader()).setContents(sequence);
    }

    public DADNASequence(DAOCoreFactory factory) {
        this.setDaoFactory(factory);
    }

    public void setSequenceStorage(DAEnsemblDNASequenceReader proxyLoader) {
        this.setProxySequenceReader(proxyLoader);
        ((DAEnsemblDNASequenceReader) this.getProxySequenceReader()).setParent(this);
    }

    public DAOCoreFactory getDaoFactory() {
        return (DAOCoreFactory) daoFactory;
    }

    public void setDaoFactory(DAOFactory daoFactory) {
        this.daoFactory = daoFactory;
        if (this.daoFactory!= null) {
            this.setSchemaVersion(daoFactory.getEnsemblSchemaVersion());
            this.setDBVersion(daoFactory.getDBVersion());
            this.setRegistry((Registry) daoFactory.getRegistry());
        }
    }

    public String getSchemaVersion() {
            return schemaVersion;
    }

    private void setSchemaVersion(String schemaVersion) {
        this.schemaVersion = schemaVersion;
    }

    public String getDBVersion() {
            return dbVersion;
    }

    private void setDBVersion(String dbversion) {
        this.dbVersion = dbversion;
    }

    public Registry getRegistry() {
            return registry;
    }

    public void setRegistry(Registry datasource) {
        this.registry = datasource;
    }

    public Integer getId() {
        return id;
    }

    public void setId(Integer id) {
        this.id = id;
    }

    public String getDBName() {
        return (daoFactory!= null)?daoFactory.getDatabaseName():null;
    }

    public Species getSpecies() {
        if (species == null && this.daoFactory != null && this.daoFactory instanceof DAOSpeciesFactory) {
            species = ((DAOSpeciesFactory) this.daoFactory).getSpecies();
        }
        return species;
    }

    public void setSpecies(Species species) {
        this.species = species;
    }

    public ObjectType getType() {
        //forces a lazy load so make sure we get CS for all retrieved sequenceIDS in the first query!
        if (this.getCoordSystem() != null) {
            return this.getCoordSystem().getType();
        } else {
            return null;
        }
    }

    public Integer getDBSeqLength() {
        if (DBSeqLength == null) {
            this.lazyLoad();
        }
        if (DBSeqLength != null) {
            return DBSeqLength;
        } else { return 0;}
    }

    public void setDBSeqLength(Integer seqLength) {
        this.DBSeqLength = seqLength;
        this.setBioEnd(seqLength);
    }

    @Override
    public Integer getBioEnd() {

        try {
            return (super.getBioEnd() != null) ? super.getBioEnd() : this.getDBSeqLength();
        } catch (NullPointerException e) {
            return this.getDBSeqLength();
        }
    }

    @Override
    public int getLength() {

        try {
            return super.getLength();
        } catch (NullPointerException e) {
            return this.getDBSeqLength();
        }
    }

    public String getName() {
        if (name == null) {
            this.lazyLoad();
        }
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public CoordinateSystem getCoordSystem() {

        if (coordSystem == null) {
            this.lazyLoad();
        }

        return coordSystem;
    }

    public void setCoordSystem(CoordinateSystem coordSystem) {
        this.coordSystem = coordSystem;
    }

    public MappingSet getMappings() {
        return this.mappings;
    }

    //Should Be Private??
    public MappingSet getMappings(ObjectType targetType) {

        //need to add lazy load here?

        if (!this.objectTypeMappings.containsKey(targetType)) {
            return new MappingSet();
        } else {
            return this.objectTypeMappings.get(targetType);
        }
    }

    public Boolean addMapping(Mapping mapping) {

//        //check we haven't already added a mapping for an object with this id!
//        if (mapping.getTarget() != null) {
//
//            for (Mapping m : this.mappings) {
//                if (m.getTarget() != null && m.getTarget().getId().equals(mapping.getTarget().getId())) {
//                    return;
//                }
//            }
//        }

        //if we fail to add the mapping this will be false
        if (this.mappings.add((Mapping) mapping)) {

            ObjectType t = mapping.getTargetType();

            if (t != null) {
                if (!this.objectTypeMappings.containsKey(t)) {
                    this.objectTypeMappings.put(t, new MappingSet());
                }

                this.objectTypeMappings.get(t).add((Mapping) mapping);
            }
            return true;
        }
        return false;
    }

    public List<DAGene> getGenesOnRegion(Coordinate coord) throws DAOException {

        List<DAGene> out = new ArrayList<DAGene>();

        List<? extends Mapping> mappings = null;

        //test if we already have got the genes for this coordinate region
        if (this.mappedRegions.containsKey(FeatureType.gene)
                && this.mappedRegions.get(FeatureType.gene).containsCoordinateWithoutGaps(coord)) {

            //we need a temporary pointer here so we can match the signatures
            ArrayList<Mapping> theseMappings = new ArrayList<Mapping>();

            //find and return genes in this region
            for (Mapping m : this.getMappings(FeatureType.gene)) {
                Coordinate c = m.getSourceCoordinates();
                if (c.overlaps(coord)) {
                    theseMappings.add(m);
                }
            }
            mappings = theseMappings;

            if (mappings == null || mappings.isEmpty()) {
                return out;
            } else {
                //these mappings are seq -> gene

                if (coord.getStrand() == null) {
                    for (Mapping m : mappings) {
                        out.add((DAGene) m.getTarget());
                        //already done in the DAO
                        //this.addMapping(m);

                    }
                } else {
                    for (Mapping m : mappings) {
                        if (m.getSourceCoordinates().getStrand().equals(coord.getStrand())) {
                            out.add((DAGene) m.getTarget());
                        }
                        //already done in the DAO
                        //this.addMapping(m);
                    }
                }
            }
            return out;

        } else {

            if (this.getDaoFactory() != null && this.getDaoFactory().getGeneDAO() != null) {

                mappings = this.getDaoFactory().getGeneDAO().getGeneMappingsOnRegion(this, coord);
            }

            if (mappings == null || mappings.isEmpty()) {
                return out;
            } else {
                //these mappings ade gene->seq
                if (coord.getStrand() == null) {
                    for (Mapping m : mappings) {
                        out.add((DAGene) m.getSource());
                        //already done in the DAO
                        //this.addMapping(m);

                    }
                } else {
                    for (Mapping m : mappings) {
                        if (m.getTargetCoordinates().getStrand().equals(coord.getStrand())) {
                            out.add((DAGene) m.getSource());
                        }
                        //already done in the DAO
                        //this.addMapping(m);
                    }
                }
            }

            //put this region in the sore of mapped regions
            if (!mappedRegions.containsKey(FeatureType.gene)) {
                mappedRegions.put(FeatureType.gene, new CoordinateSet());
            }
            this.mappedRegions.get(FeatureType.gene).add(coord);
            return out;
        }
    }

    public List<DAGene> getGenesOnRegion(Integer start, Integer stop, Strand strand) throws DAOException {

        Strand s = strand;
        Integer begin = null;
        Integer end = null;

//        if (s == null) {
//            s = Strand.FORWARD_STRAND;
//        }

        if (start != null && stop != null && start.doubleValue() > stop.doubleValue()) {
            begin = stop;
            end = start;
        } else {
            begin = start;
            end = stop;
        }

        Coordinate coord = new Coordinate(start, stop, strand);
        return this.getGenesOnRegion(coord);

    }

    public List<DAGene> getGenesOnRegion(Integer start, Integer stop) throws DAOException {

        return this.getGenesOnRegion(start, stop, null);
    }

    private void lazyLoad() {

        if (this.id != null && this.getDaoFactory() != null) {
            try {
                DADNASequence fetchedSeq = (DADNASequence) this.getDaoFactory().getSequenceDAO().getSequenceByID(id);
                this.setName(fetchedSeq.getName());
                this.setDBSeqLength(fetchedSeq.getDBSeqLength());
                this.setCoordSystem(fetchedSeq.getCoordSystem());
            } catch (Exception ex) {
            }
        }
    }

    public String getSequenceAsString(Integer begin, Integer end) {
        if (this.getLength()==0) {
            return "";
        }
        return super.getSequenceAsString(begin, end, org.biojava3.core.sequence.Strand.POSITIVE);
    }

    public String getReverseComplementSequenceAsString(Integer begin, Integer end) {
        if (this.getLength()==0) {
            return "";
        }
        DAEnsemblDNASequenceReader ss = (DAEnsemblDNASequenceReader) getProxySequenceReader();
        return ss.getReverseComplementSequenceAsString(begin, end);
    }

    public String getReverseComplementSequenceAsString() {
        if (this.getLength()==0) {
            return "";
        }
        return this.getReverseComplementSequenceAsString(this.getBioBegin(), this.getBioEnd());
    }

    public HashMap<ObjectType, MappingSet> getObjectTypeMappings() {
        return objectTypeMappings;
    }

    public HashMap<ObjectType, CoordinateSet> getMappedRegions() {
        return mappedRegions;
    }

    public String getHashID() {
        return
                ((this.getDaoFactory()!=null) ? this.getDaoFactory().getDatabaseName() : "NODATABASE")

                +"_"

                +((this.getType()!=null) ? this.getType().toString() : "NOTYPE")

                +"_"

                +((this.getId()!=null) ? this.getId().toString() : "NOID");
    }

    @Override
    public String toString() {
        return ((this.getId()!=null) ? this.getId().toString() : "NOID");
    }

    public void clearAllMappings() {
            mappings.clear();
            mappedRegions.clear();
            objectTypeMappings.clear();
    }

    public EnsemblComparaDivision getComparaDivision() {
        if (this.getSpecies()==null) {
            return null;
        }
        return this.getSpecies().getComparaDivision();
    }

   public DAOComparaFactory getComparaFactory(EnsemblComparaDivision comparaDivision) {

        if (comparaFactories.containsKey(comparaDivision)) {
            return comparaFactories.get(comparaDivision);
        } else if (this.getDaoFactory() != null) {
            DAOComparaFactory f = this.getDaoFactory().getComparaFactory(comparaDivision);
            comparaFactories.put(comparaDivision, f);
            return f;
        }
        return null;
    }

    public DAOComparaFactory getComparaFactory() {

        EnsemblComparaDivision comparaDivision = this.getComparaDivision();

        if (comparaFactories.containsKey(comparaDivision)) {
            return comparaFactories.get(comparaDivision);
        } else if (this.getDaoFactory() != null) {
            DAOComparaFactory f = this.getDaoFactory().getComparaFactory(comparaDivision);
            comparaFactories.put(comparaDivision, f);
            return f;
        }
        return null;
    }

    public String getAssembly() {

        if (this.getSpecies() == null || this.getDaoFactory() == null
                || this.getDaoFactory().getDatabase() == null) {
            return null;
        }

        try {

            if (this.getSpecies() instanceof CollectionSpecies) {
                return ((CollectionDatabase) this.getDaoFactory().getDatabase()).getAssembly(
                        (CollectionSpecies) this.getSpecies());

            } else {
                return ((SingleSpeciesDatabase) this.getDaoFactory().getDatabase()).getAssembly();

            }
        } catch (Exception e) {
            return null;
        }

    }

    
}
