/**
 * Copyright (C) 2011 The Roslin Institute <contact andy.law@roslin.ed.ac.uk>
 *
 * This file is part of the Ensembl Java API demonstration project developed by the
 * Bioinformatics Group at The Roslin Institute, The Royal (Dick) School of
 * Veterinary Studies, University of Edinburgh.
 *
 * This is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License (version 3) as published by
 * the Free Software Foundation.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * in this software distribution. If not, see <http://www.gnu.org/licenses/gpl-3.0.html/>.
 */

package uk.ac.roslin.ensembl.datasourceaware.core;

import org.apache.log4j.Logger;
import uk.ac.roslin.ensembl.dao.factory.DAOCoreFactory;
import uk.ac.roslin.ensembl.exception.DAOException;
import uk.ac.roslin.ensembl.model.ObjectType;
import uk.ac.roslin.ensembl.model.core.Transcript;
import uk.ac.roslin.ensembl.config.FeatureType;
import uk.ac.roslin.ensembl.model.core.Exon;

public class DAExon extends DAFeature implements Exon {

    private String stableID = null;
    private DATranscript transcript = null;
    private Integer transcriptID  = null;
    //the position of this exon from the 5' end of the trancript, starting with 1
    private Integer rank  = null;
    private int phase = Phase.NONE;
    private int endPhase = Phase.NONE;
    private Boolean constitutive;

    final static Logger LOGGER = Logger.getLogger(DATranscript.class);

    public DAExon() {
        super();
    }

    public DAExon(DAOCoreFactory factory) {
        super(factory);
     }

    @Override
    void reinitialize() throws DAOException {
       //not used yet
    }

    @Override
    public ObjectType getType() {
        return FeatureType.exon;
    }

    public DATranscript getTranscript() {
        if (transcript==null && this.transcriptID != null) {
            try {
                transcript = (DATranscript) this.getDaoFactory().getTranscriptDAO().getTranscriptByID(transcriptID);
            } catch (Exception e) {
                LOGGER.info("Error thrown whilst trying to retrieve Transcript for an exon", e );
            }
        }

        return this.transcript;
    }

    public void setTranscript(Transcript transcript) {
        this.transcript = (DATranscript)transcript;
    }

    public Integer getTranscriptID() {
        return transcriptID;
    }

    public void setTranscriptID(Integer transcriptID) {
        this.transcriptID = transcriptID;
    }

    public String getStableID() {
        return stableID;
    }

    public void setStableID(String stableID) {
        this.stableID = stableID;
    }

    @Override
    public String getDisplayName() {
        return (displayName!=null) ? displayName : stableID ;
    }

    public int getPhase() {
        return phase;
    }

    public void setPhase(int phase) {
       this.phase=Phase.getPhase(phase);

    }

    public int getEndPhase() {
        return endPhase;
    }

    public void setEndPhase(int endPhase) {
       this.endPhase=Phase.getPhase(endPhase);

    }

    public Boolean isConstitutive() {
        return constitutive;
    }

    public void setConstitutive(Boolean constitutive) {
        this.constitutive = constitutive;
    }

    public Integer getRank() {
        return rank;
    }

    public void setRank(Integer rank) {
        this.rank = rank;
    }

}
