/**
 * Copyright (C) 2010 The Roslin Institute <contact andy.law@roslin.ed.ac.uk>
 *
 * This file is part of the Ensembl Java API demonstration project developed by the
 * Bioinformatics Group at The Roslin Institute, The Royal (Dick) School of
 * Veterinary Studies, University of Edinburgh.
 *
 * This is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License (version 3) as published by
 * the Free Software Foundation.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * in this software distribution. If not, see <http://www.gnu.org/licenses/gpl-3.0.html/>.
 */

package uk.ac.roslin.ensembl.datasourceaware.core;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.TreeMap;
import org.apache.log4j.Logger;
import uk.ac.roslin.ensembl.dao.factory.DAOCoreFactory;
import uk.ac.roslin.ensembl.exception.DAOException;
import uk.ac.roslin.ensembl.model.ObjectType;
import uk.ac.roslin.ensembl.model.core.Transcript;
import uk.ac.roslin.ensembl.config.FeatureType;
import uk.ac.roslin.ensembl.model.core.Exon;

/**
 *
 * @author tpaterso
 */
public class DATranscript extends DAFeature implements Transcript{

    DATranslation translation = null;
    private String stableID = null;
    private String biotype = null;
    private DAGene gene = null;
    private boolean canonical = false;
    private Integer geneID = null;
    private TreeMap<Integer, DAExon> exonHash = null;
    final static Logger LOGGER = Logger.getLogger(DATranscript.class);

    public DATranscript() {
        super();
    }

    public DATranscript(DAOCoreFactory daoFactory) {
        super(daoFactory);
    }


    public ObjectType getType() {
        return FeatureType.transcript;
    }

    public DATranslation getTranslation() {
        if (translation == null) {

            if (this.getId()==null) {
                return translation;
            }

            try {
                translation = (DATranslation) this.getDaoFactory().getTranslationDAO().getTranslationByTranscriptID(this.id);
                translation.setTranscript(this);
            }
            catch (Exception e) {
                LOGGER.info("failed to get Translation", e);
            }

        }
        return translation;


    }

    @Override
    void reinitialize() throws DAOException {
       //not used yet
    }

    public String getStableID() {
        return stableID;
    }

    public void setStableID(String stableID) {
        this.stableID = stableID;
    }

    public String getBiotype() {
        return this.biotype;
    }

    public void setBiotype(String biotype) {
        this.biotype = biotype;
    }

    @Override
    public DAGene getGene() {
        if (gene==null && this.geneID != null) {
            try {
                gene = (DAGene) this.getDaoFactory().getGeneDAO().getGeneByID(geneID);
            } catch (Exception e) {
                LOGGER.info("Error thrown whilst trying to retrieve Gene for a Transcript", e );
            }
        }

        return gene;
    }

    public void setGene(DAGene gene) {
        this.gene = gene;
    }

    public Integer getGeneID() {
        return geneID;
    }

    public void setGeneID(Integer geneID) {
        this.geneID = geneID;
    }

    public boolean isCanonical() {
        return canonical;
    }

    public void setCanonical(boolean canonical) {
        this.canonical = canonical;
    }

    @Override
    public String getDisplayName() {
        return (displayName!=null) ? displayName : stableID ;
    }

    @Override
    public Collection<DAExon> getExons() {
        if (exonHash == null) {

            try {
                exonHash = new TreeMap<Integer, DAExon>();
                return (Collection<DAExon>) this.getDaoFactory().getExonDAO().getExonsForTranscript(this);
            } catch (Exception e) {
                exonHash = new TreeMap<Integer, DAExon>();
                LOGGER.info("Threw DAOException on tryiong to populate exons for a transcript", e);
            }

        }
        return this.exonHash.values();
    }

    public void addExon(Exon exon) {
        if (exonHash==null) {
             exonHash = new TreeMap<Integer, DAExon>();
        }
        try {
            DAExon e = (DAExon) exon;
            this.exonHash.put(e.getRank(), e);
        } catch (Exception ex) {
            LOGGER.info("failed to add an exon to the transcript ", ex);
        }
    }

}
