/**
 * Copyright (C) 2010 The Roslin Institute <contact andy.law@roslin.ed.ac.uk>
 *
 * This file is part of the Ensembl Java API demonstration project developed by the
 * Bioinformatics Group at The Roslin Institute, The Royal (Dick) School of
 * Veterinary Studies, University of Edinburgh.
 *
 * This is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License (version 3) as published by
 * the Free Software Foundation.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * in this software distribution. If not, see <http://www.gnu.org/licenses/gpl-3.0.html/>.
 */

package uk.ac.roslin.ensembl.datasourceaware.core;

import org.biojava3.core.sequence.Strand;

/**
 *
 * @author paterson
 */
public class GapSequence extends DADNASequence {

    private static final String  base  = "N";
    private  StringBuffer seq = null;

    public static GapSequence makeGap(Integer length) {
        if (length == null || length <0 ) {
            return null;
        }
        return new GapSequence(length);
    }

    private static String getGapString(Integer length) {
            StringBuffer sb = new StringBuffer();
            for (int i = 0; i < length; i++) {
                    sb= sb.append(base);
                }
            return sb.toString();
    }

     private GapSequence(Integer length) {
        super();
        this.setDBSeqLength(length);
        this.setBioBegin(1);
        this.setBioEnd(length);
        this.setName("Gap of "+length);
    }


    @Override
    public String getSequenceAsString() {
            return this.getSequenceAsString(this.getBioBegin(), this.getBioEnd(), null);
    }

    @Override
    public String getReverseComplementSequenceAsString() {
            return this.getSequenceAsString(this.getBioBegin(), this.getBioEnd(), null);
    }

    @Override
    public String getSequenceAsString(Integer start, Integer stop) {
        return this.getSequenceAsString(start, stop, Strand.POSITIVE);
    }

    @Override
    public String getSequenceAsString(Integer start, Integer stop, Strand strand){


        Integer begin = start;
        Integer end = stop;


        if (end == null || end > this.getBioEnd()) {
            end = this.getBioEnd();
        }
        if (begin == null || begin<this.getBioBegin()) {
            begin = this.getBioBegin();
        }
        return GapSequence.getGapString(end-begin+1);

    }

    @Override
    public String getReverseComplementSequenceAsString(Integer start, Integer stop){

        return this.getSequenceAsString(start, stop, null);

    }

    @Override
    public String toString() {
        return name;
    }

    @Override
    public int getLength() {
        return this.getDBSeqLength().intValue();
    }
}