/**
 * Copyright (C) 2010 The Roslin Institute <contact andy.law@roslin.ed.ac.uk>
 *
 * This file is part of the Ensembl Java API demonstration project developed by the
 * Bioinformatics Group at The Roslin Institute, The Royal (Dick) School of
 * Veterinary Studies, University of Edinburgh.
 *
 * This is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License (version 3) as published by
 * the Free Software Foundation.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * in this software distribution. If not, see <http://www.gnu.org/licenses/gpl-3.0.html/>.
 */

package uk.ac.roslin.ensembl.model;

import java.io.Serializable;
import java.util.TreeSet;

/**
 *
 * @author tpaterso
 */
public class CoordinateSet extends TreeSet<Coordinate>  implements Serializable {

    public CoordinateSet() {
    }


    public Integer getStart() {
        Integer out = null;

        if ( !this.isEmpty()) {
            out = this.first().getStart();
        }

        return out;
    }

    public Integer getEnd() {
        Integer out = null;

        if ( !this.isEmpty()) {

            out = this.last().getEnd();

            for (Coordinate c : this) {
             if (c.getEnd()>out) {
                 out = c.getEnd();
             }
            }
        }
         return out;
    }

    public Coordinate getRange() {
         Coordinate out = null;
         if (  !this.isEmpty()) {

             out = new Coordinate(this.getStart(),this.getEnd() ,1);

         }
         return out;
     }

    public CoordinateSet getGaps(){
        CoordinateSet gaps = new CoordinateSet();


        Integer rangeEnd = this.getEnd();
        Integer currentPosition = this.getStart();

        for (Coordinate c : this) {

            Integer s = c.getStart();
            Integer e = c.getEnd();

            //if we're already past the extent of this region
            if (currentPosition>=e) {
                continue;
            }

            if (s>currentPosition) {

                Coordinate newC = new Coordinate(currentPosition, s-1, 1);
                gaps.add(newC);
            }

            if (e>=rangeEnd) {
                break;
            }


            currentPosition = e+1;

        }

        return gaps;
    }

    public Boolean containsCoordinateWithoutGaps(Coordinate test) {
        Boolean out = false;

        Integer testBegin = test.getStart();
        Integer testEnd = test.getEnd();

        if (testBegin<this.getStart() || testEnd>this.getEnd()) {
            out = false;
        } else if (this.getGaps().isEmpty()) {
            out = true;
        } else {

            out = true;
            for(Coordinate c: this.getGaps()) {
                if(test.overlaps(c)) {
                    out = false;
                    break;
                }
            }

        }
        return out;
    }

    public CoordinateSet getUncoveredRegions(Coordinate test) {

        CoordinateSet gaps = new CoordinateSet();

        if (this.isEmpty()) {
            gaps.add(test);
            return gaps;
        }

        if (this.containsCoordinateWithoutGaps(test)) {
            return gaps;
        }

        if (!this.getRange().overlaps(test)) {
            gaps.add(test);
            return gaps;
        }

        Integer testBegin = test.getStart();
        Integer testEnd = test.getEnd();
        Integer thisBegin = this.getStart();
        Integer thisEnd = this.getEnd();

        CoordinateSet thisGaps = this.getGaps();

        //overlap at start
        if (testBegin<thisBegin) {
            Coordinate newC = new Coordinate(testBegin, thisBegin-1, 1);
            gaps.add(newC);
        }

        //overlap at end
        if (testEnd>thisEnd) {
            Coordinate newC2 = new Coordinate(thisEnd+1, testEnd,  1);
            gaps.add(newC2);
        }

        for (Coordinate cd : thisGaps) {

            //this gap completely contains the test Coordinate
            if (test.liesWithinCoordinate(cd)) {
                gaps.add(test);
                break;
            }

            // this gap lies completely within the range
            else if (cd.liesWithinCoordinate(test)) {
                gaps.add(cd);
            }

            else if (cd.overlaps(test)) {

                gaps.add(cd.getOverlap(test));
            }

        }

        return gaps;
    }

}
