/**
 * Copyright (C) 2010 The Roslin Institute <contact andy.law@roslin.ed.ac.uk>
 *
 * This file is part of the Ensembl Java API demonstration project developed by the
 * Bioinformatics Group at The Roslin Institute, The Royal (Dick) School of
 * Veterinary Studies, University of Edinburgh.
 *
 * This is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License (version 3) as published by
 * the Free Software Foundation.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * in this software distribution. If not, see <http://www.gnu.org/licenses/gpl-3.0.html/>.
 */

package uk.ac.roslin.ensembl.model;

import java.io.Serializable;
import java.util.Comparator;
import java.util.TreeSet;

/**
 * Used to be abstract with signature
 * public abstract class MappingSet<CLAZZ extends Mapping> extends TreeSet<CLAZZ> implements Serializable {
 * @author tpaterso
 */
public class MappingSet extends TreeSet<Mapping> implements Serializable {

    //this is tragically bad design - should have two subclasses, one Mapping on Source,
    //one Mapping on Target ( and potentially other subclasses with other comparators )

    public MappingSet() {
        super(Mapping.mappingOnSourceComparator);
        //super(new MappingOnSourceComparator());
    }

    public MappingSet(Comparator differentComparator) {
        super(differentComparator);
    }

    public Coordinate getExtent() {
        if (this.isEmpty()) {
            return null;
        }


        //if (this.comparator() instanceof MappingOnSourceComparator) {
        if (this.comparator().equals(Mapping.mappingOnSourceComparator)) {

            Coordinate out = new Coordinate(null, null, 1);

            if (this.first().getSourceCoordinates()!=null) {
                out.setStart(this.first().getSourceCoordinates().getStart());
            }
            Integer end = 0;
            for (Mapping m : this) {
                if (m.getSourceCoordinates() != null
                        && m.getSourceCoordinates().getEnd() > end ) {
                    end =  m.getSourceCoordinates().getEnd();
                }
            }
            if (end != 0) {
                out.setEnd(end);
            }

            return out;

        //} else if (this.comparator() instanceof MappingOnTargetComparator) {
        } else if (this.comparator().equals(Mapping.mappingOnTargetComparator)) {
            Coordinate out = new Coordinate(null, null, 1);

            if (this.first().getTargetCoordinates()!=null) {
                out.setStart(this.first().getTargetCoordinates().getStart());
            }

            Integer end = 0;
            for (Mapping m : this) {
                if (m.getTargetCoordinates() != null
                        && m.getTargetCoordinates().getEnd() > end ) {
                    end =  m.getTargetCoordinates().getEnd();
                }
            }
            if (end != 0) {
                out.setEnd(end);
            }

            return out;

        }

        return null;
    }
    
}
