/**
 * Copyright (C) 2010 The Roslin Institute <contact andy.law@roslin.ed.ac.uk>
 *
 * This file is part of the Ensembl Java API demonstration project developed by the
 * Bioinformatics Group at The Roslin Institute, The Royal (Dick) School of
 * Veterinary Studies, University of Edinburgh.
 *
 * This is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License (version 3) as published by
 * the Free Software Foundation.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * in this software distribution. If not, see <http://www.gnu.org/licenses/gpl-3.0.html/>.
 */
package uk.ac.roslin.ensembl.model.compara;

import java.io.Serializable;
import uk.ac.roslin.ensembl.model.Coordinate;


/**
 * Utility class used to hold properties of an aligned homology object (gene)
 * in a HomologyPairRelationship. The compara database doesnt hold enough info 
 * to instantiate objects fully.
 * Because a DAHomologyPairRelationship is typically instantiated from the
 * compara databases - the target gene in particular is likely to not
 * be a core-datasource aware object, and therefore species & and mapping details
 * are only simple fields grabbed from compara: these are held in properties
 * objects
 * @author tpaterso
 */
public class HomologyAlignmentProperties implements Serializable {

    private String cigarLine = null;
    private String peptideID = null;
    private String geneID = null;
    private String speciesName = null;
    private Integer cigarStart = null;
    private Integer cigarEnd = null;
    private Integer percentCovered = null;
    private Integer percentIdentity = null;
    private Integer percentSimilar = null;
    private Coordinate coords = null;
    private String sequenceName = null;

    public Integer getCigarEnd() {
        return cigarEnd;
    }

    public void setCigarEnd(Integer cigarEnd) {
        this.cigarEnd = cigarEnd;
    }

    public String getCigarLine() {
        return cigarLine;
    }

    public void setCigarLine(String cigarLine) {
        this.cigarLine = cigarLine;
    }

    public Integer getCigarStart() {
        return cigarStart;
    }

    public void setCigarStart(Integer cigarStart) {
        this.cigarStart = cigarStart;
    }

    public String getGeneID() {
        return geneID;
    }

    public void setGeneID(String geneID) {
        this.geneID = geneID;
    }

    public String getPeptideID() {
        return peptideID;
    }

    public void setPeptideID(String peptideID) {
        this.peptideID = peptideID;
    }

    public Integer getPercentCovered() {
        return percentCovered;
    }

    public void setPercentCovered(Integer percentCovered) {
        this.percentCovered = percentCovered;
    }

    public Integer getPercentIdentity() {
        return percentIdentity;
    }

    public void setPercentIdentity(Integer percentIdentity) {
        this.percentIdentity = percentIdentity;
    }

    public Integer getPercentSimilar() {
        return percentSimilar;
    }

    public void setPercentSimilar(Integer percentSimilar) {
        this.percentSimilar = percentSimilar;
    }

    public String getSpeciesName() {
        return speciesName;
    }

    public void setSpeciesName(String speciesName) {
        this.speciesName = speciesName;
    }

    public String getSequenceName() {
        return sequenceName;
    }

    public void setSequenceName(String seqName) {
        this.sequenceName = seqName;
    }

    public Coordinate getCoords() {
        return coords;
    }

    public void setCoords(Coordinate coords) {
        this.coords = coords;
    }


}
