/**
 * Copyright (C) 2010 The Roslin Institute <contact andy.law@roslin.ed.ac.uk>
 *
 * This file is part of the Ensembl Java API demonstration project developed by the
 * Bioinformatics Group at The Roslin Institute, The Royal (Dick) School of
 * Veterinary Studies, University of Edinburgh.
 *
 * This is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License (version 3) as published by
 * the Free Software Foundation.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * in this software distribution. If not, see <http://www.gnu.org/licenses/gpl-3.0.html/>.
 */

package uk.ac.roslin.ensembl.model.core;

import java.util.HashMap;
import java.util.List;
import org.biojava3.core.sequence.compound.NucleotideCompound;
import org.biojava3.core.sequence.template.Sequence;
import uk.ac.roslin.ensembl.exception.DAOException;
import uk.ac.roslin.ensembl.model.Coordinate;
import uk.ac.roslin.ensembl.model.Coordinate.Strand;
import uk.ac.roslin.ensembl.model.CoordinateSet;
import uk.ac.roslin.ensembl.model.MappableObject;
import uk.ac.roslin.ensembl.model.MappingSet;
import uk.ac.roslin.ensembl.model.ObjectType;

/**
 *
 * @author paterson
 */
public interface DNASequence extends CoreObject, MappableObject, Sequence<NucleotideCompound> {

    public Integer getDBSeqLength() ;

    public void setDBSeqLength(Integer seqLength) ;

    public CoordinateSystem getCoordSystem() ;

    public void setCoordSystem(CoordinateSystem coordSystem) ;

    public String getName() ;

    public void setName(String name) ;

    public List<? extends Gene> getGenesOnRegion(Coordinate coord) throws DAOException ;

    public List<? extends Gene> getGenesOnRegion(Integer start, Integer stop) throws DAOException ;

    public List<? extends Gene> getGenesOnRegion(Integer start, Integer stop, Strand strand) throws DAOException ;

    public String getSequenceAsString(Integer begin, Integer end) ;

    public String getReverseComplementSequenceAsString(Integer begin, Integer end) ;

    public String getReverseComplementSequenceAsString() ;

    public HashMap<ObjectType, ? extends MappingSet> getObjectTypeMappings() ;

    public HashMap<ObjectType, CoordinateSet> getMappedRegions() ;
    
}
