/**
 * Copyright (C) 2010 The Roslin Institute <contact andy.law@roslin.ed.ac.uk>
 *
 * This file is part of the Ensembl Java API demonstration project developed by the
 * Bioinformatics Group at The Roslin Institute, The Royal (Dick) School of
 * Veterinary Studies, University of Edinburgh.
 *
 * This is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License (version 3) as published by
 * the Free Software Foundation.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * in this software distribution. If not, see <http://www.gnu.org/licenses/gpl-3.0.html/>.
 */

package uk.ac.roslin.ensembl.model.core;

import java.util.HashMap;
import java.util.TreeMap;
import java.util.TreeSet;
import uk.ac.roslin.ensembl.config.EnsemblComparaDivision;
import uk.ac.roslin.ensembl.exception.DAOException;
import uk.ac.roslin.ensembl.model.database.CoreDatabase;
import uk.ac.roslin.ensembl.model.database.Database;
import uk.ac.roslin.ensembl.model.database.DatabaseType;

/**
 *
 * @author paterson
 */
public interface Species {

    public TreeSet<String> getAliases();

    public String getCommonName() ;

    public String getSpeciesBinomial() ;

    public String getDatabaseStyleName();

    public String getComparaName();

    public String getShortName();

    public void setProperty(HashMap row) ;

    public CoreDatabase getCurrentCoreDatabase();

    public String getTaxonomyID() ;

    public Integer getDBSpeciesID(String version) ;

    public String getEnsemblStablePrefix() ;

    public TreeSet<? extends Database> getDatabases();

    public void addDatabases(TreeSet<? extends Database> databases);

    public void addDatabase(Database database);

    public TreeSet<? extends Database> getDatabasesByType(DatabaseType type);

    public TreeSet<? extends Database> getDatabasesByVersion(String version);

    public Database getDatabaseByTypeAndVersion(DatabaseType type, String version);

    public Chromosome getChromosomeByName(String name) throws DAOException ;

    public Chromosome getChromosomeByName(String name, String dbVersion) throws DAOException;

    public TreeMap<String, ? extends Chromosome> getChromosomes() throws DAOException ;

    public TreeMap<String, ? extends Chromosome> getChromosomes(String dbVersion) throws DAOException;
 
    public TreeSet<String> getSchemaVersions() ;

    public TreeSet<String> getDBVersions() ;

    public Gene getGeneByID(Integer id) throws DAOException ;

    public Gene getGeneByID(Integer id, String dbVersion) throws DAOException ;

    public Gene getGeneByStableID(String stableID) throws DAOException ;

    public Gene getGeneByStableID(String stableID, String dbVersion) throws DAOException ;

    public EnsemblComparaDivision getComparaDivision() ;

    public String getAssembly() ;

    public String getAssembly(String dbVersion) ;

}
