/**
 * Copyright (C) 2010 The Roslin Institute <contact andy.law@roslin.ed.ac.uk>
 *
 * This file is part of the Ensembl Java API demonstration project developed by the
 * Bioinformatics Group at The Roslin Institute, The Royal (Dick) School of
 * Veterinary Studies, University of Edinburgh.
 *
 * This is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License (version 3) as published by
 * the Free Software Foundation.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * in this software distribution. If not, see <http://www.gnu.org/licenses/gpl-3.0.html/>.
 */


package uk.ac.roslin.ensembl.model.database;

import java.util.HashMap;
import java.util.List;
import java.util.Set;
import uk.ac.roslin.ensembl.exception.DAOException;
import uk.ac.roslin.ensembl.model.ObjectType;
import uk.ac.roslin.ensembl.model.core.Chromosome;
import uk.ac.roslin.ensembl.model.core.CoordinateSystem;
import uk.ac.roslin.ensembl.model.core.Species;

/**
 *
 * @author tpaterso
 */
public interface CollectionCoreDatabase  extends CollectionDatabase, CoreDatabase {

    public Object getCoreFactory(Species species) ;

    //note these methods mirror the ones in SingleSpeciesCoreDatabase
    //but takes the required Species parameter

    public CoordinateSystem getChromosomeLevelCS(Species species) throws DAOException;

    public CoordinateSystem getSequenceLevelCS(Species species) throws DAOException;

    public CoordinateSystem getTopLevelCS(Species species) throws DAOException;

    public CoordinateSystem getCSByID(Species species, Integer id) throws DAOException ;

    public CoordinateSystem getBuildCoordSystem(Species species, String featureType) throws DAOException;

    public Set<? extends CoordinateSystem> getCSForFeature(Species sp, ObjectType feature) throws DAOException ;

    public void setBuildLevels(Species species, HashMap<String,String> keyedValues);

    public String getBuildLevel(Species species, String featureKey) throws DAOException ;

    public List<? extends ObjectType> getFeaturesForCS(Species sp, CoordinateSystem coordSys) throws DAOException;

    public Integer getMaxLengthForFeature(Species sp, ObjectType feature, CoordinateSystem cs) throws DAOException ;

    public void addFeatureCS(String featureType, Integer csID, Integer maxLength, Species sp) ;
    
    public Chromosome getChromosomeByName(Species species, String name) throws DAOException ;

    public List<? extends Chromosome> getChromosomes(Species species) throws DAOException ;
    
}
