/**
 * Copyright (C) 2010 The Roslin Institute <contact andy.law@roslin.ed.ac.uk>
 *
 * This file is part of the Ensembl Java API demonstration project developed by the
 * Bioinformatics Group at The Roslin Institute, The Royal (Dick) School of
 * Veterinary Studies, University of Edinburgh.
 *
 * This is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License (version 3) as published by
 * the Free Software Foundation.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * in this software distribution. If not, see <http://www.gnu.org/licenses/gpl-3.0.html/>.
 */

package uk.ac.roslin.ensembl.model.database;

import uk.ac.roslin.ensembl.model.core.CollectionOfSpecies;
import uk.ac.roslin.ensembl.model.core.CollectionSpecies;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Properties;
import java.util.TreeSet;

import uk.ac.roslin.ensembl.config.DBConnection.DataSource;
import uk.ac.roslin.ensembl.config.EnsemblComparaDivision;
import uk.ac.roslin.ensembl.exception.ConfigurationException;
import uk.ac.roslin.ensembl.exception.DAOException;
import uk.ac.roslin.ensembl.model.core.Species;

public interface Registry {

    public Properties getConfigProperties();

    public DataSource getDatasourceType() ;

    public String findMybatisSchemaForSchemaVersion(DatabaseType type, String schema_version);

    public String getMostRecentEnsemblVersion() ;

    //public DatabaseDAO getDatabaseDAO();

    public Database makeDatabase(String db_name) throws ConfigurationException;

    public ComparisonDatabase getComparaDatabase(EnsemblComparaDivision comparaDivision, String db_version);

    public Database getDatabase(String species_name_or_alias) ;

    public Database getDatabase(String species_name_or_alias, String db_version) ;

    public Database getDatabase(String species_name_or_alias, DatabaseType database_type, String db_version) ;

    public Database getDatabaseForFullName(String fullname) ;

    public TreeSet<? extends Database> getDatabasesByType(DatabaseType type) ;


    public Species getSpeciesByAlias(String alias_or_name) ;

    public Species getSpeciesByEnsemblName(String ensemblName) ;

    public Collection<? extends Species> getSpecies();

    public Collection<? extends CollectionSpecies> getCollectionSpecies();

    public void addSpecies(String dbStyleName, Species species ) ;

    public CollectionOfSpecies getCollection(String name, String dbVersion);

    public Collection<? extends CollectionOfSpecies> getCollectionRegistriesByName(String name) ;

    public Collection<? extends CollectionOfSpecies> getCollectionsByDBVersion(String dbVersion) ;

    public List<? extends CollectionOfSpecies> getCollections() ;

    public int getHighestEnsemblSchemaVersion() ;

    public int getHighestDBVersion() ;

    public String getMostRecentDBVersion();

    //handling aliases

    public String getEnsemblNameForAlias(String alias);

    public boolean addSessionAlias(String ensembl_genus_species_name, String alias) throws Exception;

    //Reporting

    public StringBuffer printRegistryReport();

    public String getAssembly(SingleSpeciesDatabase database) throws DAOException ;

    public HashMap<CollectionSpecies,String> getAssemblies(CollectionDatabase database) throws DAOException ;

    public void setCoreDBBuildLevels(SingleSpeciesCoreDatabase database) throws DAOException ;


}
