/**
 * Copyright (C) 2010 The Roslin Institute <contact andy.law@roslin.ed.ac.uk>
 *
 * This file is part of the Ensembl Java API demonstration project developed by the
 * Bioinformatics Group at The Roslin Institute, The Royal (Dick) School of
 * Veterinary Studies, University of Edinburgh.
 *
 * This is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License (version 3) as published by
 * the Free Software Foundation.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * in this software distribution. If not, see <http://www.gnu.org/licenses/gpl-3.0.html/>.
 */

package uk.ac.roslin.ensembl.model.database;

import java.util.List;
import java.util.Set;
import uk.ac.roslin.ensembl.exception.DAOException;
import uk.ac.roslin.ensembl.model.ObjectType;
import uk.ac.roslin.ensembl.model.core.Chromosome;
import uk.ac.roslin.ensembl.model.core.CoordinateSystem;

/**
 *
 * @author tpaterso
 */
public interface SingleSpeciesCoreDatabase extends SingleSpeciesDatabase, CoreDatabase {

   public Object getCoreFactory() ;

    //note these methods are mirrored in CollectionCoreDatabase
    //where they take the required Species parameter

   public CoordinateSystem getTopLevelCoordSystem() throws DAOException;

   public CoordinateSystem getChromosomeLevelCoordSystem()throws DAOException;

   public CoordinateSystem getSequenceLevelCoordSystem() throws DAOException;
   
   public CoordinateSystem getCSByID(Integer id) throws DAOException;

   public CoordinateSystem getBuildCoordSystem(String featureType) throws DAOException ;

   public Set<? extends CoordinateSystem> getCSForFeature(ObjectType featureType) throws DAOException;

   public Chromosome getChromosomeByName(String name) throws DAOException ;

   public List<? extends Chromosome> getChromosomes() throws DAOException ;

   public void setBuildLevel(String featureKey, String level) ;

   public String getBuildLevel(String featureType) throws DAOException ;

   public List<? extends ObjectType> getFeaturesForCS(CoordinateSystem coordSys) throws DAOException ;

   public Integer getMaxLengthForFeature(ObjectType featureType, CoordinateSystem cs) throws DAOException ;

   public void addFeatureCS(String featureType, Integer csID, Integer maxLength) ;

}
