/**
 * Copyright (C) 2010 The Roslin Institute <contact andy.law@roslin.ed.ac.uk>
 *
 * This file is part of the Ensembl Java API demonstration project developed by the
 * Bioinformatics Group at The Roslin Institute, The Royal (Dick) School of
 * Veterinary Studies, University of Edinburgh.
 *
 * This is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License (version 3) as published by
 * the Free Software Foundation.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * in this software distribution. If not, see <http://www.gnu.org/licenses/gpl-3.0.html/>.
 */

package uk.ac.roslin.ensembl.test;

import java.util.ArrayList;
import java.util.List;
import org.apache.log4j.PropertyConfigurator;
import uk.ac.roslin.ensembl.config.DBConnection.DataSource;
import uk.ac.roslin.ensembl.config.EnsemblCoordSystemType;
import uk.ac.roslin.ensembl.dao.database.DBRegistry;
import uk.ac.roslin.ensembl.dao.database.DBSpecies;
import uk.ac.roslin.ensembl.dao.factory.DAOCoreFactory;
import uk.ac.roslin.ensembl.datasourceaware.core.DAChromosome;
import uk.ac.roslin.ensembl.datasourceaware.core.DACoreObject;
import uk.ac.roslin.ensembl.datasourceaware.core.DADNASequence;
import uk.ac.roslin.ensembl.datasourceaware.core.DAGene;
import uk.ac.roslin.ensembl.model.core.DNASequence;

/**
 *
 * @author tpaterso
 */
public class ChromosomeCacheTest {

    public static void main(String[] args) throws Exception {

        PropertyConfigurator.configure("log4j.properties");
        DBRegistry eReg = new DBRegistry(DataSource.ENSEMBLDB);
        DBSpecies sp = eReg.getSpeciesByAlias("human");

        DAGene g = sp.getGeneByStableID("ENSG00000153551");
        DAChromosome c1 = (DAChromosome) g.getMappings(EnsemblCoordSystemType.chromosome).first().getTarget();
        System.out.println("first  fetch of chr: "+c1.getHashID() +" : "+c1.hashCode());

        DAChromosome c2 = sp.getChromosomeByName(c1.getChromosomeName());
        System.out.println("second fetch of chr: "+c2.getHashID() +" : "+c2.hashCode());

        DAGene g2 = sp.getGeneByStableID("ENSG00000170293");
        DAChromosome c3 = (DAChromosome) g2.getMappings(EnsemblCoordSystemType.chromosome).first().getTarget();
        System.out.println("third  fetch of chr: "+c3.getHashID() +" : "+c3.hashCode());

        DAChromosome c4 = sp.getChromosomeByName("4");
        System.out.println("first  fetch of chr: "+c4.getHashID() +" : "+c4.hashCode());

        DAGene g3 = sp.getGeneByStableID("ENSG00000186777");
        DAChromosome c5 = (DAChromosome) g3.getMappings(EnsemblCoordSystemType.chromosome).first().getTarget();
        System.out.println("second fetch of chr: "+c5.getHashID() +" : "+c5.hashCode());

        DADNASequence s = (DADNASequence) g3.getDaoFactory().getSequenceDAO().getSequenceByID(27524);
        System.out.println("third  fetch of chr: "+s.getHashID() +" : "+s.hashCode());

        DADNASequence s1 = new DADNASequence(g3.getDaoFactory());
        s1.setId(27524);
        System.out.println("s1 type before validation - should be DASequence: "
                +s1.getClass().getSimpleName() +" : "+s1.getHashID() +" : "+s1.hashCode());

        s1 = (DADNASequence) g3.getDaoFactory().getSequenceDAO().getValidatedSequence(s1);

        System.out.println("s1 type after validation - should be DAChromosome: "
                +s1.getClass().getSimpleName() +" : "+s1.getHashID() +" : "+s1.hashCode() );

        DADNASequence s2 = (DADNASequence) g3.getDaoFactory().getSequenceDAO().getSequenceByID(27524);

        System.out.println("s2 type before validation- should be a chromosome: "
                +s2.getClass().getSimpleName()+" : "+s2.getHashID() +" : "+s2.hashCode() );

        s2 = (DADNASequence) g3.getDaoFactory().getSequenceDAO().getValidatedSequence(s2);
        System.out.println("s2 type after validation- should still be a chromosome: "
                +s2.getClass().getSimpleName()+" : "+s2.getHashID() +" : "+s2.hashCode());


        DAOCoreFactory f = g3.getDaoFactory();

        DADNASequence s4 = new DADNASequence(f);
        s4.setId(27524);
        DADNASequence s5 = new DADNASequence(f);
        s5.setId(54919);
        DADNASequence s6 = new DADNASequence(f);
        DADNASequence s8 = new DADNASequence(f);
        s8.setId(-999);

        List<DNASequence> test = new ArrayList<DNASequence>();
        test.add(s4);
        test.add(s5);
        test.add(s6);
        test.add(s8);

        System.out.println("");
        System.out.println("before");
        for (DNASequence x : test) {
            if (x == null) {
                System.out.println("null object");
            }
            System.out.println(x.getId()+" : "+x.getClass().getSimpleName()+" : "+x.getHashID()+" : "+x.hashCode());
        }

        test =  (List<DNASequence> ) f.getSequenceDAO().getValidatedSequences(test);

        System.out.println("");
        System.out.println("after");
        for (DNASequence x : test) {
            System.out.println(x.getId()+" : "+x.getClass().getSimpleName()+" : "+x.getHashID()+" : "+x.hashCode());
        }


        System.out.println("\n\n*************************\nCOMPLETED FUNCTIONAL TEST\n*************************\n");
    }


}
