/**
 * Copyright (C) 2010 The Roslin Institute <contact andy.law@roslin.ed.ac.uk>
 *
 * This file is part of the Ensembl Java API demonstration project developed by the
 * Bioinformatics Group at The Roslin Institute, The Royal (Dick) School of
 * Veterinary Studies, University of Edinburgh.
 *
 * This is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License (version 3) as published by
 * the Free Software Foundation.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * in this software distribution. If not, see <http://www.gnu.org/licenses/gpl-3.0.html/>.
 */
package uk.ac.roslin.ensembl.test;

import java.util.List;
import org.apache.log4j.PropertyConfigurator;
import uk.ac.roslin.ensembl.config.DBConnection.DataSource;
import uk.ac.roslin.ensembl.config.EnsemblComparaDivision;
import uk.ac.roslin.ensembl.dao.database.DBCollectionSpecies;
import uk.ac.roslin.ensembl.dao.database.DBRegistry;
import uk.ac.roslin.ensembl.dao.database.DBSpecies;
import uk.ac.roslin.ensembl.model.Mapping;
import uk.ac.roslin.ensembl.model.MappingSet;
import uk.ac.roslin.ensembl.datasourceaware.compara.DAHomologyPairRelationship;
import uk.ac.roslin.ensembl.datasourceaware.core.DADNASequence;
import uk.ac.roslin.ensembl.datasourceaware.core.DAGene;
import uk.ac.roslin.ensembl.exception.DAOException;
import uk.ac.roslin.ensembl.model.core.Species;

/**
 *
 * @author tpaterso
 */
public class ComparaTest2 {

    public static void main(String[] args) throws Exception {

        PropertyConfigurator.configure("log4j.properties");

        DBRegistry eReg = new DBRegistry(DataSource.ENSEMBLGENOMES);

//            DBComparisonDatabase db = (DBComparisonDatabase) eReg.getDatabase(null, EnsemblDBType.compara, null);
//            DBDAOComparaFactory fact = (DBDAOComparaFactory) db.getFactory();

        DBSpecies sp =  eReg
                .getSpeciesByAlias("Bacillus pumilus (strain SAFR-032)");


        DAGene g = sp.getGeneByStableID("EBBACG00000064655");

        String csv = new String();

        List<DAHomologyPairRelationship> out = g.getHomologies();
        System.out.println(g.getSpecies().getCommonName() + " gene " + g.getStableID() + " has " + out.size() + " homologies.");
        System.out.println("_____________________________________________\r\n");
        for (DAHomologyPairRelationship h : out) {

            System.out.print(h.getTargetProperties().getSpeciesName());
            System.out.print(" gene: " + h.getTarget().getStableID());
            System.out.println(" [" + h.getType().toString() + "] (last common ancestor: " + h.getLastCommonAncestor() + ")");

            System.out.println("MAPPING DATA IN COMPARA");
            System.out.println("'chromosome' name: " + h.getTargetProperties().getSequenceName()
                    + " [" + h.getTargetProperties().getCoords().toString() + "]");

            csv += h.getTargetProperties().getSequenceName() + ","
                    + h.getTargetProperties().getCoords().getStart() + ","
                    + h.getTargetProperties().getCoords().getEnd() + ","
                    + h.getTargetProperties().getCoords().getStrand().toString() + ",";

            System.out.println("MAPPING DATA LAZY LOADED FROM CORE");

            MappingSet m = null;

            Species sppp = h.getSource().getRegistry().getSpeciesByAlias(h.getTargetProperties().getSpeciesName());

            try {
                m = h.getTarget().getAnnotationLevelMappings();
            } catch (DAOException dAOException) {
            }
            //m = h.getTarget().getBuildLevelMappings();
            //m = h.getTarget().getTopLevelMappings();

            System.out.print("ANNOTATION LEVEL: ");
            if (m != null && !m.isEmpty()) {
                for (Mapping mp : m) {
                    System.out.println(mp.getTarget().getClass().getSimpleName()
                            + " name: "
                            + ((DADNASequence) mp.getTarget()).getName()
                            + " id: " + mp.getTarget().getId() + " ["
                            + mp.getTargetCoordinates().toString() + "]");

                    if (!h.getTargetProperties().getSequenceName().contentEquals(((DADNASequence) mp.getTarget()).getName())) {
                        System.out.println("\n\n\n*********ERROR in name");
                    }
                    if (h.getTargetProperties().getCoords().getStart() - mp.getTargetCoordinates().getStart() != 0) {
                        System.out.println("\n\n\n*********ERROR in start coord");
                    }
                    if (h.getTargetProperties().getCoords().getEnd() - mp.getTargetCoordinates().getEnd() != 0) {
                        System.out.println("\n\n\n*********ERROR in end coord");
                    }
                    if (!h.getTargetProperties().getCoords().getStrand().equals(mp.getTargetCoordinates().getStrand())) {
                        System.out.println("\n\n\n*********ERROR in strande");
                    }

                    csv += ((DADNASequence) mp.getTarget()).getName() + ","
                            + mp.getTargetCoordinates().getStart() + ","
                            + mp.getTargetCoordinates().getEnd() + ","
                            + mp.getTargetCoordinates().getStrand().toString() + "\n\r";
                }
            } else {
                System.out.println("");
            }
            //m = h.getTarget().getAnnotationLevelMappings();
try {
                m = h.getTarget().getBuildLevelMappings();
            } catch (DAOException dAOException) {
            }
            //m = h.getTarget().getTopLevelMappings();

            System.out.print("BUILD LEVEL: ");
            if (m != null && !m.isEmpty()) {

                for (Mapping mp : m) {
                    System.out.println(mp.getTarget().getClass().getSimpleName()
                            + " name: "
                            + ((DADNASequence) mp.getTarget()).getName()
                            + " id: " + mp.getTarget().getId() + " ["
                            + mp.getTargetCoordinates().toString() + "]");

                    if (!h.getTargetProperties().getSequenceName().contentEquals(((DADNASequence) mp.getTarget()).getName())) {
                        System.out.println("\n\n\n*********ERROR in name");
                    }
                    if (h.getTargetProperties().getCoords().getStart() - mp.getTargetCoordinates().getStart() != 0) {
                        System.out.println("\n\n\n*********ERROR in start coord");
                    }
                    if (h.getTargetProperties().getCoords().getEnd() - mp.getTargetCoordinates().getEnd() != 0) {
                        System.out.println("\n\n\n*********ERROR in end coord");
                    }
                    if (!h.getTargetProperties().getCoords().getStrand().equals(mp.getTargetCoordinates().getStrand())) {
                        System.out.println("\n\n\n*********ERROR in strande");
                    }

                    csv += ((DADNASequence) mp.getTarget()).getName() + ","
                            + mp.getTargetCoordinates().getStart() + ","
                            + mp.getTargetCoordinates().getEnd() + ","
                            + mp.getTargetCoordinates().getStrand().toString() + "\n\r";
                }
            } else {
                System.out.println("");
            }
            //m = h.getTarget().getAnnotationLevelMappings();
            //m = h.getTarget().getBuildLevelMappings();
try {
                m = h.getTarget().getTopLevelMappings();
            } catch (DAOException dAOException) {
            }
            System.out.print("TOP LEVEL: ");
            if (m != null && !m.isEmpty()) {
                for (Mapping mp : m) {
                    System.out.println(mp.getTarget().getClass().getSimpleName()
                            + " name: "
                            + ((DADNASequence) mp.getTarget()).getName()
                            + " id: " + mp.getTarget().getId() + " ["
                            + mp.getTargetCoordinates().toString() + "]");

                    if (!h.getTargetProperties().getSequenceName().contentEquals(((DADNASequence) mp.getTarget()).getName())) {
                        System.out.println("\n\n\n*********ERROR in name");
                    }
                    if (h.getTargetProperties().getCoords().getStart() - mp.getTargetCoordinates().getStart() != 0) {
                        System.out.println("\n\n\n*********ERROR in start coord");
                    }
                    if (h.getTargetProperties().getCoords().getEnd() - mp.getTargetCoordinates().getEnd() != 0) {
                        System.out.println("\n\n\n*********ERROR in end coord");
                    }
                    if (!h.getTargetProperties().getCoords().getStrand().equals(mp.getTargetCoordinates().getStrand())) {
                        System.out.println("\n\n\n*********ERROR in strande");
                    }

                    csv += ((DADNASequence) mp.getTarget()).getName() + ","
                            + mp.getTargetCoordinates().getStart() + ","
                            + mp.getTargetCoordinates().getEnd() + ","
                            + mp.getTargetCoordinates().getStrand().toString() + "\n\r";
                }
            } else {
                System.out.println("");
            }
            System.out.println("___________________________________________________");
            System.out.println("");
        }
        //  System.out.println(csv);
        System.out.println("\n\n*************************\nCOMPLETED FUNCTIONAL TEST\n*************************\n");

    }
}
