/**
 * Copyright (C) 2010 The Roslin Institute <contact andy.law@roslin.ed.ac.uk>
 *
 * This file is part of the Ensembl Java API demonstration project developed by the
 * Bioinformatics Group at The Roslin Institute, The Royal (Dick) School of
 * Veterinary Studies, University of Edinburgh.
 *
 * This is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License (version 3) as published by
 * the Free Software Foundation.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * in this software distribution. If not, see <http://www.gnu.org/licenses/gpl-3.0.html/>.
 */
package uk.ac.roslin.ensembl.test;

import java.util.HashMap;
import java.util.List;
import org.apache.log4j.PropertyConfigurator;
import uk.ac.roslin.ensembl.config.DBConnection.DataSource;
import uk.ac.roslin.ensembl.config.EnsemblComparaDivision;
import uk.ac.roslin.ensembl.dao.compara.HomologyDAO;
import uk.ac.roslin.ensembl.dao.database.DBRegistry;
import uk.ac.roslin.ensembl.dao.database.DBSpecies;
import uk.ac.roslin.ensembl.datasourceaware.compara.DAHomologyPairRelationship;
import uk.ac.roslin.ensembl.model.Mapping;
import uk.ac.roslin.ensembl.model.MappingSet;
import uk.ac.roslin.ensembl.datasourceaware.core.DAChromosome;
import uk.ac.roslin.ensembl.datasourceaware.core.DADNASequence;
import uk.ac.roslin.ensembl.datasourceaware.core.DAGene;
import uk.ac.roslin.ensembl.model.Coordinate;

/**
 *
 * @author tpaterso
 */
public class ComparaTest5 {

    public static void main(String[] args) throws Exception {

        PropertyConfigurator.configure("log4j.properties");

        DBRegistry eReg = new DBRegistry(DataSource.ENSEMBLDB);

        DBSpecies finch = eReg.getSpeciesByAlias("zebra finch");
        DBSpecies chicken = eReg.getSpeciesByAlias("chicken");

        DAChromosome cChr3 = chicken.getChromosomeByName("3");
        HomologyDAO dao = cChr3.getComparaFactory().getHomologyDAO();

        HashMap<DADNASequence, MappingSet> syntenies =
                (HashMap<DADNASequence, MappingSet>)
                    //dao.getRegionsOfConservedSynteny(cChr3, new Coordinate(1, 13000,1), finch);
                    dao.getRegionsOfConservedSynteny(cChr3, new Coordinate(1, 1000000), finch);

        System.out.println("SYNTENIC REGIONS in Zebra Finch of Chick Chromosome 3\n__________________________________________________\r\n");

        int count =1;

        for ( DADNASequence dna: syntenies.keySet()) {


            System.out.println(count++ + ". "+dna.getCoordSystem().getType().toString()
                        +" "
                        +dna.getName()+ " "+syntenies.get(dna).getExtent().toString());

            for (Mapping mp : syntenies.get(dna)) {
                System.out.println("\t"+((DAGene) mp.getTarget()).getStableID()
                        +" : "+mp.getSourceCoordinates().toString());
            }

        }

        System.out.println("\nList of genes on chick chr3 (1-1000000), with finch homologues...\n");

       List<DAGene> genes = cChr3.getGenesOnRegion(1, 1000000);

        for (DAGene g : genes) {

            System.out.println(g.getSpecies().getCommonName()+" Gene "+g.getStableID()
                    +" "+finch.getCommonName()+ " homologues:");



            for (DAHomologyPairRelationship hpr : g.getHomologies(finch)) {

                DAGene target = hpr.getTarget();
                MappingSet mappings = target.getAnnotationLevelMappings();

                for (Mapping m : mappings) {

                System.out.println("\t"+hpr.getType().toString()+" : "
                        +target.getStableID()+" ["+m.getTargetType().toString()
                        +" "+((DADNASequence) m.getTarget()).getName()
                        +":"+m.getTargetCoordinates().toString()+"]");
                }
            }
        }
        System.out.println("\n\n*************************\nCOMPLETED FUNCTIONAL TEST\n*************************\n");

    }

}
