/**
 * Copyright (C) 2010 The Roslin Institute <contact andy.law@roslin.ed.ac.uk>
 *
 * This file is part of the Ensembl Java API demonstration project developed by the
 * Bioinformatics Group at The Roslin Institute, The Royal (Dick) School of
 * Veterinary Studies, University of Edinburgh.
 *
 * This is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License (version 3) as published by
 * the Free Software Foundation.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * in this software distribution. If not, see <http://www.gnu.org/licenses/gpl-3.0.html/>.
 */
package uk.ac.roslin.ensembl.test;

import java.util.HashMap;
import java.util.List;
import org.apache.log4j.PropertyConfigurator;
import uk.ac.roslin.ensembl.config.DBConnection.DataSource;
import uk.ac.roslin.ensembl.config.EnsemblCoordSystemType;
import uk.ac.roslin.ensembl.config.EnsemblComparaDivision;
import uk.ac.roslin.ensembl.dao.database.DBRegistry;
import uk.ac.roslin.ensembl.dao.database.DBSpecies;
import uk.ac.roslin.ensembl.model.Mapping;
import uk.ac.roslin.ensembl.model.MappingSet;
import uk.ac.roslin.ensembl.datasourceaware.compara.DAHomologyPairRelationship;
import uk.ac.roslin.ensembl.datasourceaware.core.DAChromosome;
import uk.ac.roslin.ensembl.datasourceaware.core.DADNASequence;
import uk.ac.roslin.ensembl.datasourceaware.core.DAGene;
import uk.ac.roslin.ensembl.model.Coordinate;

/**
 *
 * @author tpaterso
 */
public class ComparaTest8 {

    public static void main(String[] args) throws Exception {

        PropertyConfigurator.configure("log4j.properties");

        DBRegistry eReg = new DBRegistry(DataSource.ENSEMBLDB);
        DBSpecies cow = eReg.getSpeciesByAlias("cow");
        DBSpecies pig = eReg.getSpeciesByAlias("pig");
        DAChromosome chr1 = cow.getChromosomeByName("1");

        Coordinate coord = new Coordinate(40000, 100000);

        HashMap<DADNASequence, MappingSet> out = chr1.getRegionsOfConservedSynteny(coord, pig);

        for (DAGene g : chr1.getGenesOnRegion(coord)) {
            System.out.println("Gene: " + g.getStableID()+ "(hashcode:" + g.hashCode() + ")");
            System.out.println("has " + g.getHomologiesWithoutLazyLoad().size() + " homol relationships");
            for (DAHomologyPairRelationship hpr : g.getHomologiesWithoutLazyLoad()) {
                if (hpr.getTarget().getStableID().equals("ENSSSCG00000014461")) {
                    DAGene gene = hpr.getTarget();
                    System.out.println("\trelationship to ENSSSCG00000014461 (hashcode: " + gene.hashCode() + ")");
                    System.out.println("\tENSSSCG00000014461 has " + gene.getHomologiesWithoutLazyLoad().size() + " homol relationships");
                    for (DAHomologyPairRelationship hp : gene.getHomologiesWithoutLazyLoad()) {
                        System.out.println("\ttarget: " + hp.getTarget().getStableID()+"(hashcode: " + hp.getTarget().hashCode() + ")");
                    }

                }


            }

        System.out.println("");
        }
        System.out.println("\n\n*************************\nCOMPLETED FUNCTIONAL TEST\n*************************\n");

    }
}
