/**
 * Copyright (C) 2010 The Roslin Institute <contact andy.law@roslin.ed.ac.uk>
 *
 * This file is part of the Ensembl Java API demonstration project developed by the
 * Bioinformatics Group at The Roslin Institute, The Royal (Dick) School of
 * Veterinary Studies, University of Edinburgh.
 *
 * This is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License (version 3) as published by
 * the Free Software Foundation.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * in this software distribution. If not, see <http://www.gnu.org/licenses/gpl-3.0.html/>.
 */
package uk.ac.roslin.ensembl.test;

import java.io.File;
import java.util.List;
import org.apache.log4j.PropertyConfigurator;
import uk.ac.roslin.ensembl.config.EnsemblDBType;
import uk.ac.roslin.ensembl.config.FeatureType;
import uk.ac.roslin.ensembl.config.RegistryConfiguration;
import uk.ac.roslin.ensembl.dao.database.DBDatabase;
import uk.ac.roslin.ensembl.dao.database.DBRegistry;
import uk.ac.roslin.ensembl.dao.database.DBSingleSpeciesCoreDatabase;
import uk.ac.roslin.ensembl.dao.database.DBSpecies;
import uk.ac.roslin.ensembl.datasourceaware.core.DAChromosome;
import uk.ac.roslin.ensembl.exception.DAOException;
import uk.ac.roslin.ensembl.model.Mapping;
import uk.ac.roslin.ensembl.model.MappingSet;
import uk.ac.roslin.ensembl.model.core.Chromosome;
import uk.ac.roslin.ensembl.model.core.Gene;
import uk.ac.roslin.ensembl.model.core.ProteinFeature;
import uk.ac.roslin.ensembl.model.core.Species;
import uk.ac.roslin.ensembl.model.database.Registry;
import uk.ac.roslin.ensembl.model.database.SingleSpeciesCoreDatabase;

/**
 *
 * @author tpaterso
 */
public class LocalConfigTest {

    public static void main(String[] args) throws Exception {

        PropertyConfigurator.configure("log4j.properties");

        File fileMR = new File("src/main/resources/example_local_configuration.properties");
        File fileDB = new File("src/main/resources/example_archive_config.properties");
        //DBRegistry eReg = DBRegistryFactory.createRegistryWithLocalDBConnectionAndMappingRulesFiles(fileDB, fileMR);
        RegistryConfiguration conf = new RegistryConfiguration();
        conf.setDBByFile(fileDB);
        conf.setSchemaByFile(fileMR);
        DBRegistry eReg = new DBRegistry(conf,true);
        Species sp = eReg.getSpeciesByAlias("chicken");

        Chromosome chr = sp.getChromosomeByName("25", "47");

        System.out.println("Chromosome: "+chr.getChromosomeName()+",  length: "+chr.getLength());

        System.out.println(eReg.getBriefRegistryReport());


        System.out.println("ensemblarchives registry report: ");
        System.out.println("******************************* ");


        System.out.println(eReg.printRegistryReport().toString());


       System.out.println("species count : "+eReg.getSpecies().size());
        int i = 1;
        for (DBSpecies s: eReg.getSpecies()) {
            System.out.println(i++ +": "+s.getSpeciesBinomial());

            for(DBDatabase d : s.getDatabasesByType(EnsemblDBType.core)) {

                System.out.println(d.getdBName()+ " : "+d.getBuild());
            }


        }

        String human = "human"; // relies on alias look up
        String release47 = "47";
        int translationID = 1;

        SingleSpeciesCoreDatabase db = (SingleSpeciesCoreDatabase) eReg.getDatabase(human);

        //DBSpecies homo = (DBSpecies) ensembldbRegistry.getSpeciesByAlias("human");


       if (db!=null) {

        System.out.println("current human top level: "+db.getTopLevelCoordSystem().getId());
        System.out.println("current human chromosome level: "+db.getChromosomeLevelCoordSystem().getId());
        System.out.println("current human sequence level: "+db.getSequenceLevelCoordSystem().getId());

       }

        System.out.println("Get features from Ensembl release 47 for human translation id ="+translationID);

        List<? extends ProteinFeature> result47 = ((DBSingleSpeciesCoreDatabase) eReg
                .getDatabase(human, EnsemblDBType.core, release47))
                .getCoreFactory()
                    .getProteinFeatureDAO()
                        .getProteinFeaturesByTranslationID(translationID);


        System.out.println("retrieved: "+result47.size()+" features.");

        System.out.println("\n\n*************************\nCOMPLETED FUNCTIONAL TEST\n*************************\n");

    }
}
