/**
 * Copyright (C) 2010 The Roslin Institute <contact andy.law@roslin.ed.ac.uk>
 *
 * This file is part of the Ensembl Java API demonstration project developed by the
 * Bioinformatics Group at The Roslin Institute, The Royal (Dick) School of
 * Veterinary Studies, University of Edinburgh.
 *
 * This is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License (version 3) as published by
 * the Free Software Foundation.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * in this software distribution. If not, see <http://www.gnu.org/licenses/gpl-3.0.html/>.
 */
package uk.ac.roslin.ensembl.test;

import java.util.List;
import java.util.Set;
import java.util.TreeSet;
import org.apache.log4j.PropertyConfigurator;
import uk.ac.roslin.ensembl.config.DBConnection.DataSource;
import uk.ac.roslin.ensembl.datasourceaware.DAObject;
import uk.ac.roslin.ensembl.datasourceaware.core.DAChromosome;
import uk.ac.roslin.ensembl.datasourceaware.core.DAGene;
import uk.ac.roslin.ensembl.datasourceaware.core.DATranslation;
import uk.ac.roslin.ensembl.dao.database.DBSingleSpeciesCoreDatabase;
import uk.ac.roslin.ensembl.exception.DAOException;
import uk.ac.roslin.ensembl.model.Coordinate.Strand;
import uk.ac.roslin.ensembl.model.Mapping;
import uk.ac.roslin.ensembl.model.core.ProteinFeature;
import uk.ac.roslin.ensembl.model.core.Species;
import uk.ac.roslin.ensembl.model.database.SingleSpeciesCoreDatabase;
import uk.ac.roslin.ensembl.config.EnsemblCoordSystemType;
import uk.ac.roslin.ensembl.config.EnsemblDBType;
import uk.ac.roslin.ensembl.config.FeatureType;
import uk.ac.roslin.ensembl.config.RegistryConfiguration;
import uk.ac.roslin.ensembl.dao.database.DBRegistry;
import uk.ac.roslin.ensembl.model.MappingSet;
import uk.ac.roslin.ensembl.model.ObjectType;
import uk.ac.roslin.ensembl.model.core.Chromosome;
import uk.ac.roslin.ensembl.model.core.CoordinateSystem;
import uk.ac.roslin.ensembl.model.core.Gene;
import uk.ac.roslin.ensembl.model.core.Translation;
import uk.ac.roslin.ensembl.model.database.Registry;

public class QuickTest {

    public static void main(String[] args) throws Exception {

        PropertyConfigurator.configure("log4j.properties");

        //Registry eReg = DBRegistryFactory.createDefaultRegistry(DataSource.ENSEMBLDB);
        //Registry eReg = new DBRegistry(new RegistryConfiguration(DataSource.ENSEMBLDB));
        Registry eReg = new DBRegistry(DataSource.ENSEMBLDB);

        Species sp = eReg.getSpeciesByAlias("chicken");

        Chromosome chrDefault = sp.getChromosomeByName("25");

        List<Gene> genes = null;
        MappingSet mappings = null;
            try {

                genes =(List<Gene>)  chrDefault.getGenesOnRegion(1, chrDefault.getLength());
                mappings = chrDefault.getMappings(FeatureType.gene);
                System.out.println("No of Genes on default version: "+genes.size());
            } catch (DAOException ex) {
                System.out.println("Failed to retrieve genes for "
                        + ((DAChromosome) chrDefault).getSpecies()
                        + "chr " + chrDefault
                        + " db " + chrDefault.getDBVersion()
                        + "\r\n" + ex.getMessage());

            }


//        for (Integer version : eReg.getKnownSchemaVersions()) {
//
//            chrDefault = sp.getChromosomeByName("25",version.toString());
//
//            System.out.println("first 100 bp: "+chrDefault.getSequenceAsString(1, 100));
//
//            try {
//
//                genes = chrDefault.getGenesOnRegion(1, chrDefault.getLength());
//
//                System.out.println("No of Genes on version "+version+": "+genes.size());
//            } catch (DAOException ex) {
//                System.out.println("Failed to retrieve genes for "
//                        + chrDefault.getSpecies()
//                        + "chr " + chrDefault
//                        + " release " + chrDefault.getSchemaVersion()
//                        + "\r\n" + ex.getMessage());
//
//            }
//
//
//
//        }


         


           for (Mapping mp : mappings) {
                    System.out.println("genomeTrackName"+ "\t"
                            + mp.getSourceCoordinates().getStart() +"\t"
                            + mp.getSourceCoordinates().getEnd() +"\t"
                            +((DAGene) mp.getTarget()).getStableID() +"\t0\t"
                            + (mp.getSourceCoordinates().getStrand().equals(Strand.FORWARD_STRAND) ? "+" :
                                (mp.getSourceCoordinates().getStrand().equals(Strand.REVERSE_STRAND) ? "-" : "") )
                                +"\r\n");
           }

            System.out.println("");
            System.out.println("______________________________");
            System.out.println("");



        System.out.println(mappings.size()+" mappings");
        System.out.println(genes.size()+" genes");

        System.out.println("");

        int i = 1;
        TreeSet<Integer> mappedObjects = new TreeSet<Integer>();
        for (Mapping m: mappings) {
            mappedObjects.add(((DAObject)m.getTarget()).getId());
            System.out.println(i++ +" "+((DAObject)m.getTarget()).getId()+" "+m.getSourceCoordinates().getStart()+" - "+m.getSourceCoordinates().getEnd());
        }
        i = 1;
        TreeSet<Integer> genex = new TreeSet<Integer>();
        for (Gene g: genes) {
            genex.add(g.getId());
            System.out.println(i++ +" "+g.getId()+" mappings  "+g.getMappings().size()+" type "+ g.getType());
            try{
                System.out.println("DESCRIPTION: "+g.getDescription());
            System.out.println("XREF: "+((DAGene)g).getDisplayXRef().getDBName()+": "+((DAGene)g).getDisplayXRef().getDescription());
            } catch (Exception e) {}
        }

        System.out.println("");

            System.out.println("mappedObjects: "+mappedObjects.size());
            System.out.println("genes: "+genex.size());

        genex.removeAll(mappedObjects);
        System.out.println("extra genes: "+genex.size());
        for (Integer g: genex) {
            System.out.println("extra: "+g);
        }

        System.out.println("");

                    System.out.println("");
            System.out.println("______________________________");
            System.out.println("");

        System.out.println("CHROMOSOME " + chrDefault.getChromosomeName() + " FROM SPECIES: HashID: " + chrDefault.getHashID()
                + " VMID:" + chrDefault.hashCode());

        System.out.println("\n all current release chr..\n_______________________\n");

        for (Chromosome c : sp.getChromosomes().values()) {
            System.out.println("CHROMOSOME " + c.getChromosomeName() + " FROM SPECIES: HashID: " + c.getHashID()
                    + " VMID:" + c.hashCode());
        }

         System.out.println("\n all  release 58  chr..\n_______________________\n");
        for (Chromosome c : sp.getChromosomes("58").values()) {
            System.out.println("CHROMOSOME " + c.getChromosomeName() + " FROM SPECIES: HashID: " + c.getHashID()
                    + " VMID:" + c.hashCode());
        }


        SingleSpeciesCoreDatabase db = (SingleSpeciesCoreDatabase) eReg.getDatabase("chicken");

        System.out.println("\nChromosomal Features: ");
        for (ObjectType f : db.getFeaturesForCS(db.getChromosomeLevelCoordSystem())) {
            System.out.println("\t" + f.toString());
        }
        System.out.println("Top Level Features: ");
        for (ObjectType f : db.getFeaturesForCS(db.getTopLevelCoordSystem())) {
            System.out.println("\t" + f.toString());
        }
        System.out.println("Seq Level Features: ");
        for (ObjectType f : db.getFeaturesForCS(db.getSequenceLevelCoordSystem())) {
            System.out.println("\t" + f.toString());
        }

        System.out.println("\nCoord Systems\n___________\n");

        for (ObjectType t : FeatureType.getAllTypes()) {
            Set<? extends CoordinateSystem> s = db.getCSForFeature(t);

            if (s != null && !s.isEmpty()) {
                System.out.println(t.toString());
                for (CoordinateSystem cs : s) {
                    System.out.println("\tcs-id: " + cs.getId() + " - "
                            + "cs-type: " + cs.getType().toString());
                }
            } else {
                System.out.println(t.toString() + "\n\tNONE");
            }
        }

        CoordinateSystem geneBuildCS = db.getBuildCoordSystem(FeatureType.gene.toString());

        System.out.println("\n\nmax gene length: "+db.getMaxLengthForFeature(FeatureType.gene, geneBuildCS));

        Species hsa = eReg.getSpeciesByAlias("human");

        System.out.println("\nCURRENT VERSION OF ENSG00000196653\n_________________\n");

        Gene gene = hsa.getGeneByStableID("ENSG00000196653");
        System.out.println("gene name: " + gene.getDescription());
        System.out.println("transcript id: " + gene.getCanonicalTranscript().getId());

        Translation translation = gene.getCanonicalTranslation();
        System.out.println("translation id: " + translation.getId());
        System.out.println("translation hash: " + translation.getHashID());
        System.out.println("translation database: " + ((DATranslation)translation).getDaoFactory().getDatabaseName());
        System.out.println("ProteinFeatures: " + translation.getProteinFeatures().size());


        System.out.println("\n51 VERSION OF ENSG00000196653\n_________________\n");

        gene = hsa.getGeneByStableID("ENSG00000196653", "51");
        System.out.println("gene name: " + gene.getDescription());
        System.out.println("transcript id: " + gene.getCanonicalTranscript().getId());


        translation = gene.getCanonicalTranslation();
        System.out.println("translation id: " + translation.getId());
        System.out.println("translation hash: " + translation.getHashID());
        System.out.println("translation database: " + ((DATranslation)translation).getDaoFactory().getDatabaseName());
        System.out.println("ProteinFeatures: " + translation.getProteinFeatures().size());

        System.out.println("\n50 VERSION OF ENSG00000196653\n_________________\n");

        gene = hsa.getGeneByStableID("ENSG00000196653", "50");
        System.out.println("gene name: " + gene.getDescription());
        System.out.println("transcript id: " + gene.getCanonicalTranscript().getId());

        translation = gene.getCanonicalTranslation();
        System.out.println("translation id: " + translation.getId());
        System.out.println("translation hash: " + translation.getHashID());
        System.out.println("translation database: " + ((DATranslation)translation).getDaoFactory().getDatabaseName());
        System.out.println("ProteinFeatures: " + translation.getProteinFeatures().size());

 System.out.println("Gene -> Chromosome mappings ");

MappingSet ms = gene.getMappings(EnsemblCoordSystemType.chromosome);



 for (Mapping m : ms ) {
     System.out.println(m.getTarget().getClass().getSimpleName()
             +" chr name: "
             +( (DAChromosome) m.getTarget()).getChromosomeName()
             +" id: "+m.getTarget().getId() );
     System.out.println("CHROMOSOME FROM GENE MAPPING: HashID: "+m.getTarget().getHashID()
             +" VMID:"+ m.getTarget().hashCode());
 }


  for (Gene g: chrDefault.getGenesOnRegion(3080, 12536)) {

        System.out.println("GENE FROM CHROMOSOME: HashID: "+g.getHashID()
                +" VMID:"+g.hashCode());

  }

  System.out.println("Chromosome -> gene mappings ");

 MappingSet ms2 = chrDefault.getMappings(FeatureType.gene) ;


 for (Mapping m :ms2 ) {
     System.out.println(m.getTarget().getClass().getSimpleName()
             +" gene stableID: "
             +( (DAGene) m.getTarget()).getStableID()
             +" id: "+m.getTarget().getId() );
        System.out.println("Gene on chromosome: HashID: "+m.getTarget().getHashID()
                +" VMID:"+m.getTarget().hashCode());
 }

 System.out.println("Gene -> Chromosome mappings ");


MappingSet ms1 =  gene.getMappings(EnsemblCoordSystemType.chromosome);

 for (Mapping m : ms1 ) {
     System.out.println(m.getTarget().getClass().getSimpleName()
             +" chr name: "
             +( (DAChromosome) m.getTarget()).getChromosomeName()
             +" id: "+m.getTarget().getId() );
      System.out.println("Chr on Gene: HashID: "+m.getTarget().getHashID()
              +" VMID:"+m.getTarget().hashCode());}



        String human = "human"; // relies on alias look up
        String release51 = "51";
        String release50 = "50";

        int translationID = 70113;

        System.out.println("Get features from Ensembl release 50 for human translation id ="+translationID);

        List<? extends ProteinFeature> result50 = ((DBSingleSpeciesCoreDatabase) eReg
                .getDatabase(human, EnsemblDBType.core, release50))
                .getCoreFactory()
                    .getProteinFeatureDAO()
                        .getProteinFeaturesByTranslationID(translationID);


        System.out.println("retrieved: "+result50.size()+" features.");

        System.out.println("Get features from Ensembl release 51 for human translation id ="+translationID);



        List<? extends ProteinFeature> result51 = ((DBSingleSpeciesCoreDatabase) eReg
              .getDatabase(human, EnsemblDBType.core, release51))
                .getCoreFactory()
                    .getProteinFeatureDAO()
                        .getProteinFeaturesByTranslationID(translationID);


        System.out.println("retrieved: "+result51.size()+" features. Using the new alias.");


        System.out.println("\n\n*************************\nCOMPLETED FUNCTIONAL TEST\n*************************\n");




    }
}
