/**
 * Copyright (C) 2010 The Roslin Institute <contact andy.law@roslin.ed.ac.uk>
 *
 * This file is part of the Ensembl Java API demonstration project developed by the
 * Bioinformatics Group at The Roslin Institute, The Royal (Dick) School of
 * Veterinary Studies, University of Edinburgh.
 *
 * This is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License (version 3) as published by
 * the Free Software Foundation.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * in this software distribution. If not, see <http://www.gnu.org/licenses/gpl-3.0.html/>.
 */
package uk.ac.roslin.ensembl.test;


import org.apache.log4j.PropertyConfigurator;
import uk.ac.roslin.ensembl.config.DBConnection.DataSource;
import uk.ac.roslin.ensembl.config.EnsemblCoordSystemType;
import uk.ac.roslin.ensembl.datasourceaware.core.DAGene;
import uk.ac.roslin.ensembl.model.Mapping;
import uk.ac.roslin.ensembl.dao.database.DBRegistry;
import uk.ac.roslin.ensembl.dao.database.DBSpecies;
import uk.ac.roslin.ensembl.datasourceaware.core.DADNASequence;
import uk.ac.roslin.ensembl.datasourceaware.core.DATranscript;


public class QuickTest2 {

    public static void main(String[] args) throws Exception {

        PropertyConfigurator.configure("log4j.properties");

        //Registry eReg = DBRegistryFactory.createDefaultRegistry(DataSource.ENSEMBLDB);
        //Registry eReg = new DBRegistry(new RegistryConfiguration(DataSource.ENSEMBLDB));
        DBRegistry eReg = new DBRegistry(DataSource.ENSEMBLDB);

        DBSpecies sp = eReg.getSpeciesByAlias("cow");
        DAGene gene = sp.getGeneByStableID("ENSBTAG00000021527");
        //DAGene gene = sp.getGeneByStableID("ENSG00000139618");
        Mapping mapping = gene.getAnnotationLevelMappings().first();


        System.out.println("Gene: "+ gene.getStableID());
        System.out.println("\tchr start: "+mapping.getTargetCoordinates().getStart());
        System.out.println("\tchr stop: "+mapping.getTargetCoordinates().getEnd());
        System.out.println("\tassembly: "+gene.getAssembly());
        System.out.println("\tdescription: "+ gene.getDescription());
        System.out.println("\tsymbol: "+gene.getDisplayName());
        System.out.println("\tstrand: "+mapping.getTargetCoordinates().getStrand());
        System.out.println("\ttaxonID: "+gene.getSpecies().getTaxonomyID());
        System.out.println("\tstatus: "+gene.getStatus());
        System.out.println("\ttype: "+ gene.getBiotype());

        System.out.println("\tTranscript Count: "+ gene.getTranscripts().size());
        for (DATranscript t : gene.getTranscripts()) {
            System.out.println("\t\tTranscript: "+ t.getStableID());
            System.out.println("\t\t "+ t.getDisplayName());
            System.out.println("\t\t "+ t.getStatus());
            System.out.println("\t\t "+ t.getBiotype());
            System.out.println("\t\t "+ t.getDescription());
            System.out.println("\t\t "+ t.getGene().getStableID());
            System.out.println("\t\tCanonical ?  "+ t.isCanonical());
            System.out.println("\t\tXREF: "+ t.getDisplayXRef().getDBName());
            System.out.println("\t\tXREF: "+ t.getDisplayXRef().getDisplayID());
            System.out.println("\t\tXREF: "+ t.getDisplayXRef().getInfoType());
            System.out.println("\t\tXREF: "+ t.getDisplayXRef().getInfo());
            for (Mapping m:t.getMappings(EnsemblCoordSystemType.chromosome)) {
                System.out.println("\t\tMapping: "+m.getTargetHashID());
                System.out.println("\t\t\tCoords: "+m.getTargetCoordinates().toString());
            }

        }
        System.out.println("Canonical Transcript: "+gene.getCanonicalTranscript().getStableID());

        gene = sp.getGeneByID(31059);
        mapping = gene.getAnnotationLevelMappings().first();


        System.out.println("Gene: "+ gene.getStableID());
        System.out.println("\tchr start: "+mapping.getTargetCoordinates().getStart());
        System.out.println("\tchr stop: "+mapping.getTargetCoordinates().getEnd());
        System.out.println("\tassembly: "+gene.getAssembly());
        System.out.println("\tdescription: "+ gene.getDescription());
        System.out.println("\tsymbol: "+gene.getDisplayName());
        System.out.println("\tstrand: "+mapping.getTargetCoordinates().getStrand());
        System.out.println("\ttaxonID: "+gene.getSpecies().getTaxonomyID());
        System.out.println("\tstatus: "+gene.getStatus());
        System.out.println("\ttype: "+ gene.getBiotype());

        System.out.println("\tTranscript Count: "+ gene.getTranscripts().size());
        for (DATranscript t : gene.getTranscripts()) {
            System.out.println("\t\tTranscript: "+ t.getStableID());
            System.out.println("\t\t "+ t.getDisplayName());
            System.out.println("\t\t "+ t.getStatus());
            System.out.println("\t\t "+ t.getBiotype());
            System.out.println("\t\t "+ t.getDescription());
            System.out.println("\t\t "+ t.getGene().getStableID());
            System.out.println("\t\tCanonical ?  "+ t.isCanonical());

            if (t.getDisplayXRef()!=null) {
            System.out.println("\t\tXREF: "+ t.getDisplayXRef().getDBName());
            System.out.println("\t\tXREF: "+ t.getDisplayXRef().getDisplayID());
            System.out.println("\t\tXREF: "+ t.getDisplayXRef().getInfoType());
            System.out.println("\t\tXREF: "+ t.getDisplayXRef().getInfo());
            }
            for (Mapping m:t.getMappings(EnsemblCoordSystemType.chromosome)) {
                System.out.println("\t\tMapping: "+m.getTargetHashID());
                System.out.println("\t\t\tCoords: "+m.getTargetCoordinates().toString());
            }
        }
        System.out.println("Canonical Transcript: "+gene.getCanonicalTranscript().getStableID());



        eReg = new DBRegistry(DataSource.ENSEMBLGENOMES);


        sp = eReg.getSpeciesByAlias("b_subtilis");
        gene = sp.getGeneByStableID("EBBACG00000001042");

        mapping = gene.getAnnotationLevelMappings().first();


        System.out.println("Gene: "+ gene.getStableID());
        System.out.println("\tchr start: "+mapping.getTargetCoordinates().getStart());
        System.out.println("\tchr stop: "+mapping.getTargetCoordinates().getEnd());
        System.out.println("\tassembly: "+gene.getAssembly());
        System.out.println("\tdescription: "+ gene.getDescription());
        System.out.println("\tsymbol: "+gene.getDisplayName());
        System.out.println("\tstrand: "+mapping.getTargetCoordinates().getStrand());
        System.out.println("\ttaxonID: "+gene.getSpecies().getTaxonomyID());
        System.out.println("\tstatus: "+gene.getStatus());
        System.out.println("\ttype: "+ gene.getBiotype());

        System.out.println("\tTranscript Count: "+ gene.getTranscripts().size());
        for (DATranscript t : gene.getTranscripts()) {
            System.out.println("\t\tTranscript: "+ t.getStableID());
            System.out.println("\t\t "+ t.getDisplayName());
            System.out.println("\t\t "+ t.getStatus());
            System.out.println("\t\t "+ t.getBiotype());
            System.out.println("\t\t "+ t.getDescription());
            System.out.println("\t\t "+ t.getGene().getStableID());
            System.out.println("\t\tCanonical ?  "+ t.isCanonical());
            System.out.println("\t\tXREF: "+ t.getDisplayXRef().getDBName());
            System.out.println("\t\tXREF: "+ t.getDisplayXRef().getDisplayID());
            System.out.println("\t\tXREF: "+ t.getDisplayXRef().getInfoType());
            System.out.println("\t\tXREF: "+ t.getDisplayXRef().getInfo());
            for (Mapping m:t.getMappings(EnsemblCoordSystemType.chromosome)) {
                System.out.println("\t\tMapping: "+m.getTargetHashID());
                System.out.println("\t\t\tCoords: "+m.getTargetCoordinates().toString());
            }
        }
        if ( gene.getCanonicalTranscript()!= null) {
        System.out.println("Canonical Transcript: "+gene.getCanonicalTranscript().getStableID());
        }



  
    }
}
