/**
 * Copyright (C) 2010 The Roslin Institute <contact andy.law@roslin.ed.ac.uk>
 *
 * This file is part of the Ensembl Java API demonstration project developed by the
 * Bioinformatics Group at The Roslin Institute, The Royal (Dick) School of
 * Veterinary Studies, University of Edinburgh.
 *
 * This is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License (version 3) as published by
 * the Free Software Foundation.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * in this software distribution. If not, see <http://www.gnu.org/licenses/gpl-3.0.html/>.
 */
package uk.ac.roslin.ensembl.test;


import java.io.BufferedWriter;
import java.io.FileWriter;
import java.util.TreeMap;
import org.apache.log4j.PropertyConfigurator;
import uk.ac.roslin.ensembl.config.DBConnection.DataSource;
import uk.ac.roslin.ensembl.config.EnsemblCoordSystemType;
import uk.ac.roslin.ensembl.config.EnsemblDBType;
import uk.ac.roslin.ensembl.config.FeatureType;
import uk.ac.roslin.ensembl.dao.database.DBCollectionCoreDatabase;
import uk.ac.roslin.ensembl.dao.database.DBDatabase;
import uk.ac.roslin.ensembl.dao.database.DBRegistry;
import uk.ac.roslin.ensembl.dao.database.DBSingleSpeciesCoreDatabase;
import uk.ac.roslin.ensembl.dao.database.DBSpecies;
import uk.ac.roslin.ensembl.datasourceaware.core.DAChromosome;
import uk.ac.roslin.ensembl.datasourceaware.core.DAExon;
import uk.ac.roslin.ensembl.datasourceaware.core.DAGene;
import uk.ac.roslin.ensembl.datasourceaware.core.DATranscript;
import uk.ac.roslin.ensembl.model.Mapping;
import uk.ac.roslin.ensembl.model.database.CoreDatabase;
import uk.ac.roslin.ensembl.model.database.SingleSpeciesCoreDatabase;


public class SetzermannTest {

    public static void main(String[] args) throws Exception {

        PropertyConfigurator.configure("log4j.properties");

        DBRegistry eReg = new DBRegistry(DataSource.ENSEMBLDB);
        DBSpecies cow = eReg.getSpeciesByAlias("cow");
        String taxonID = cow.getTaxonomyID();
        
        TreeMap<String, DAChromosome> chrHash = cow.getChromosomes();
        String assembly = chrHash.get(chrHash.firstKey()).getAssembly();
        
        
        FileWriter fstream = new FileWriter("genes.txt");
        BufferedWriter outFile = new BufferedWriter(fstream);
        
        outFile.append("GeneID|start|stop|strand|assembly|chromosome|description|"
                +"DisplayName|taxonID|Status|Biotype\r\n"
                );
        
        for (DAChromosome chr: chrHash.values() ) {
            for (DAGene g: chr.getGenesOnRegion(chr.getBioBegin(), chr.getBioEnd())) {
                    
                Mapping m = g.getChromosomeMapping(chr);
                
                outFile.append(g.getStableID()).append("|");
                if (m != null && m.getTargetCoordinates()!= null) {
                    outFile.append(m.getTargetCoordinates().getStart().toString()).append("|");
                    outFile.append(m.getTargetCoordinates().getEnd().toString()).append("|");
                    outFile.append(m.getTargetCoordinates().getStrand().toString()).append("|");
                } else {
                    outFile.append(" | | |");
                }
                outFile.append(assembly).append("|");
                outFile.append(chr.getName()).append("|");
                outFile.append(g.getDescription()).append("|");
                outFile.append(g.getDisplayName()).append("|");
                outFile.append(taxonID).append("|");
                outFile.append(g.getStatus()).append("|");
                outFile.append(g.getBiotype()).append("\r\n");
            }
            //so we only do it for one chromosome :)
            break;
        }
        
        outFile.close();
        
        fstream = new FileWriter("transcripts.txt");
        outFile = new BufferedWriter(fstream);

        outFile.append("geneID|transcriptID|start|stop\r\n"); 
              
        for (DAChromosome chr: chrHash.values() ) {
           
            for (DAGene g : chr.getGenesOnRegion(chr.getBioBegin(), chr.getBioEnd())) {
                for (DATranscript t: g.getTranscripts()) {
                    
                    Mapping m = t.getChromosomeMapping(chr);
                    outFile.append(g.getStableID()).append("|");
                    outFile.append(t.getStableID());
                    if (m!= null&& m.getTargetCoordinates()!=null) {
                        outFile.append("|").append(m.getTargetCoordinates().getStart().toString()).append("|") ;
                        outFile.append(m.getTargetCoordinates().getEnd().toString()); 
                    }
                    outFile.append("\r\n");
                }
            }
            //so we only do it for one chromosome :)
            break;
        }
        

        outFile.close();
        
        fstream = new FileWriter("exons.txt");
        outFile = new BufferedWriter(fstream);
        
        outFile.append("geneID|transcriptID|exonID|rank|start|stop|constitutive\r\n"); 
        
        for (DAChromosome chr: chrHash.values() ) {
            for (DAGene g : chr.getGenesOnRegion(chr.getBioBegin(), chr.getBioEnd())) {
                for (DATranscript t: g.getTranscripts()) {
                    for (DAExon e : t.getExons()) {
                        Mapping m = e.getChromosomeMapping(chr); 
                        outFile.append(g.getStableID()).append("|");
                        outFile.append(t.getStableID()).append("|");
                        outFile.append(e.getStableID()).append("|");
                        outFile.append(e.getRank().toString()).append("|");
                        if (m!= null&& m.getTargetCoordinates()!=null) {
                            outFile.append(m.getTargetCoordinates().getStart().toString()).append("|") ;
                            outFile.append(m.getTargetCoordinates().getEnd().toString()).append("|") ; 
                        }
                        outFile.append(e.isConstitutive().toString()).append("\r\n");                        
                      
                    }
                }
            }
            //so we only do it for one chromosome :)
            break;
        }
        
        outFile.close();

    }
}
