/**
 * Copyright (C) 2010 The Roslin Institute <contact andy.law@roslin.ed.ac.uk>
 *
 * This file is part of the Ensembl Java API demonstration project developed by the
 * Bioinformatics Group at The Roslin Institute, The Royal (Dick) School of
 * Veterinary Studies, University of Edinburgh.
 *
 * This is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License (version 3) as published by
 * the Free Software Foundation.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * in this software distribution. If not, see <http://www.gnu.org/licenses/gpl-3.0.html/>.
 */

package uk.ac.roslin.ensembl.test;

import java.io.File;
import java.util.List;
import org.apache.log4j.PropertyConfigurator;
import uk.ac.roslin.ensembl.config.DBConnection.DataSource;
import uk.ac.roslin.ensembl.config.EnsemblDBType;
import uk.ac.roslin.ensembl.config.RegistryConfiguration;
import uk.ac.roslin.ensembl.dao.database.DBRegistry;
import uk.ac.roslin.ensembl.dao.database.DBDatabase;
import uk.ac.roslin.ensembl.dao.database.DBSingleSpeciesCoreDatabase;
import uk.ac.roslin.ensembl.dao.database.DBSpecies;
import uk.ac.roslin.ensembl.model.core.ProteinFeature;
import uk.ac.roslin.ensembl.model.database.SingleSpeciesCoreDatabase;

public class TestArchives {

    public  static void main(String[] args) throws Exception {

        PropertyConfigurator.configure("log4j.properties");

        File fileMR = new File("src/main/resources/example_local_configuration.properties");
        

        RegistryConfiguration conf = new RegistryConfiguration(DataSource.ENSEMBLDB_ARCHIVES);
        conf.setSchemaByFile(fileMR);
        DBRegistry registry = new DBRegistry(conf,true);
        
        //this wway fails to set the schema mappings and known version
        //DBRegistry registry = new DBRegistry(DataSource.ENSEMBLDB_ARCHIVES);




        System.out.println(registry.getBriefRegistryReport());


        System.out.println("ensemblarchives registry report: ");
        System.out.println("******************************* ");


        System.out.println(registry.printRegistryReport().toString());


       System.out.println("species count : "+registry.getSpecies().size());
        int i = 1;
        for (DBSpecies s: registry.getSpecies()) {
            System.out.println(i++ +": "+s.getSpeciesBinomial());

            for(DBDatabase d : s.getDatabasesByType(EnsemblDBType.core)) {

                System.out.println(d.getdBName()+ " : "+d.getBuild());
            }


        }

        String human = "human"; // relies on alias look up
        String release47 = "47";
        int translationID = 1;

        SingleSpeciesCoreDatabase db = (SingleSpeciesCoreDatabase) registry.getDatabase(human);

        //DBSpecies homo = (DBSpecies) ensembldbRegistry.getSpeciesByAlias("human");


       if (db!=null) {

        System.out.println("current human top level: "+db.getTopLevelCoordSystem().getId());
        System.out.println("current human chromosome level: "+db.getChromosomeLevelCoordSystem().getId());
        System.out.println("current human sequence level: "+db.getSequenceLevelCoordSystem().getId());

       }

        System.out.println("Get features from Ensembl release 47 for human translation id ="+translationID);

        List<? extends ProteinFeature> result47 = ((DBSingleSpeciesCoreDatabase) registry
                .getDatabase(human, EnsemblDBType.core, release47))
                .getCoreFactory()
                    .getProteinFeatureDAO()
                        .getProteinFeaturesByTranslationID(translationID);


        System.out.println("retrieved: "+result47.size()+" features.");
 System.out.println("\n\n*****************************\n* COMPLETED FUNCTIONAL TEST *\n*****************************\n");

    }





}
