/**
 * Copyright (C) 2010 The Roslin Institute <contact andy.law@roslin.ed.ac.uk>
 *
 * This file is part of the Ensembl Java API demonstration project developed by the
 * Bioinformatics Group at The Roslin Institute, The Royal (Dick) School of
 * Veterinary Studies, University of Edinburgh.
 *
 * This is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License (version 3) as published by
 * the Free Software Foundation.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * in this software distribution. If not, see <http://www.gnu.org/licenses/gpl-3.0.html/>.
 */

package uk.ac.roslin.ensembl.test;

import java.util.List;
import java.util.TreeMap;
import org.apache.log4j.PropertyConfigurator;
import org.biojava3.core.sequence.Strand;
import uk.ac.roslin.ensembl.config.DBConnection.DataSource;
import uk.ac.roslin.ensembl.datasourceaware.core.DAChromosome;
import uk.ac.roslin.ensembl.datasourceaware.core.DADNASequence;
import uk.ac.roslin.ensembl.datasourceaware.core.DAGene;
import uk.ac.roslin.ensembl.datasourceaware.core.DATranslation;
import uk.ac.roslin.ensembl.dao.database.DBDatabase;
import uk.ac.roslin.ensembl.dao.database.DBRegistry;
import uk.ac.roslin.ensembl.dao.database.DBSingleSpeciesCoreDatabase;
import uk.ac.roslin.ensembl.dao.database.DBSpecies;
import uk.ac.roslin.ensembl.dao.database.factory.DBDAOSingleSpeciesCoreFactory;
import uk.ac.roslin.ensembl.model.Coordinate;
import uk.ac.roslin.ensembl.model.Mapping;
import uk.ac.roslin.ensembl.model.core.ProteinFeature;
import uk.ac.roslin.ensembl.model.database.SingleSpeciesCoreDatabase;
import uk.ac.roslin.ensembl.config.EnsemblDBType;

public class TestEnsembl {

    public  static void main(String[] args) throws Exception {

         PropertyConfigurator.configure("log4j.properties");

        DBRegistry ensembldbRegistry = new DBRegistry(DataSource.ENSEMBLDB);

        for (DAGene g : ensembldbRegistry.getSpeciesByAlias("Cow").getChromosomeByName("1", "60").getGenesOnRegion(1,1000000) ) {
            System.out.println(g.getDescription());
        }



        System.out.println("ensembldb registry report: ");
        System.out.println("************************** ");
        System.out.println( ensembldbRegistry.printRegistryReport().toString());

        System.out.println("species count : "+ensembldbRegistry.getSpecies().size());
        int i = 1;
        for (DBSpecies s: ensembldbRegistry.getSpecies()) {
            System.out.println(i++ +": "+s.getSpeciesBinomial());

            for(DBDatabase d : s.getDatabasesByType(EnsemblDBType.core)) {

                System.out.println(d.getdBName()+ " : "+d.getBuild());
            }


        }
//
//         System.out.println("TESTING LIVE ENSEMBLDB (5306): comparing Schema changes between version 50 and 51");
//
//        System.out.println("**********************************\nhuman databases\n\n");
//
//        for ( Database d : ensembldbRegistry.getSpeciesByAlias("human").getDatabases() ) {
//            System.out.println("\t"+d.getdBName());
//        }

        SingleSpeciesCoreDatabase db = (SingleSpeciesCoreDatabase) ensembldbRegistry.getDatabase("human");

        //DBSpecies homo = (DBSpecies) ensembldbRegistry.getSpeciesByAlias("human");


        System.out.println("current human top level: "+db.getTopLevelCoordSystem().getId());
        System.out.println("current human chromosome level: "+db.getChromosomeLevelCoordSystem().getId());
        System.out.println("current human sequence level: "+db.getSequenceLevelCoordSystem().getId());

        String human = "human"; // relies on alias look up
        String release51 = "51";
        String release50 = "50";

        int translationID = 70113;

        System.out.println("Get features from Ensembl release 50 for human translation id ="+translationID);

        List<? extends ProteinFeature> result50 = ((DBSingleSpeciesCoreDatabase) ensembldbRegistry
                .getDatabase(human, EnsemblDBType.core, release50))
                .getCoreFactory()
                    .getProteinFeatureDAO()
                        .getProteinFeaturesByTranslationID(translationID);


        System.out.println("retrieved: "+result50.size()+" features.");

        System.out.println("Get features from Ensembl release 51 for human translation id ="+translationID);

        human = "bob";
        try {
            ensembldbRegistry.addSessionAlias("homo_sapiens", human);
            System.out.println("added alias bob ok");
        } catch (Exception exception) {
            System.out.println("failed to add alias bob for homo_sapiens");
        }



        try {
            ensembldbRegistry.addSessionAlias("homo sapiens", "norman");
            System.out.println("added alias norman ok - but shouldnt have");
        } catch (Exception exception) {
            System.out.println("failed to add alias norman - this is correct");
        }

        List<? extends ProteinFeature> result51 = ((DBSingleSpeciesCoreDatabase) ensembldbRegistry
              .getDatabase(human, EnsemblDBType.core, release51))
                .getCoreFactory()
                    .getProteinFeatureDAO()
                        .getProteinFeaturesByTranslationID(translationID);


        System.out.println("retrieved: "+result51.size()+" features. Using the new alias.");


        DBDAOSingleSpeciesCoreFactory fact = ((DBSingleSpeciesCoreDatabase) ensembldbRegistry.getDatabase("chicken")).getCoreFactory();
        DBDAOSingleSpeciesCoreFactory fact50 = ((DBSingleSpeciesCoreDatabase) ensembldbRegistry.getDatabase("chicken","50")).getCoreFactory();
        DBDAOSingleSpeciesCoreFactory fact51 = ((DBSingleSpeciesCoreDatabase) ensembldbRegistry.getDatabase("chicken","51")).getCoreFactory();

        DAChromosome chr = fact.getChromosomeDAO().getChromosomeByName("25");
        System.out.println("chr type; "+chr.getType().toString());
        DAChromosome chr50 = fact50.getChromosomeDAO().getChromosomeByName("25");
        DAChromosome chr51 = fact51.getChromosomeDAO().getChromosomeByName("25");

        System.out.println("Chromosome:\n\tName: "+chr.getChromosomeName()+ " db version-"+chr.getDBVersion());
        System.out.println("\tLength: "+chr.getDBSeqLength());
        System.out.println("\tcoord sys ID: "+chr.getCoordSystem().getId());
        System.out.println("\tseq level coord sys ID: "+((DBSingleSpeciesCoreDatabase) chr.getDaoFactory().getDatabase()).getSequenceLevelCoordSystem().getId());
        System.out.println("\tseq region ID :"+chr.getId());
        System.out.println("\tseq type :"+chr.getType().toString());

        System.out.println("Chromosome:\n\tName: "+chr50.getChromosomeName()+"db version-"+chr50.getDBVersion());
        System.out.println("\tLength: "+chr50.getDBSeqLength());
        System.out.println("\tcoord sys ID: "+chr50.getCoordSystem().getId());
        System.out.println("\tseq level coord sys ID: "+((DBSingleSpeciesCoreDatabase) chr50.getDaoFactory().getDatabase()).getSequenceLevelCoordSystem().getId());
        System.out.println("\tseq region ID :"+chr50.getId());
        System.out.println("\tseq type :"+chr50.getType().toString());

        System.out.println("Chromosome:\n\tName: "+chr51.getChromosomeName()+ " db version-"+chr51.getDBVersion());
        System.out.println("\tLength: "+chr51.getDBSeqLength());
        System.out.println("\tcoord sys ID: "+chr51.getCoordSystem().getId());
        System.out.println("\tseq level coord sys ID: "+ ((DBSingleSpeciesCoreDatabase) chr51.getDaoFactory().getDatabase()).getSequenceLevelCoordSystem().getId());
        System.out.println("\tseq region ID :"+chr51.getId());
        System.out.println("\tseq type :"+chr51.getType().toString());


        // better way of demoing schema 50/51 change


       DBSpecies hsa = ensembldbRegistry.getSpeciesByAlias("human");

       TreeMap<String, DAChromosome> chrs = hsa.getChromosomes();

       for(String s: chrs.keySet()) {
           System.out.println("Human Chromosome: "+s+" length: "+chrs.get(s).getLength());
       }



       DAGene gene58 = hsa.getGeneByStableID("ENSG00000139618");
        System.out.println("gene name: "+gene58.getDescription());
       System.out.println("transcript id: "+gene58.getCanonicalTranscriptID());
       DATranslation translation = gene58.getCanonicalTranslation();
       System.out.println("translation id: "+translation.getId());
       System.out.println("translation hash: "+translation.getHashID());
       System.out.println("translation database: "+translation.getDaoFactory().getDatabaseName());
       System.out.println("ProteinFeatures: "+translation.getProteinFeatures().size());


       DAGene gene50 = hsa.getGeneByStableID("ENSG00000139618", "50");
        System.out.println("gene name: "+gene50.getDescription());
       System.out.println("transcript id: "+gene50.getCanonicalTranscriptID());
       translation = gene50.getCanonicalTranslation();
       System.out.println("translation id: "+translation.getId());
       System.out.println("translation hash: "+translation.getHashID());
       System.out.println("translation database: "+translation.getDaoFactory().getDatabaseName());
       System.out.println("ProteinFeatures: "+translation.getProteinFeatures().size());




//        for (Mapping mapping: chr.getCompleteAssembly().getMappings()) {
//

//        //for (Mapping mapping: chr.getWholeAssembly()) {
//
//            System.out.println( ((DASequenceOLD) mapping.getTarget()).getName()
//                    +" : " + mapping.getTargetCoordinates().getStart()
//                    +" - " + mapping.getTargetCoordinates().getEnd()
//                    +" "+ ( mapping.getTargetCoordinates().getStrand()==Coordinate.Strand.REVERSE_STRAND ? "reverse" : "" )
//                    +" (source: " + mapping.getSourceCoordinates().getStart()
//                    +" - " + mapping.getSourceCoordinates().getEnd()
//                    +")");
//        }


         for (Mapping mapping: chr.getCompleteAssembly().getStitchedMappings()) {
        //for (Mapping mapping : chr.stitchComponents(1, 3000, chr.getWholeAssembly())) {
                    System.out.println( ((DADNASequence) mapping.getTarget()).getName()
                    +" : " + mapping.getTargetCoordinates().getStart()
                    +" - " + mapping.getTargetCoordinates().getEnd()
                    +" "+ ( mapping.getTargetCoordinates().getStrand()==Coordinate.Strand.REVERSE_STRAND ? "reverse" : "" )
                    +" (source: " + mapping.getSourceCoordinates().getStart()
                    +" - " + mapping.getSourceCoordinates().getEnd()
                    +")");
                    try {
                //System.out.println(((DASequenceOLD) mapping.getTarget()).getSequenceAsString());
            } catch (Exception e) {
            }
        }

         System.out.println("\n***************************************\n");

         for (Mapping mapping: chr.getCompleteAssembly().getStitchedMappings(1, 3000)) {
                    System.out.println( ((DADNASequence) mapping.getTarget()).getName()
                    +" : " + mapping.getTargetCoordinates().getStart()
                    +" - " + mapping.getTargetCoordinates().getEnd()
                    +" "+ ( mapping.getTargetCoordinates().getStrand()==Coordinate.Strand.REVERSE_STRAND ? "reverse" : "" )
                    +" (source: " + mapping.getSourceCoordinates().getStart()
                    +" - " + mapping.getSourceCoordinates().getEnd()
                    +")");
                    try {
                        System.out.println("mapped sequence length: "+((DADNASequence) mapping.getTarget()).getDBSeqLength());
                        System.out.println("mapped sequence start: "+((DADNASequence) mapping.getTarget()).getBioBegin());
                        System.out.println("mapped sequence end: "+((DADNASequence) mapping.getTarget()).getBioEnd());
                        System.out.println("mapped sequence fullseq length: "+((DADNASequence) mapping.getTarget()).getSequenceAsString().length());
                        System.out.println("mapped sequence fullseq:\n "+((DADNASequence) mapping.getTarget()).getSequenceAsString());
                        
                        if (mapping.getTargetCoordinates().getStrand() == Coordinate.Strand.FORWARD_STRAND) {

                            System.out.println("mapped sequence desired (+) fragment from object:\n "
                                    + ((DADNASequence) mapping.getTarget()).getSubSequence(mapping.getTargetCoordinates().getStart(),
                                    mapping.getTargetCoordinates().getEnd()).getSequenceAsString());

                            System.out.println("mapped sequence desired (+) fragment from String :\n "
                                    + ((DADNASequence) mapping.getTarget()).getSequenceAsString(mapping.getTargetCoordinates().getStart(),
                                    mapping.getTargetCoordinates().getEnd(), Strand.POSITIVE));
                        } else {
                            
                            System.out.println("mapped sequence desired (-) fragment from String :\n "
                                    + ((DADNASequence) mapping.getTarget()).getReverseComplementSequenceAsString(mapping.getTargetCoordinates().getStart(),
                                    mapping.getTargetCoordinates().getEnd()));

                        }
                        System.out.println("");
                        //System.out.println(((DASequenceOLD) mapping.getTarget()).getSequenceAsString());
            } catch (Exception e) {
            }
        }




        System.out.println("chromosome sequence 200000-210000\n"
                +chr.getSequenceAsString(200000, 210000, Strand.POSITIVE));
        System.out.println("chromosome rev complement sequence 200000-210000\n"
                +chr.getReverseComplementSequenceAsString(200000, 210000));
        System.out.println("");
        System.out.println("chromosome sequence 1-50\n"
                +chr.getSequenceAsString(1, 50, Strand.POSITIVE));
        System.out.println("chromosome rev complement sequence 1-50\n"
                +chr.getReverseComplementSequenceAsString(1, 50));
        System.out.println("");
        System.out.println("chromosome sequence 1750-1800\n"
                +chr.getSequenceAsString(1750, 1800, Strand.POSITIVE));
        System.out.println("chromosome ReverseComplement sequence 1750-1800\n"
                +chr.getReverseComplementSequenceAsString(1750, 1800));
        System.out.println("");
        System.out.println("chromosome sequence 142250, 142300\n"
                +chr.getSequenceAsString(142250, 142300, Strand.POSITIVE));
        System.out.println("chromosome ReverseComplement sequence 142250, 142300\n"
                +chr.getReverseComplementSequenceAsString(142250, 142300));
        System.out.println("");
        System.out.println("chromosome sequence 142250, 142400\n"
                +chr.getSequenceAsString(142250, 142400, Strand.POSITIVE));
        System.out.println("chromosome ReverseComplement sequence 142250, 142400\n"
                +chr.getReverseComplementSequenceAsString(142250, 142400));
        System.out.println("");


        System.out.println("\n\n*****************************\n* COMPLETED FUNCTIONAL TEST *\n*****************************\n");


    }

}
