/**
 * Copyright (C) 2010 The Roslin Institute <contact andy.law@roslin.ed.ac.uk>
 *
 * This file is part of the Ensembl Java API demonstration project developed by the
 * Bioinformatics Group at The Roslin Institute, The Royal (Dick) School of
 * Veterinary Studies, University of Edinburgh.
 *
 * This is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License (version 3) as published by
 * the Free Software Foundation.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * in this software distribution. If not, see <http://www.gnu.org/licenses/gpl-3.0.html/>.
 */

package uk.ac.roslin.ensembl.test;

import org.apache.log4j.PropertyConfigurator;
import uk.ac.roslin.ensembl.config.DBConnection.DataSource;
import uk.ac.roslin.ensembl.datasourceaware.core.DAChromosome;
import uk.ac.roslin.ensembl.datasourceaware.core.DAGene;
import uk.ac.roslin.ensembl.dao.database.DBRegistry;
import uk.ac.roslin.ensembl.dao.database.DBSingleSpeciesCoreDatabase;
import uk.ac.roslin.ensembl.dao.database.DBSpecies;
import uk.ac.roslin.ensembl.model.Mapping;
import uk.ac.roslin.ensembl.config.EnsemblCoordSystemType;
import uk.ac.roslin.ensembl.config.FeatureType;

public class TestEnsembl2 {

    
    public  static void main(String[] args) throws Exception {

        PropertyConfigurator.configure("log4j.properties");
     
        DBRegistry ensembldbRegistry = new DBRegistry(DataSource.ENSEMBLDB);

//        DBSingleSpeciesCoreDatabase db  = (DBSingleSpeciesCoreDatabase) ensembldbRegistry.getDatabase("chicken");
//
//        System.out.println("Chicken Build Levels");
//
//        for (FeatureType ft: db.getBuildLevels().keySet()) {
//            System.out.println("\t"+ft.toString()+" : "+db.getBuildLevel(ft.toString()));
//        }
//
//
//        System.out.println("\nChicken Features");
//
//        for (FeatureType t : FeatureType.getAllTypes()) {
//            Set<DACoordinateSystem> s = db.getCSForFeature(t);
//
//            if (s != null && !s.isEmpty()) {
//                System.out.println("\t"+t.toString());
//                for (DACoordinateSystem cs : s) {
//                    System.out.println("\t\tcs-id: " + cs.getId() + " - " + "cs-type: " + cs.getType().toString());
//                }
//            } else {
//                System.out.println("\t"+t.toString()+"\n\t\tNONE");
//            }
//        }
//
//        System.out.println("\nChromosomal Features: ");
//        for (FeatureType f : db.getFeaturesForCS(db.getChromosomeLevelCoordSystem()) ) {
//            System.out.println("\t"+f.toString());
//        }
//        System.out.println("Top Level Features: ");
//        for (FeatureType f : db.getFeaturesForCS(db.getTopLevelCoordSystem()) ) {
//            System.out.println("\t"+f.toString());
//        }
//        System.out.println("Seq Level Features: ");
//        for (FeatureType f : db.getFeaturesForCS(db.getSequenceLevelCoordSystem()) ) {
//            System.out.println("\t"+f.toString());
//        }


         DBSingleSpeciesCoreDatabase GGdb  = (DBSingleSpeciesCoreDatabase) ensembldbRegistry.getSpeciesByAlias("chicken").getCurrentCoreDatabase();


         DBSpecies sp = ensembldbRegistry.getSpeciesByAlias("chicken");
         DAChromosome chr = sp.getChromosomeByName("25");
         System.out.println("CHROMOSOME "+chr.getChromosomeName()+" FROM SPECIES: HashID: "+chr.getHashID()
                 +" VMID:"+chr.hashCode());


         for (DAChromosome c : sp.getChromosomes().values()) {
         System.out.println("CHROMOSOME "+c.getChromosomeName()+" FROM SPECIES: HashID: "+c.getHashID()
                 +" VMID:"+c.hashCode());
         }

         for (DAChromosome c : sp.getChromosomes("58").values()) {
         System.out.println("CHROMOSOME "+c.getChromosomeName()+" FROM SPECIES: HashID: "+c.getHashID()
                 +" VMID:"+c.hashCode());
         }


// DAGene gene = sp.getGeneByID(1128);
        DAGene gene = sp.getGeneByStableID("ENSGALG00000009011");

        System.out.println("GENE FROM SPECIES: HashID: "+gene.getHashID()
                +" VMID:"+gene.hashCode());

 System.out.println("Gene -> Chromosome mappings ");

 for (Mapping m : gene.getMappings(EnsemblCoordSystemType.chromosome) ) {
     System.out.println(m.getTarget().getClass().getSimpleName()
             +" chr name: "
             +( (DAChromosome) m.getTarget()).getChromosomeName()
             +" id: "+m.getTarget().getId() );
     System.out.println("CHROMOSOME FROM GENE MAPPING: HashID: "+m.getTarget().getHashID()
             +" VMID:"+ m.getTarget().hashCode());
 }
  gene = chr.getGenesOnRegion(3080, 12536).get(0);

        System.out.println("GENE FROM CHROMOSOME: HashID: "+gene.getHashID()
                +" VMID:"+((Object) gene).hashCode());


        System.out.println("Chromosome -> gene mappings ");

 for (Mapping m : chr.getMappings(FeatureType.gene) ) {
     System.out.println(m.getTarget().getClass().getSimpleName()
             +" gene stableID: "
             +( (DAGene) m.getTarget()).getStableID()
             +" id: "+m.getTarget().getId() );
        System.out.println("Gene on chromosome: HashID: "+m.getTarget().getHashID()
                +" VMID:"+m.getTarget().hashCode());
 }

 System.out.println("Gene -> Chromosome mappings ");

 for (Mapping m : gene.getMappings(EnsemblCoordSystemType.chromosome) ) {
     System.out.println(m.getTarget().getClass().getSimpleName()
             +" chr name: "
             +( (DAChromosome) m.getTarget()).getChromosomeName()
             +" id: "+m.getTarget().getId() );
      System.out.println("Chr on Gene: HashID: "+m.getTarget().getHashID()
              +" VMID:"+m.getTarget().hashCode());}



//         System.out.println("gene: "+gene.getId()+" ("+gene.getStableID()+") "+gene.getBiotype());
//         System.out.println("\t"+gene.getDescription());
//         System.out.println("Species: "+gene.getSpecies().getCommonName());
//         System.out.println("Created "+gene.getCreationDate().toString());
//         System.out.println("Modified "+gene.getModificationDate().toString());
//         System.out.println("XREF: "+gene.getDisplayXRef().getDBName() +":" +gene.getDisplayXRef().getDisplayID() );
//
//         for (Mapping m : gene.getMappings()) {
//             System.out.println("HASHES "+m.getSourceHashID()+" "+m.getTargetHashID());
//            System.out.println("Mapping source: "+m.getSource().getId());
//            System.out.println("Mapping target: "+m.getTarget().getId());
//            System.out.println("Mapping target type: "+m.getTarget().getClass().getSimpleName());
//
//            System.out.println("target coords:"+m.getTargetCoordinates().getStart()+"-"
//                     +m.getTargetCoordinates().getEnd() +"("
//                     +m.getTargetCoordinates().getStrand().toString() +")");
//             System.out.println("CS: "+((DADNASequence) m.getTarget()).getCoordSystem().getType().toString());
//         }
//
////
////         gene = GGdb.getFactory().getGeneDAO().addGeneByIDOnParent(1128, chr);
////
////         System.out.println("gene: "+gene.getId()+" ("+gene.getStableID()+") "+gene.getBiotype());
////         System.out.println("\t"+gene.getDescription());
////         System.out.println("Species: "+gene.getSpecies().getCommonName());
////         System.out.println("Created "+gene.getCreationDate().toString());
////         System.out.println("Modified "+gene.getModificationDate().toString());
////         for (Mapping m : gene.getMappings()) {
////             System.out.println("HASHES "+m.getSourceHashID()+" "+m.getTargetHashID());
////            System.out.println("Mapping source: "+m.getSource().getId());
////            System.out.println("Mapping target: "+m.getTarget().getId());
////            System.out.println("target coords:"+m.getTargetCoordinates().getStart()+"-"
////                     +m.getTargetCoordinates().getEnd() +"("
////                     +m.getTargetCoordinates().getStrand().toString() +")");
////             System.out.println("CS: "+((DADNASequence) m.getTarget()).getCoordSystem().getType().toString());
////         }
////
////         gene = GGdb.getFactory().getGeneDAO().getGeneByStableID("ENSGALG00000009011");
////         System.out.println("gene: "+gene.getId()+" ("+gene.getStableID()+") "+gene.getBiotype());
////         System.out.println("\t"+gene.getDescription());
////         System.out.println("Species: "+gene.getSpecies().getCommonName());
////         System.out.println("Created "+gene.getCreationDate().toString());
////         System.out.println("Modified "+gene.getModificationDate().toString());
////         for (Mapping m : gene.getMappings()) {
////            System.out.println("HASHES "+m.getSourceHashID()+" "+m.getTargetHashID());
////            System.out.println("Mapping source: "+m.getSource().getId());
////            System.out.println("Mapping target: "+m.getTarget().getId());
////            System.out.println("target coords:"+m.getTargetCoordinates().getStart()+"-"
////                     +m.getTargetCoordinates().getEnd() +"("
////                     +m.getTargetCoordinates().getStrand().toString() +")");
////             System.out.println("CS: "+((DADNASequence) m.getTarget()).getCoordSystem().getType().toString());
////         }
////
////         gene = GGdb.getFactory().getGeneDAO().addGeneByStableIDOnParent("ENSGALG00000009011", chr);
////
////         System.out.println("gene: "+gene.getId()+" ("+gene.getStableID()+") "+gene.getBiotype());
////         System.out.println("\t"+gene.getDescription());
////         System.out.println("Species: "+gene.getSpecies().getCommonName());
////         System.out.println("Created "+gene.getCreationDate().toString());
////         System.out.println("Modified "+gene.getModificationDate().toString());
////         for (Mapping m : gene.getMappings()) {
////            System.out.println("HASHES "+m.getSourceHashID()+" "+m.getTargetHashID());
////            System.out.println("Mapping source: "+m.getSource().getId());
////            System.out.println("Mapping target: "+m.getTarget().getId());
////            System.out.println("target coords:"+m.getTargetCoordinates().getStart()+"-"
////                     +m.getTargetCoordinates().getEnd() +"("
////                     +m.getTargetCoordinates().getStrand().toString() +")");
////             System.out.println("CS: "+((DADNASequence) m.getTarget()).getCoordSystem().getType().toString());
////         }
////
////
////
////         Coordinate coord = new Coordinate();
////
////         coord.setStart(1);
////         coord.setEnd(1000000);
////         //coord.setStrandInt(-1);
////         //coord.setStrandInt(1);
////         //coord.setStrandInt(null);
////
////        // List<DAGene> genes = GGdb.getFactory().getGeneDAO().getGeneMappingsOnRegion(chr,coord);
////        // List<? extends Gene> genes = chr.getGeneMappingsOnRegion(coord);
////         //List<? extends Gene> genes = chr.getGeneMappingsOnRegion(1, 1000000);
////        // List<? extends Gene> genes = chr.getGeneMappingsOnRegion(1, 1000000, null);
////        // List<? extends Gene> genes = chr.getGeneMappingsOnRegion(1, 1000000, Strand.FORWARD_STRAND);
////         List<? extends Gene> genes = chr.getGenesOnRegion(1, 1000000, Strand.REVERSE_STRAND);
////         System.out.println(genes.size()+ " genes");
////
////         DAGene g = new DAGene();
////
////         System.out.println("g type: "+g.getType());
////
////         DAGene g2 = new DAGene(GGdb.getFactory());
////
////         System.out.println("g2 type: "+g2.getType());
////
// List<DAGene> genes = chr.getGenesOnRegion(1, 500000, uk.ac.roslin.ensembl.model.Coordinate.Strand.REVERSE_STRAND);
// System.out.println(genes.size()+ " returned  genes");
// System.out.println("total gene mappings: "+chr.getMappings(FeatureType.gene).size());
//
//
// gene = genes.get(0);
//
//
//
// genes = chr.getGenesOnRegion(1, 500000, uk.ac.roslin.ensembl.model.Coordinate.Strand.FORWARD_STRAND);
// System.out.println(genes.size()+ " returned  genes");
// System.out.println("total gene mappings: "+chr.getMappings(FeatureType.gene).size());
//
// genes = chr.getGenesOnRegion(1, 500000);
// System.out.println(genes.size()+ " returned  genes");
// System.out.println("total gene mappings: "+chr.getMappings(FeatureType.gene).size());
//
//
//
// genes = chr.getGenesOnRegion(500000,1000000, uk.ac.roslin.ensembl.model.Coordinate.Strand.REVERSE_STRAND);
// System.out.println(genes.size()+ " returned genes");
// System.out.println("total gene mappings: "+chr.getMappings(FeatureType.gene).size());
//
// genes = chr.getGenesOnRegion(1010000, 1020000, uk.ac.roslin.ensembl.model.Coordinate.Strand.REVERSE_STRAND);
// System.out.println(genes.size()+ " returned genes");
// System.out.println("total gene mappings: "+chr.getMappings(FeatureType.gene).size());
//
// genes = chr.getGenesOnRegion(1030000, 1040000, uk.ac.roslin.ensembl.model.Coordinate.Strand.REVERSE_STRAND);
// System.out.println(genes.size()+ " returned genes");
// System.out.println("total gene mappings: "+chr.getMappings(FeatureType.gene).size());
//
// genes = chr.getGenesOnRegion(1050000, 1060000, uk.ac.roslin.ensembl.model.Coordinate.Strand.REVERSE_STRAND);
// System.out.println(genes.size()+ " returned genes");
// System.out.println("total gene mappings: "+chr.getMappings(FeatureType.gene).size());
//
//
//         System.out.println("gaps in the gene mappings: ");
////         for (Coordinate cd : Coordinate.getCoordinateGaps(chr.getMappedRegions().get(FeatureType.gene))) {
//         for (Coordinate cd : chr.getMappedRegions().get(FeatureType.gene).getGaps()) {
//             System.out.println("gap "+cd.getStart()+" - "+cd.getEnd());
//         }
//         System.out.println("gaps in the gene mappings for the whole chromosome: ");
//         Coordinate cx = new Coordinate(chr.getBioBegin(), chr.getBioEnd(),1);
////         for (Coordinate cd : cx.getGaps(chr.getMappedRegions().get(FeatureType.gene))) {
//         for (Coordinate cd : chr.getMappedRegions().get(FeatureType.gene).getUncoveredRegions(cx)) {
//             System.out.println("gap "+cd.getStart()+" - "+cd.getEnd());
//         }
//
////         Coordinate test = new Coordinate(900000,1200000,1);
////
////         System.out.println("testing 900000-1200000 query");
//////         System.out.println("test lies within known region: "+test.liesWithinCoordinateSetWithoutGaps(chr.getMappedRegions().get(FeatureType.gene)));
////         System.out.println("test lies within known region: "+chr.getMappedRegions().get(FeatureType.gene).containsCoordinateWithoutGaps(test));
////
//////         if (test.getGaps(chr.getMappedRegions().get(FeatureType.gene)).isEmpty()) {
////         if (chr.getMappedRegions().get(FeatureType.gene).getUncoveredRegions(test).isEmpty()) {
////
////             System.out.println("test region is fully covered");
////
////         } else {
////             System.out.println("test region is not fully covered");
//// //            for (Coordinate cd : test.getGaps(chr.getMappedRegions().get(FeatureType.gene))) {
////             for (Coordinate cd : chr.getMappedRegions().get(FeatureType.gene).getUncoveredRegions(test)) {
////                System.out.println("gap "+cd.getStart()+" - "+cd.getEnd());
////             }
////         }
////
//////         System.out.println("test has regions not covered: "+test.getGaps(chr.getMappedRegions().get(FeatureType.gene)).size());
////         System.out.println("test has regions not covered: "+chr.getMappedRegions().get(FeatureType.gene).getUncoveredRegions(test).size());
////
////         test = new Coordinate(1,900000,1);
////
////         System.out.println("testing 1-900000 query");
////
//////         System.out.println("test lies within known region: "+test.liesWithinCoordinateSetWithoutGaps(chr.getMappedRegions().get(FeatureType.gene)));
////         System.out.println("test lies within known region: "+chr.getMappedRegions().get(FeatureType.gene).containsCoordinateWithoutGaps(test));
////
//////         if (test.getGaps(chr.getMappedRegions().get(FeatureType.gene)).isEmpty()) {
////         if (chr.getMappedRegions().get(FeatureType.gene).getUncoveredRegions(test).isEmpty()) {
////             System.out.println("test region is fully covered");
////         } else {
////             System.out.println("test region is not fully covered");
////             for (Coordinate cd : chr.getMappedRegions().get(FeatureType.gene).getUncoveredRegions(test)) {
////                System.out.println("gap "+cd.getStart()+" - "+cd.getEnd());
////             }
////         }
////
//// //        System.out.println("test has regions not covered: "+test.getGaps(chr.getMappedRegions().get(FeatureType.gene)).size());
////         System.out.println("test has regions not covered: "+chr.getMappedRegions().get(FeatureType.gene).getUncoveredRegions(test).size());
////
////
////         genes = chr.getGenesOnRegion(1, 10000, uk.ac.roslin.ensembl.model.Coordinate.Strand.REVERSE_STRAND);
////
////         System.out.println(genes.size()+ " genes");
////
////         for (Mapping m : chr.getMappings(FeatureType.gene)) {
////
////             System.out.println( ((DAGene) m.getTarget()).getStableID()+" "+m.getSourceHashID()+" "+m.getTargetHashID());
////
////         }
////
////
        System.out.println("\n\n*****************************\n* COMPLETED FUNCTIONAL TEST *\n*****************************\n");


    }

}
