/**
 * Copyright (C) 2010 The Roslin Institute <contact andy.law@roslin.ed.ac.uk>
 *
 * This file is part of the Ensembl Java API demonstration project developed by the
 * Bioinformatics Group at The Roslin Institute, The Royal (Dick) School of
 * Veterinary Studies, University of Edinburgh.
 *
 * This is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License (version 3) as published by
 * the Free Software Foundation.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * in this software distribution. If not, see <http://www.gnu.org/licenses/gpl-3.0.html/>.
 */

package uk.ac.roslin.ensembl.test;


import java.util.ArrayList;
import java.util.List;
import org.apache.log4j.PropertyConfigurator;
import uk.ac.roslin.ensembl.config.DBConnection.DataSource;
import uk.ac.roslin.ensembl.datasourceaware.core.DAAssembledDNASequence;
import uk.ac.roslin.ensembl.datasourceaware.core.DAChromosome;
import uk.ac.roslin.ensembl.datasourceaware.core.DADNASequence;
import uk.ac.roslin.ensembl.dao.database.DBRegistry;
import uk.ac.roslin.ensembl.dao.database.DBSingleSpeciesCoreDatabase;

public class TestEnsembl3 {

    public  static void main(String[] args) throws Exception {

        PropertyConfigurator.configure("log4j.properties");
        DBRegistry ensembldbRegistry = new DBRegistry(DataSource.ENSEMBLDB);


        System.out.println("\n_____________________________________________________________\n");


        DBSingleSpeciesCoreDatabase GGdb  = (DBSingleSpeciesCoreDatabase) ensembldbRegistry.getSpeciesByAlias("chicken").getCurrentCoreDatabase();


        DADNASequence s1 = new DADNASequence(GGdb.getCoreFactory());


        s1.setId(102799);

        System.out.println("s1 type before validation - should be DASequence: "+s1.getClass().getSimpleName());

        s1 = GGdb.getCoreFactory().getSequenceDAO().getValidatedSequence(s1);

        System.out.println("s1 type after validation - should be DAChromosome: "+s1.getClass().getSimpleName());
        DADNASequence s2 = GGdb.getCoreFactory().getSequenceDAO().getSequenceByID(102799);
        System.out.println("102799 type before validation- should be a chromosome: "+s2.getClass().getSimpleName());
        s2 = GGdb.getCoreFactory().getSequenceDAO().getValidatedSequence(s2);
        System.out.println("102799 type after validation- should still be a chromosome: "+s2.getClass().getSimpleName());

        DADNASequence s3 = GGdb.getCoreFactory().getSequenceDAO().getSequenceByID(54919);
        System.out.println("54919 type before validation - should be a sequence: "+s3.getClass().getSimpleName());
        s3 = GGdb.getCoreFactory().getSequenceDAO().getValidatedSequence(s3);
        System.out.println("54919 type after validation- should still be a sequence: "+s3.getClass().getSimpleName());



        DADNASequence s4 = new DADNASequence(GGdb.getCoreFactory());
        s4.setId(102799);
        DADNASequence s5 = new DADNASequence(GGdb.getCoreFactory());
        s5.setId(54919);
        DADNASequence s6 = new DADNASequence(GGdb.getCoreFactory());
        DADNASequence s8 = new DADNASequence(GGdb.getCoreFactory());
        s8.setId(-999);

        List<DADNASequence> test = new ArrayList<DADNASequence>();
        test.add(s4);
        test.add(s5);
        test.add(s6);
        test.add(s8);

        System.out.println("");
        System.out.println("before");
        for (DADNASequence s : test) {
            System.out.println(s.getId()+" : "+s.getClass().getSimpleName());
        }

        test = (List<DADNASequence>) GGdb.getCoreFactory().getSequenceDAO()
                .getValidatedSequences( test);
        
        System.out.println("");
        System.out.println("after");
        for (DADNASequence s : test) {
            System.out.println(s.getId()+" : "+s.getClass().getSimpleName());
        }

        System.out.println("");


        s6 = GGdb.getCoreFactory().getSequenceDAO().getValidatedSequence(s6);

        String n = (s6==null)? "yes":"no";

        System.out.println("is s6 null :" +n);


        DADNASequence s7 = new DADNASequence(GGdb.getCoreFactory());
        s7.setId(-999);
        s7 = GGdb.getCoreFactory().getSequenceDAO().getValidatedSequence(s7);


        n = (s7==null)? "yes":"no";

        System.out.println("is s7 null :" +n);


        DADNASequence a = new DADNASequence();

        System.out.println("a : "+a.getType());
        DADNASequence b = new DAAssembledDNASequence();

        System.out.println("b : "+b.getType());
        DADNASequence c = new DAChromosome();

        System.out.println("c : "+c.getType());


         DADNASequence d = new DADNASequence(GGdb.getCoreFactory());
         d.setId(102799);
         System.out.println("d type: "+d.getCoordSystem().getType());

        System.out.println("\n\n*****************************\n* COMPLETED FUNCTIONAL TEST *\n*****************************\n");


    }

}
