/**
 * Copyright (C) 2010 The Roslin Institute <contact andy.law@roslin.ed.ac.uk>
 *
 * This file is part of the Ensembl Java API demonstration project developed by the
 * Bioinformatics Group at The Roslin Institute, The Royal (Dick) School of
 * Veterinary Studies, University of Edinburgh.
 *
 * This is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License (version 3) as published by
 * the Free Software Foundation.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * in this software distribution. If not, see <http://www.gnu.org/licenses/gpl-3.0.html/>.
 */

package uk.ac.roslin.ensembl.test;


import org.apache.log4j.PropertyConfigurator;
import org.biojava3.core.sequence.Strand;
import uk.ac.roslin.ensembl.config.DBConnection.DataSource;
import uk.ac.roslin.ensembl.datasourceaware.core.DAChromosome;
import uk.ac.roslin.ensembl.datasourceaware.core.DADNASequence;

import uk.ac.roslin.ensembl.dao.database.DBRegistry;
import uk.ac.roslin.ensembl.dao.database.DBSingleSpeciesCoreDatabase;
import uk.ac.roslin.ensembl.dao.database.DBSpecies;
import uk.ac.roslin.ensembl.model.Coordinate;
import uk.ac.roslin.ensembl.model.Mapping;

public class TestEnsembl5 {

    public static void main(String[] args) throws Exception {

        PropertyConfigurator.configure("log4j.properties");


        DBRegistry ensembldbRegistry = new DBRegistry(DataSource.ENSEMBLDB);

        DBSpecies sp = ensembldbRegistry.getSpeciesByAlias("chicken");

        DAChromosome chr = sp.getChromosomeByName("25");

        System.out.println("chr type: " + chr.getType().toString());


        System.out.println("Chromosome:\n\tName: " + chr.getChromosomeName() + " db version-" + chr.getDBVersion());
        System.out.println("\tLength: " + chr.getDBSeqLength());
        System.out.println("\tcoord sys ID: " + chr.getCoordSystem().getId());
        System.out.println("\tseq level coord sys ID: " + ((DBSingleSpeciesCoreDatabase) chr.getDaoFactory().getDatabase()).getSequenceLevelCoordSystem().getId());
        System.out.println("\tseq region ID :" + chr.getId());
        System.out.println("\tseq type :" + chr.getType().toString());

        System.out.println("chromosome sequence 200000-210000\n"
                + chr.getSequenceAsString(200000, 210000, Strand.POSITIVE));
        System.out.println("chromosome rev complement sequence 200000-210000\n"
                + chr.getReverseComplementSequenceAsString(200000, 210000));
        System.out.println("");
        System.out.println("chromosome sequence 1-50\n"
                + chr.getSequenceAsString(1, 50, Strand.POSITIVE));
        System.out.println("chromosome rev complement sequence 1-50\n"
                + chr.getReverseComplementSequenceAsString(1, 50));
        System.out.println("");
        System.out.println("chromosome sequence 1750-1800\n"
                + chr.getSequenceAsString(1750, 1800, Strand.POSITIVE));
        System.out.println("chromosome ReverseComplement sequence 1750-1800\n"
                + chr.getReverseComplementSequenceAsString(1750, 1800));
        System.out.println("");
        System.out.println("chromosome sequence 142250, 142300\n"
                + chr.getSequenceAsString(142250, 142300));
        System.out.println("chromosome ReverseComplement sequence 142250, 142300\n"
                + chr.getReverseComplementSequenceAsString(142250, 142300));
        System.out.println("");
        System.out.println("chromosome sequence 142250, 142400\n"
                + chr.getSequenceAsString(142250, 142400));
        System.out.println("chromosome ReverseComplement sequence 142250, 142400\n"
                + chr.getReverseComplementSequenceAsString(142250, 142400));
        System.out.println("");


        for (Mapping mapping : chr.getCompleteAssembly().getStitchedMappings()) {
            System.out.println(((DADNASequence) mapping.getTarget()).getName()
                    + " : " + mapping.getTargetCoordinates().getStart()
                    + " - " + mapping.getTargetCoordinates().getEnd()
                    + " " + (mapping.getTargetCoordinates().getStrand() == Coordinate.Strand.REVERSE_STRAND ? "reverse" : "")
                    + " (source: " + mapping.getSourceCoordinates().getStart()
                    + " - " + mapping.getSourceCoordinates().getEnd()
                    + ")");
        }

        System.out.println("----------------------");
        System.out.println("----------------------");
        System.out.println("----------------------");
        System.out.println("----------------------");


for (Mapping mapping : chr.getCompleteAssembly().getStitchedMappings(1, 3000)) {
    System.out.println(((DADNASequence) mapping.getTarget()).getName()
            + " : " + mapping.getTargetCoordinates().getStart()
            + " - " + mapping.getTargetCoordinates().getEnd()
            + " " + (mapping.getTargetCoordinates().getStrand() == Coordinate.Strand.REVERSE_STRAND ? "reverse" : "")
            + " (source: " + mapping.getSourceCoordinates().getStart()
            + " - " + mapping.getSourceCoordinates().getEnd()
            + ")");

    System.out.println("mapped sequence length: " + ((DADNASequence) mapping.getTarget()).getDBSeqLength());
    System.out.println("mapped sequence start: " + ((DADNASequence) mapping.getTarget()).getBioBegin());
    System.out.println("mapped sequence end: " + ((DADNASequence) mapping.getTarget()).getBioEnd());
    System.out.println("mapped sequence fullseq length: " + ((DADNASequence) mapping.getTarget()).getSequenceAsString().length());

    if (mapping.getTargetCoordinates().getStrand() == Coordinate.Strand.FORWARD_STRAND) {

        System.out.println("mapped sequence desired (+) fragment:\n "
                + ((DADNASequence) mapping.getTarget()).getSequenceAsString(mapping.getTargetCoordinates().getStart(),
                mapping.getTargetCoordinates().getEnd()));
    } else {

        System.out.println("mapped sequence desired (-) fragment:\n "
                + ((DADNASequence) mapping.getTarget()).getReverseComplementSequenceAsString(mapping.getTargetCoordinates().getStart(),
                mapping.getTargetCoordinates().getEnd()));

    }
    System.out.println("----------------------");

}



        System.out.println(ensembldbRegistry.getSpeciesByAlias("human").getComparaName());
        System.out.println(ensembldbRegistry.getSpeciesByEnsemblName("homo_sapiens").getComparaName());

                System.out.println("\n\n*****************************\n* COMPLETED FUNCTIONAL TEST *\n*****************************\n");


    }
}
