/**
 * Copyright (C) 2010 The Roslin Institute <contact andy.law@roslin.ed.ac.uk>
 *
 * This file is part of the Ensembl Java API demonstration project developed by the
 * Bioinformatics Group at The Roslin Institute, The Royal (Dick) School of
 * Veterinary Studies, University of Edinburgh.
 *
 * This is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License (version 3) as published by
 * the Free Software Foundation.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * in this software distribution. If not, see <http://www.gnu.org/licenses/gpl-3.0.html/>.
 */

package uk.ac.roslin.ensembl.test;

import java.util.List;
import java.util.TreeSet;
import uk.ac.roslin.ensembl.config.DBConnection.DataSource;
import uk.ac.roslin.ensembl.datasourceaware.core.DAChromosome;
import uk.ac.roslin.ensembl.datasourceaware.core.DADNASequence;
import uk.ac.roslin.ensembl.dao.database.DBCollection;
import uk.ac.roslin.ensembl.dao.database.DBCollectionCoreDatabase;
import uk.ac.roslin.ensembl.dao.database.DBCollectionDatabase;
import uk.ac.roslin.ensembl.dao.database.DBCollectionSpecies;
import uk.ac.roslin.ensembl.model.database.CollectionDatabase;
import uk.ac.roslin.ensembl.model.core.CollectionOfSpecies;
import uk.ac.roslin.ensembl.model.core.CollectionSpecies;
import uk.ac.roslin.ensembl.model.database.Database;
import uk.ac.roslin.ensembl.dao.database.DBRegistry;
import uk.ac.roslin.ensembl.dao.database.DBSingleSpeciesCoreDatabase;
import uk.ac.roslin.ensembl.dao.database.factory.DBDAOSingleSpeciesCoreFactory;
import uk.ac.roslin.ensembl.model.Coordinate;
import uk.ac.roslin.ensembl.model.Mapping;
import uk.ac.roslin.ensembl.model.core.ProteinFeature;
import uk.ac.roslin.ensembl.model.database.SingleSpeciesCoreDatabase;
import uk.ac.roslin.ensembl.config.EnsemblDBType;
import uk.ac.roslin.ensembl.dao.database.DBSpecies;

public class TestGenomes {

    public  static void main(String[] args) throws Exception {

        DBRegistry ensemblgenomesRegistry = new DBRegistry(DataSource.ENSEMBLGENOMES);
        System.out.println(ensemblgenomesRegistry.getBriefRegistryReport());

        ensemblgenomesRegistry.getSpeciesByEnsemblName("oryza_sativa");

        ensemblgenomesRegistry.getDatabase("rice", EnsemblDBType.core, "7");

        System.out.println("ensemblgenomes registry report: ");
        System.out.println("******************************* ");


        System.out.println("all DBSpecies");
        for (DBSpecies s :ensemblgenomesRegistry.getSpecies()) {
            System.out.println(s.getCommonName());
        }
        System.out.println("all DBCollectionSpecies");
        for (DBCollectionSpecies s :ensemblgenomesRegistry.getCollectionSpecies()) {
            System.out.println(s.getCommonName());
        }




        for (DBCollection c : ensemblgenomesRegistry.getCollections()) {
System.out.println(c.getCollectionName()+" (v"+c.getDBVersion()+")");
    int i = 1;
             for (DBCollectionSpecies s: c.getSpecies()) {
                 System.out.println(i++ +":\t"+s.getComparaName());
             }
         }

      


        System.out.println(ensemblgenomesRegistry.printRegistryReport().toString());

  
        System.out.println("TESTING LIVE ENSEMBLGENOMES");

        System.out.println("**********************************\ngrape databases\n\n");

        for ( Database d : ensemblgenomesRegistry.getSpeciesByAlias("grape").getDatabases() ) {
            System.out.println("\t"+d.getdBName());
        }

        SingleSpeciesCoreDatabase db = (SingleSpeciesCoreDatabase) ensemblgenomesRegistry.getDatabase("grape");
        System.out.println("current grape top level: "+db.getTopLevelCoordSystem().getId());
        System.out.println("current grape chromosome level: "+db.getChromosomeLevelCoordSystem().getId());
        System.out.println("current grape sequence level: "+db.getSequenceLevelCoordSystem().getId());

        String grape = "grape"; // relies on alias look up


       int translationID = 21316;

        System.out.println("Get features from Ensemblgenomes release 7 for grape translation id ="+translationID);

        List<? extends ProteinFeature> result3 = ((DBSingleSpeciesCoreDatabase) ensemblgenomesRegistry
              .getDatabase(grape, EnsemblDBType.core, "7") )
                .getCoreFactory()
                    .getProteinFeatureDAO()
                        .getProteinFeaturesByTranslationID(translationID);


        System.out.println("retrieved: "+result3.size()+" features.");

        System.out.println("Get features from Ensemblgenomes release 4 for grape translation id ="+translationID);

        grape = "bob";
        try {
            ensemblgenomesRegistry.addSessionAlias("vitis_vinifera", grape);
            System.out.println("added alias bob ok");
        } catch (Exception exception) {
            System.out.println("failed to add alias bob for vitis_vinifera");
        }


        List<? extends ProteinFeature> result4 = ((DBSingleSpeciesCoreDatabase) ensemblgenomesRegistry
              .getDatabase(grape, EnsemblDBType.core, "7"))
                .getCoreFactory()
                    .getProteinFeatureDAO()
                        .getProteinFeaturesByTranslationID(translationID);


        System.out.println("retrieved: "+result4.size()+" features. Using the new alias.");


       CollectionOfSpecies reg  =  ensemblgenomesRegistry.getCollection("escherichia_shigella", ensemblgenomesRegistry.getMostRecentDBVersion());
      
        System.out.println("These are the species in the current escherichia_shigella collection");
        for (CollectionSpecies s : reg.getSpecies()) {
            System.out.println(s.getSpeciesBinomial()+" : "+ s.getCommonName());
        }

        System.out.println("the current collection databases are:");
        System.out.println("*************************************");
        for (CollectionOfSpecies r : ensemblgenomesRegistry.getCollectionsByDBVersion(ensemblgenomesRegistry.getMostRecentDBVersion())) {

            System.out.println("\tcollection name: "+r.getCollectionName());
            System.out.println("\tschema version: "+r.getSchemaVersion());
            System.out.println("\tdb version: "+r.getDBVersion());
            System.out.println("\t_________________________________\n Species included:\n");

            for (CollectionSpecies sp : r.getSpecies()) {
                System.out.println("\t\t"+sp.getSpeciesBinomial()+" : "+ sp.getCommonName());
            }

            System.out.println("\t_________________________________\n Databases:\n");
            System.out.println("\t\tDatabases");

            for (CollectionDatabase d : r.getDatabases()) {
                System.out.println("\t\t\t"+d.getdBName());
            }

            System.out.println("\t\t"+EnsemblDBType.collection_core);

            for (CollectionDatabase d : r.getDatabasesByType(EnsemblDBType.collection_core)) {
                System.out.println("\t\t\t"+d.getdBName());
            }
            System.out.println("\t\t"+EnsemblDBType.collection_funcgen);

            for (CollectionDatabase d : r.getDatabasesByType(EnsemblDBType.collection_funcgen)) {
                System.out.println("\t\t\t"+d.getdBName());
            }
            System.out.println("\t\t"+EnsemblDBType.collection_variation);

            for (CollectionDatabase d : r.getDatabasesByType(EnsemblDBType.collection_variation)) {
                System.out.println("\t\t\t"+d.getdBName());
            }

            System.out.println("");
        }


         DBDAOSingleSpeciesCoreFactory fact = ((DBSingleSpeciesCoreDatabase) ensemblgenomesRegistry.getDatabase("grape")).getCoreFactory();

         DAChromosome chr = fact.getChromosomeDAO().getChromosomeByName("15");

                 System.out.println("Chromosome:\n\tName: "+chr.getChromosomeName()+ " db version-"+chr.getDBVersion());
        System.out.println("\tLength: "+chr.getDBSeqLength());
        System.out.println("\tcoord sys ID: "+chr.getCoordSystem().getId());
        System.out.println("\tseq level coord sys ID: "+((DBSingleSpeciesCoreDatabase) chr.getDaoFactory().getDatabase()).getSequenceLevelCoordSystem().getId());
        System.out.println("\tseq region ID :"+chr.getId());
        System.out.println("\tseq type :"+chr.getType().toString());

          for (Mapping mapping: chr.getCompleteAssembly().getStitchedMappings()) {
        //for (Mapping mapping : chr.stitchComponents(1, 3000, chr.getWholeAssembly())) {
                    System.out.println( ((DADNASequence) mapping.getTarget()).getName()
                    +" : " + mapping.getTargetCoordinates().getStart()
                    +" - " + mapping.getTargetCoordinates().getEnd()
                    +" "+ ( mapping.getTargetCoordinates().getStrand()==Coordinate.Strand.REVERSE_STRAND ? "reverse" : "" )
                    +" (source: " + mapping.getSourceCoordinates().getStart()
                    +" - " + mapping.getSourceCoordinates().getEnd()
                    +")");
                    try {
                //System.out.println(((DASequenceOLD) mapping.getTarget()).getSequenceAsString());
            } catch (Exception e) {
            }
        }

         System.out.println("\n***************************************\n");

         for (Mapping mapping: chr.getCompleteAssembly().getStitchedMappings(1, 3000)) {
                    System.out.println( ((DADNASequence) mapping.getTarget()).getName()
                    +" : " + mapping.getTargetCoordinates().getStart()
                    +" - " + mapping.getTargetCoordinates().getEnd()
                    +" "+ ( mapping.getTargetCoordinates().getStrand()==Coordinate.Strand.REVERSE_STRAND ? "reverse" : "" )
                    +" (source: " + mapping.getSourceCoordinates().getStart()
                    +" - " + mapping.getSourceCoordinates().getEnd()
                    +")");
                    try {
                //System.out.println(((DASequenceOLD) mapping.getTarget()).getSequenceAsString());
            } catch (Exception e) {
            }
        }


         DBCollection collection =ensemblgenomesRegistry.getCollection("pyrococcus", ensemblgenomesRegistry.getMostRecentDBVersion() );

         for (DBCollectionSpecies sp : collection.getSpecies()) {

             System.out.println("species: \n\t"+sp.getDBSpeciesID(ensemblgenomesRegistry.getMostRecentDBVersion())+" : "+sp.getSpeciesBinomial()+ " ("+sp.getCommonName()+")");

             System.out.println("ChromosomeLevelCoordSystemID: "
                     +collection.getCoreDatabase().getChromosomeLevelCS(sp).getId()
                     +"\nSequenceLevelCoordSystemID: "
                     +collection.getCoreDatabase().getSequenceLevelCS(sp).getId()
                     +"\nTopLevelCoordSystemID: "
                     +collection.getCoreDatabase().getTopLevelCS(sp).getId());
             System.out.println("\n***************************\n");




         }

         DBCollectionSpecies pumi = (DBCollectionSpecies) ensemblgenomesRegistry.getSpeciesByAlias("Bacillus pumilus (strain SAFR-032)");
         
         System.out.println("DB for Bacillus pumilus SAFR-032");

         TreeSet<Integer> versions = new TreeSet<Integer>();

         for (DBCollectionDatabase d :pumi.getDatabasesByType(EnsemblDBType.collection_core)) {

             System.out.println("core version "+d.getDBVersion());
             versions.add(Integer.parseInt(d.getDBVersion()));

         }


         DAChromosome chrP = ((DBCollectionCoreDatabase) pumi.getDatabaseByTypeAndVersion(EnsemblDBType.collection_core, versions.last().toString()))
                 .getCoreFactory(pumi).getChromosomeDAO().getChromosomeByName("Chromosome");

         System.out.println(chrP.getSpecies().getSpeciesBinomial()+" : "+chrP.getId() +" : "+chrP.getChromosomeName());

        System.out.println("\n\n*****************************\n* COMPLETED FUNCTIONAL TEST *\n*****************************\n");

    }





}
