/**
 * Copyright (C) 2010 The Roslin Institute <contact andy.law@roslin.ed.ac.uk>
 *
 * This file is part of the Ensembl Java API demonstration project developed by the
 * Bioinformatics Group at The Roslin Institute, The Royal (Dick) School of
 * Veterinary Studies, University of Edinburgh.
 *
 * This is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License (version 3) as published by
 * the Free Software Foundation.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * in this software distribution. If not, see <http://www.gnu.org/licenses/gpl-3.0.html/>.
 */

package uk.ac.roslin.ensembl.test;

import java.util.List;

import uk.ac.roslin.ensembl.config.DBConnection.DataSource;
import uk.ac.roslin.ensembl.datasourceaware.core.DAChromosome;
import uk.ac.roslin.ensembl.datasourceaware.core.DAGene;
import uk.ac.roslin.ensembl.dao.database.DBCollectionSpecies;
import uk.ac.roslin.ensembl.dao.database.DBRegistry;
import uk.ac.roslin.ensembl.dao.database.DBCollection;

import uk.ac.roslin.ensembl.model.core.Species;
import uk.ac.roslin.ensembl.config.FeatureType;

public class TestGenomes3 {

    public  static void main(String[] args) throws Exception {

        DBRegistry ensemblgenomesRegistry = new DBRegistry(DataSource.ENSEMBLGENOMES);

        DBCollectionSpecies pab = (DBCollectionSpecies) ensemblgenomesRegistry.getSpeciesByAlias("Pyrococcus abyssi");




        DAChromosome chr = pab.getChromosomeByName("chromosome");


        System.out.println("Species Binomial: "+chr.getSpecies().getSpeciesBinomial());
        System.out.println("Schema Version: "+chr.getSchemaVersion());
        System.out.println("DB Version: "+chr.getDBVersion());
        System.out.println("Chromosome Name: "+chr.getChromosomeName());
        System.out.println("Chromosome Id: "+chr.getId());

        System.out.println("Chromosome Begin: "+chr.getBioBegin());
        System.out.println("Chromosome Length: "+chr.getLength());
        System.out.println("Chromosome DBLength: "+chr.getDBSeqLength());
        System.out.println("Chromosome End: "+chr.getBioEnd());
        System.out.println("Chromosome CoordSystem ID: "+chr.getCoordSystem().getId());
        System.out.println("Chromosome CoordSystem Type: "+chr.getCoordSystem().getType().toString());

        System.out.println("chr.getSequenceAsString(1, 100)\n"+chr.getSequenceAsString(1, 100));
        System.out.println("chr.getReverseComplementSequenceAsString(1, 100)\n"+chr.getReverseComplementSequenceAsString(1, 100));



        System.out.print("schema versions for this species: ");
        for(String s: pab.getSchemaVersions()) {
            System.out.print(s+" ");
        }
        System.out.print("db versions for this species: ");
        for(String s: pab.getDBVersions()) {
            System.out.print(s+" ");
        }

        try{
        DAChromosome chr56 = pab.getChromosomeByName("chromosome", "6");

        System.out.println("Schema Version: "+chr56.getSchemaVersion());
        System.out.println("DB Version: "+chr56.getDBVersion());
        System.out.println("Chromosome Name: "+chr56.getChromosomeName());
        System.out.println("Chromosome Id: "+chr56.getId());
        System.out.println("Chromosome Length: "+chr56.getLength());
        System.out.println("Chromosome CoordSystem ID: "+chr56.getCoordSystem().getId());
        System.out.println("Chromosome CoordSystem Type: "+chr56.getCoordSystem().getType().toString());

        System.out.println("chr.getSequenceAsString(1, 100)\n"+chr56.getSequenceAsString(1, 100));
        System.out.println("chr.getReverseComplementSequenceAsString(1, 100)\n"+chr56.getReverseComplementSequenceAsString(1, 100));
        } catch (Exception e) {
            System.out.println("");
            System.out.println("");
            System.out.println("");
            System.out.println(e.getMessage());
            System.out.println("");
            System.out.println("");
            System.out.println("");
        }


//       System.out.println("Features\n-------------");
//
//        for (FeatureType t : FeatureType.getAllTypes()) {
//
//
//            Set <DACoordinateSystem> s = (pab.getCurrentCoreDatabase()).getCSForFeature(pab, t);
//
//            if (s!=null && !s.isEmpty()) {
//            System.out.println(t.toString());
//            for ( DACoordinateSystem cs : s) {
//                System.out.println("cs-id: "+cs.getId()+" - "+"cs-type: "+cs.getType().toString());
//            }
//            }
//        }
//
//
//         DAGene gene = f.getGeneDAO().getGeneByID(3555);
//
//         System.out.println("gene: "+gene.getId()+" ("+gene.getStableID()+") "+gene.getBiotype());
//         System.out.println("\t"+gene.getDescription());
//         System.out.println("Species: "+gene.getSpecies().getCommonName());
//         System.out.println("Created "+gene.getCreationDate().toString());
//         System.out.println("Modified "+gene.getModificationDate().toString());
//
//         for (Mapping m : gene.getMappings()) {
//             System.out.println("HASHES "+m.getSourceHashID()+" "+m.getTargetHashID());
//             System.out.println("Mapping: "+m.getSource().getClass().getSimpleName() + " id: "+ m.getSource().getId() );
//             System.out.println("\tTarget: "+m.getTarget().getClass().getSimpleName() +" id: "+m.getTarget().getId()
//                     + "type: "+m.getTarget().getType() );
//             System.out.println("\tTargetCoords: "+m.getTargetCoordinates().getStart()
//                     +" - "+m.getTargetCoordinates().getEnd() + "["+m.getTargetCoordinates().getStrand() +"]");
//
//         }
//
//         gene = f.getGeneDAO().addGeneByIDOnParent(3555, chr);
//
//         System.out.println("gene: "+gene.getId()+" ("+gene.getStableID()+") "+gene.getBiotype());
//         System.out.println("\t"+gene.getDescription());
//         System.out.println("Species: "+gene.getSpecies().getCommonName());
//         System.out.println("Created "+gene.getCreationDate().toString());
//         System.out.println("Modified "+gene.getModificationDate().toString());
//         for (Mapping m : gene.getMappings()) {
//             System.out.println("HASHES "+m.getSourceHashID()+" "+m.getTargetHashID());
//            System.out.println("Mapping source: "+m.getSource().getId());
//             System.out.println("\tTarget: "+m.getTarget().getClass().getSimpleName() +" id: "+m.getTarget().getId()
//                     + "type: "+m.getTarget().getType() );
//            System.out.println("target coords:"+m.getTargetCoordinates().getStart()+"-"
//                     +m.getTargetCoordinates().getEnd() +"("
//                     +m.getTargetCoordinates().getStrand().toString() +")");
//             System.out.println("CS: "+((DADNASequence) m.getTarget()).getCoordSystem().getType().toString());
//         }
//
//         gene = f.getGeneDAO().getGeneByStableID("EBPYRG00000003245");
//         System.out.println("gene: "+gene.getId()+" ("+gene.getStableID()+") "+gene.getBiotype());
//         System.out.println("\t"+gene.getDescription());
//         System.out.println("Species: "+gene.getSpecies().getCommonName());
//         System.out.println("Created "+gene.getCreationDate().toString());
//         System.out.println("Modified "+gene.getModificationDate().toString());
//         for (Mapping m : gene.getMappings()) {
//             System.out.println("HASHES "+m.getSourceHashID()+" "+m.getTargetHashID());
//            System.out.println("Mapping source: "+m.getSource().getId());
//             System.out.println("\tTarget: "+m.getTarget().getClass().getSimpleName() +" id: "+m.getTarget().getId()
//                     + "type: "+m.getTarget().getType() );
//            System.out.println("target coords:"+m.getTargetCoordinates().getStart()+"-"
//                     +m.getTargetCoordinates().getEnd() +"("
//                     +m.getTargetCoordinates().getStrand().toString() +")");
//             System.out.println("CS: "+((DADNASequence) m.getTarget()).getCoordSystem().getType().toString());
//         }
//
//                  gene = f.getGeneDAO().addGeneByStableIDOnParent("EBPYRG00000003245", chr);
//
//         System.out.println("gene: "+gene.getId()+" ("+gene.getStableID()+") "+gene.getBiotype());
//         System.out.println("\t"+gene.getDescription());
//         System.out.println("Species: "+gene.getSpecies().getCommonName());
//         System.out.println("Created "+gene.getCreationDate().toString());
//         System.out.println("Modified "+gene.getModificationDate().toString());
//         for (Mapping m : gene.getMappings()) {
//             System.out.println("HASHES "+m.getSourceHashID()+" "+m.getTargetHashID());
//            System.out.println("Mapping source: "+m.getSource().getId());
//             System.out.println("\tTarget: "+m.getTarget().getClass().getSimpleName() +" id: "+m.getTarget().getId()
//                     + "type: "+m.getTarget().getType() );
//            System.out.println("target coords:"+m.getTargetCoordinates().getStart()+"-"
//                     +m.getTargetCoordinates().getEnd() +"("
//                     +m.getTargetCoordinates().getStrand().toString() +")");
//             System.out.println("CS: "+((DADNASequence) m.getTarget()).getCoordSystem().getType().toString());
//         }
//
//          Coordinate coord = new Coordinate();
//
//         coord.setStart(1);
//         coord.setEnd(1000000);
//         //coord.setStrandInt(-1);
//         //coord.setStrandInt(1);
//         //coord.setStrandInt(null);
//
//         List<DAGene> genes = GGdb.getFactory().getGeneDAO().getGeneMappingsOnRegion(chr,coord);
//         List<? extends Gene> genes = chr.getGeneMappingsOnRegion(coord);
         List<DAGene> genes = chr.getGenesOnRegion(1, 10000000);
//         List<? extends Gene> genes = chr.getGeneMappingsOnRegion(1, 1000000, null);
//         List<? extends Gene> genes = chr.getGeneMappingsOnRegion(1, 1000000, Strand.FORWARD_STRAND);
//         List<? extends Gene> genes = chr.getGenesOnRegion(1, 500000, uk.ac.roslin.ensembl.model.Coordinate.Strand.REVERSE_STRAND);
         System.out.println(genes.size()+ " returned  genes");
         System.out.println("total gene mappings: "+chr.getMappings(FeatureType.gene).size());

         System.out.println("First Gene: "+genes.get(0).getStableID());
         System.out.println("Coordinates: "
                 +genes.get(0).getMappings().first().getTargetCoordinates().getStart()
                 +" - "
                 +genes.get(0).getMappings().first().getTargetCoordinates().getEnd());

//         genes = chr.getGenesOnRegion(1,1000000, uk.ac.roslin.ensembl.model.Coordinate.Strand.REVERSE_STRAND);
//         System.out.println(genes.size()+ " returned genes");
//         System.out.println("total gene mappings: "+chr.getMappings(FeatureType.gene).size());
//
//         genes = chr.getGenesOnRegion(500000,1000000, uk.ac.roslin.ensembl.model.Coordinate.Strand.REVERSE_STRAND);
//         System.out.println(genes.size()+ " returned genes");
//         System.out.println("total gene mappings: "+chr.getMappings(FeatureType.gene).size());
//
//         genes = chr.getGenesOnRegion(1010000, 1020000, uk.ac.roslin.ensembl.model.Coordinate.Strand.REVERSE_STRAND);
//         System.out.println(genes.size()+ " returned genes");
//         System.out.println("total gene mappings: "+chr.getMappings(FeatureType.gene).size());
//
//         genes = chr.getGenesOnRegion(1030000, 1040000, uk.ac.roslin.ensembl.model.Coordinate.Strand.REVERSE_STRAND);
//         System.out.println(genes.size()+ " returned genes");
//         System.out.println("total gene mappings: "+chr.getMappings(FeatureType.gene).size());
//
//         genes = chr.getGenesOnRegion(1050000, 1060000, uk.ac.roslin.ensembl.model.Coordinate.Strand.REVERSE_STRAND);
//         System.out.println(genes.size()+ " returned genes");
//         System.out.println("total gene mappings: "+chr.getMappings(FeatureType.gene).size());
//
//
//         for (Mapping m : chr.getMappings(FeatureType.gene)) {
//
//             System.out.println( ((DAGene) m.getTarget()).getStableID()+" "+m.getSourceHashID()+" "+m.getTargetHashID());
//
//         }
//
//
//         System.out.println("gaps in the gene mappings: ");
////         for (Coordinate cd : Coordinate.getCoordinateGaps(chr.getMappedRegions().get(FeatureType.gene))) {
//         for (Coordinate cd : chr.getMappedRegions().get(FeatureType.gene).getGaps()) {
//             System.out.println("gap "+cd.getStart()+" - "+cd.getEnd());
//         }
//         System.out.println("gaps in the gene mappings for the whole chromosome: ");
//         Coordinate cx = new Coordinate(chr.getBioBegin(), chr.getBioEnd(),1);
////         for (Coordinate cd : cx.getGaps(chr.getMappedRegions().get(FeatureType.gene))) {
//         for (Coordinate cd : chr.getMappedRegions().get(FeatureType.gene).getUncoveredRegions(cx)) {
//             System.out.println("gap "+cd.getStart()+" - "+cd.getEnd());
//         }
//
//         Coordinate test = new Coordinate(900000,1200000,1);
//
//         System.out.println("testing 900000-1200000 query");
////         System.out.println("test lies within known region: "+test.liesWithinCoordinateSetWithoutGaps(chr.getMappedRegions().get(FeatureType.gene)));
//         System.out.println("test lies within known region: "+chr.getMappedRegions().get(FeatureType.gene).containsCoordinateWithoutGaps(test));
//
////         if (test.getGaps(chr.getMappedRegions().get(FeatureType.gene)).isEmpty()) {
//         if (chr.getMappedRegions().get(FeatureType.gene).getUncoveredRegions(test).isEmpty()) {
//
//             System.out.println("test region is fully covered");
//
//         } else {
//             System.out.println("test region is not fully covered");
// //            for (Coordinate cd : test.getGaps(chr.getMappedRegions().get(FeatureType.gene))) {
//             for (Coordinate cd : chr.getMappedRegions().get(FeatureType.gene).getUncoveredRegions(test)) {
//                System.out.println("gap "+cd.getStart()+" - "+cd.getEnd());
//             }
//         }
//
////         System.out.println("test has regions not covered: "+test.getGaps(chr.getMappedRegions().get(FeatureType.gene)).size());
//         System.out.println("test has regions not covered: "+chr.getMappedRegions().get(FeatureType.gene).getUncoveredRegions(test).size());
//
//         test = new Coordinate(1,900000,1);
//
//         System.out.println("testing 1-900000 query");
//
////         System.out.println("test lies within known region: "+test.liesWithinCoordinateSetWithoutGaps(chr.getMappedRegions().get(FeatureType.gene)));
//         System.out.println("test lies within known region: "+chr.getMappedRegions().get(FeatureType.gene).containsCoordinateWithoutGaps(test));
//
////         if (test.getGaps(chr.getMappedRegions().get(FeatureType.gene)).isEmpty()) {
//         if (chr.getMappedRegions().get(FeatureType.gene).getUncoveredRegions(test).isEmpty()) {
//             System.out.println("test region is fully covered");
//         } else {
//             System.out.println("test region is not fully covered");
//             for (Coordinate cd : chr.getMappedRegions().get(FeatureType.gene).getUncoveredRegions(test)) {
//                System.out.println("gap "+cd.getStart()+" - "+cd.getEnd());
//             }
//         }
//
// //        System.out.println("test has regions not covered: "+test.getGaps(chr.getMappedRegions().get(FeatureType.gene)).size());
//         System.out.println("test has regions not covered: "+chr.getMappedRegions().get(FeatureType.gene).getUncoveredRegions(test).size());
//
//
//         genes = chr.getGenesOnRegion(1, 10000, uk.ac.roslin.ensembl.model.Coordinate.Strand.REVERSE_STRAND);
//
//         System.out.println(genes.size()+ " genes");
//


 for (DBCollection c : ensemblgenomesRegistry.getCollections()) {
     System.out.println(c.getCollectionName()+" (v"+c.getDBVersion()+")");
     for (DBCollectionSpecies s: c.getSpecies()) {
         System.out.println("\t"+s.getComparaName());
     }
 }

 for (Species s: ensemblgenomesRegistry.getSpecies()) {
     System.out.println(s.getComparaName());
 }

 for (Species s: ensemblgenomesRegistry.getCollectionSpecies()) {
     System.out.println(s.getComparaName());
 }

        System.out.println(ensemblgenomesRegistry.getSpeciesByAlias("Pyrococcus abyssi (strain Orsay / GE5)").getComparaName());
        System.out.println(ensemblgenomesRegistry.getSpeciesByEnsemblName("p_abyssi").getComparaName());
        System.out.println(ensemblgenomesRegistry.getCollectionRegistriesByName("pyrococcus"));
        System.out.println(ensemblgenomesRegistry.getCollection("pyrococcus", "7"));



       System.out.println("\n\n*****************************\n* COMPLETED FUNCTIONAL TEST *\n*****************************\n");


    }





}
